/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */


#include "ClientPushManager.h"
#include "pim/ClientSettings.h"
#include "pim/SettingFunctions.h"
#include "http/GPRSConnection.h"
#include "pim/Utils.h"
#include "pim/difftime.h"

#include <Notify.h>

#include <string>
#include <map>
using namespace std;

#define CHECK_INTERNET_CONNECTION_TIME          300  // check GPRS every CHECK_TIME seconds (5min)
static HANDLE IConnectionThread = 0;

// Init static pointer.
ClientPushManager* ClientPushManager::pinstance = NULL;

DWORD WINAPI initClientPush(LPVOID moduleHandle);
LRESULT CALLBACK wndMainProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK wndMainProcNote(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK wndMainProcBriefcase(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK wndMainProcPicture(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);

#define _ErrorLabel Error

#define CHR(hResult) \
    if(FAILED(hResult)) { hr = (hResult); goto _ErrorLabel;}

#define CBR(fBool) \
    if(!(fBool)) { hr = (E_FAIL); goto _ErrorLabel;}

#define RELEASE_OBJ(s)  \
    if (s != NULL)      \
    {                   \
        s->Release();   \
        s = NULL;       \
    }

/**
 * Method to create the sole instance of CTPManager
 */
ClientPushManager* ClientPushManager::getInstance() {

    if (pinstance == NULL) {
        pinstance = new ClientPushManager;
    }
    return pinstance;
}

ClientPushManager::ClientPushManager() {

    cp_hInstanc     = NULL;
    cp_hWnd         = NULL;    
    g_polApp        = NULL;
    g_pCol          = NULL;
    cpThread        = NULL;
    dmt             = NULL;
    node            = NULL;
    appName         = TEXT("");
    pushInterval    = 1; // by default 1 min

    syncingContact = false;
    syncingCalendar = false;
    syncingTask = false;
    syncingNote = false;
    syncingBriefcase = false;
    syncingPicture      = false;
}

/**
 * Denstructor: stop any active thread, and close socket
 * connection if still active.
 */
ClientPushManager::~ClientPushManager() {
    
    cp_hInstanc     = NULL;
    cp_hWnd         = NULL;    
    g_polApp        = NULL;
    g_pCol          = NULL;
    cpThread        = NULL;
    if (dmt)  {delete dmt;  dmt  = NULL; }
    if (node) {delete node; node = NULL; }    
    
}

HANDLE ClientPushManager::startCP(HANDLE moduleHandle) {

    if (cpThread) {
        stopThread(cpThread);
    }    
    
    initClientPush((LPVOID)moduleHandle);
    return cpThread;
}


int ClientPushManager::stopCP() {

    if (!cpThread) {
        LOG.debug("No CP thread available -> exiting.");
        return 1;
    }   
    int ret = 0;
    LOG.info("Closing CP thread...");   

    if (stopThread(cpThread)) {
        LOG.debug("cpThread killed");
    }
    return ret;
}

bool ClientPushManager::stopThread(HANDLE thread, DWORD exitcode) {

    if (thread) {
        TerminateThread(thread, exitcode);
        CloseHandle(thread);
        thread = NULL;
        return true;
    }
    return false;
}


bool ClientPushManager::subscribeContact() {

    HRESULT hr = subscribeToNotifications(g_polApp, olContactItem, PIMFOLDERNOTIFICATION_REMOTE);
    return ((hr == S_OK) ? true : false);
}

bool ClientPushManager::subscribeCalendar() {

    HRESULT hr = subscribeToNotifications(g_polApp, olAppointmentItem, PIMFOLDERNOTIFICATION_REMOTE);
    return ((hr == S_OK) ? true : false);
}

bool ClientPushManager::subscribeTask() {

    HRESULT hr = subscribeToNotifications(g_polApp, olTaskItem, PIMFOLDERNOTIFICATION_REMOTE);
    return ((hr == S_OK) ? true : false);
}

bool ClientPushManager::subscribeBriefcase() {
    getRegConfig()->read();
    WindowsSyncSourceConfig* ssconf = getRegConfig()->getWinSyncSourceConfig(FILES_NAME_A);
    string dir = ssconf->getSyncDir();
    wchar_t* dirw = toWideChar(dir.c_str());
    
    BOOL ret = FALSE;
    SHCHANGENOTIFYENTRY notifyEntry;
    notifyEntry.dwEventMask = SHCNE_CREATE | SHCNE_DELETE | 
                                     SHCNE_RENAMEITEM;   // renamed existing item. Used also for update
    
    notifyEntry.pszWatchDir = dirw; //TEXT("\\My Documents\\briefcase");
    notifyEntry.fRecursive = FALSE;
    
    if (ret == SHChangeNotifyRegister(cp_hWndBriefcase, &notifyEntry)) {    
        LOG.error("Problem notify folder");
    }    
    delete [] dirw;
    return (ret == TRUE) ? true : false;

}

bool ClientPushManager::subscribeNotes() {
    getRegConfig()->read();
    WindowsSyncSourceConfig* ssconf = getRegConfig()->getWinSyncSourceConfig(NOTES_NAME_A);
    string dir = ssconf->getSyncDir();
    wchar_t* dirw = toWideChar(dir.c_str());
    
    BOOL ret = FALSE;
    SHCHANGENOTIFYENTRY notifyEntry;
    notifyEntry.dwEventMask = SHCNE_CREATE | SHCNE_DELETE | 
                                     SHCNE_RENAMEITEM;   // renamed existing item. Used also for update
    
    notifyEntry.pszWatchDir = dirw; //TEXT("\\My Documents\\briefcase");
    notifyEntry.fRecursive = FALSE;
    
    if (ret == SHChangeNotifyRegister(cp_hWndNote, &notifyEntry)) {    
        LOG.error("Problem notify folder");
    }   
    delete [] dirw;
    return (ret == TRUE) ? true : false;

}

bool ClientPushManager::subscribePicture() {
    getRegConfig()->read();
    WindowsSyncSourceConfig* ssconf = getRegConfig()->getWinSyncSourceConfig(SOURCE_NAME_PICTURE);
    string dir = ssconf->getSyncDir();
    wchar_t* dirw = toWideChar(dir.c_str());
    
    BOOL ret = FALSE;
    SHCHANGENOTIFYENTRY notifyEntry;
    notifyEntry.dwEventMask = SHCNE_CREATE | SHCNE_DELETE | 
                                     SHCNE_RENAMEITEM;   // renamed existing item. Used also for update
    
    notifyEntry.pszWatchDir = dirw; //TEXT("\\My Documents\\briefcase");
    notifyEntry.fRecursive = FALSE;
    
    if (ret == SHChangeNotifyRegister(cp_hWndPicture, &notifyEntry)) {    
        LOG.error("Problem notify folder");
    }    
    delete [] dirw;
    return (ret == TRUE) ? true : false;

}

bool ClientPushManager::unsubscribeContact() {

    HRESULT hr = subscribeToNotifications(g_polApp, olContactItem, 0);
    return ((hr == S_OK) ? true : false);
}

bool ClientPushManager::unsubscribeCalendar() {

    HRESULT hr = subscribeToNotifications(g_polApp, olAppointmentItem, 0);
    return ((hr == S_OK) ? true : false);
}

bool ClientPushManager::unsubscribeTask() {

    HRESULT hr = subscribeToNotifications(g_polApp, olTaskItem, 0);
    return ((hr == S_OK) ? true : false);
}

bool ClientPushManager::unsubscribeBriefcase() {
    
    BOOL ret = FALSE;
    if (ret == SHChangeNotifyDeregister(cp_hWndBriefcase)) {    
        LOG.error("Problem unregister listener folder briefcase");
    }    
    return (ret == TRUE) ? true : false;
}

bool ClientPushManager::unsubscribeNotes() {
    
    BOOL ret = FALSE;
    if (ret == SHChangeNotifyDeregister(cp_hWndNote)) {    
        LOG.error("Problem unregister listener folder notes");
    }    
    return (ret == TRUE) ? true : false;
}

bool ClientPushManager::unsubscribePicture() {
    
    BOOL ret = FALSE;
    if (ret == SHChangeNotifyDeregister(cp_hWndPicture)) {    
        LOG.error("Problem unregister listener folder picture");
    }    
    return (ret == TRUE) ? true : false;
}

int ClientPushManager::suspendCP(int value) {
    
    if (value & 2) {
        unsubscribeContact();
    }
    if (value & 4) {
        unsubscribeCalendar();
    }
    if (value & 8) {
        unsubscribeTask();
    }
    if (value & 16) {
        unsubscribeNotes();
    } 
    if (value & 32) {
        unsubscribeBriefcase();
    }
    if (value & 64) {
        unsubscribePicture();
    }
    return 0;
}

int ClientPushManager::resumeCP(int value) {
    
    if (value & 2) {
        subscribeContact();
    }
    if (value & 4) {
        subscribeCalendar();
    }
    if (value & 8) {
        subscribeTask();
    }
    if (value & 16) {
        subscribeNotes();
    }
    if (value & 32) {
        subscribeBriefcase();
    }
    if (value & 64) {
        subscribePicture();
    }
    return 0;
}

DWORD WINAPI initClientPush(LPVOID moduleHandle) {
    
    MSG         msg;
    int         rc = 0;
    HRESULT     hr = S_OK;

    SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_BELOW_NORMAL);    
    
    ClientPushManager* cpManager = ClientPushManager::getInstance();
    try {
        // initialize the tree  
        cpManager->dmt = DMTreeFactory::getDMTree(APPLICATION_URI);
        if (!cpManager->dmt)  { LOG.error("CP: dmt is wrong!!!"); }
        cpManager->node = cpManager->dmt->readManagementNode(APPLICATION_URI);
        if (!cpManager->node) { LOG.error("CP: node is wrong!!!");}
        
        cpManager->initCommonVar();

        // Initialize application
        rc = cpManager->createClientPushClasses(cpManager, (HANDLE)moduleHandle);
        if (rc < 0) {
            LOG.error("CP: Error in creating the push class. Exit...");        
            return rc;
        }
        LOG.info("CP: Client push class created");        
        // Init POOM

        LOG.debug("Init Poom");
        hr = cpManager->InitPOOM();
        CHR(hr);
        
        LOG.debug("CP: Log on the IPOutlookApp2 interface");    
        hr = cpManager->g_polApp->Logon((long)cpManager->cp_hWnd); 
        CHR(hr);
        LOG.debug("CP: Subscription of Contact folder...");
        cpManager->subscribeContact();
        if (hr == S_OK) {
            LOG.info("CP: Contact folder subscribed.");
        }
        LOG.debug("CP: Subscription of Calendar folder...");
        cpManager->subscribeCalendar();
        if (hr == S_OK) {
            LOG.info("CP: Calendar folder subscribed.");
        }
        LOG.info("CP: Subscription of Task folder...");
        cpManager->subscribeTask();
        if (hr == S_OK) {
            LOG.info("CP: Task folder subscribed.");
        }         
        
        // new for brifcase
        LOG.info("CP: Subscription of Briefcase folder...");
        cpManager->subscribeBriefcase();
        if (hr == S_OK) {
            LOG.info("CP: Briefcase folder subscribed.");
        } 

        // new for notes
        LOG.info("CP: Subscription of Notes folder...");
        cpManager->subscribeNotes();
        if (hr == S_OK) {
            LOG.info("CP: Notes folder subscribed.");
        } 

        // new for picture
        LOG.info("CP: Subscription of Picture folder...");
        cpManager->subscribePicture();
        if (hr == S_OK) {
            LOG.info("CP: Piacture folder subscribed.");
        } 

        LOG.info("CP: Init waiting PIM notification messages...");
        // Main message loop
        while (GetMessage(&msg, NULL, 0, 0)) 
        {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
    } catch(...) {
        LOG.error("CP: Generic error in client Push: Error = 0x%x", GetLastError());
    }

Error:
       
    // UnInit POOM
    cpManager->UnInitPOOM();      
    UnregisterClass(cpManager->cp_wncClass.lpszClassName, cpManager->cp_wncClass.hInstance);    
    
    return 0;
}

/**
* It creates the classes for the different sources to listen to.
* They are PIM, briefcase and notes. If the briefcase and notes are on the
* same directory, only briefcase is created
*/
int ClientPushManager::createClientPushClasses (ClientPushManager* cpManager, HANDLE module) {
    
    int ret = 0;
    
    // class for pim 
    ret = createClientPushClass(&(cpManager->cp_wncClass), wndMainProc, pimClassName, module, 
                                (cpManager->cp_hWnd));
    
    // briefcase
    ret = createClientPushClass(&(cpManager->cp_wncClassBriefcase), wndMainProcBriefcase, briefcaseClassName, module,
        (cpManager->cp_hWndBriefcase));
    
    // note
    ret = createClientPushClass(&(cpManager->cp_wncClassNote), wndMainProcNote, notesClassName, module,
        (cpManager->cp_hWndNote));
    
    // picture
    ret = createClientPushClass(&(cpManager->cp_wncClassPicture), wndMainProcPicture, 
                                pictureClassName, module,
        (cpManager->cp_hWndPicture));
    
    return ret;
}


///////////////////////////////////////////////////////////////////////////////
// Function:
//      InitApp
//
// Purpose:
//      Checks if the app dialog class already exists. If so, it means the app 
//      is already running so we quit this instance.
//
// Parameters:
//      hInstance       - Handle to current app instance
//      lpWndClass      - pointer to WNDCLASS struct which is filled with app settings.
//
// Returns
//      0   - Success.
//      -1  - Failure.
//
int ClientPushManager::createClientPushClass (WNDCLASS *lpWndClass, 
                                              WNDPROC wndMainProc,
                                              const wchar_t* g_szClassName,
                                              HANDLE module, HWND& hWnd) {
    int ret = 0;           

    lpWndClass->style                = 0;
    lpWndClass->lpfnWndProc          = (WNDPROC) wndMainProc;
    lpWndClass->cbClsExtra           = 0;
    lpWndClass->cbWndExtra           = 0;
    lpWndClass->hInstance            = (HINSTANCE)module;
    lpWndClass->hIcon                = NULL;
    lpWndClass->hCursor              = 0;
    lpWndClass->hbrBackground        = (HBRUSH) GetStockObject(WHITE_BRUSH);
    lpWndClass->lpszMenuName         = NULL;
    lpWndClass->lpszClassName        = g_szClassName;
   
    DWORD dwError = 0;
    SetLastError(0);  
    if (RegisterClass(lpWndClass) == 0) {
        
        dwError = GetLastError();
        if (dwError == ERROR_CLASS_ALREADY_EXISTS) {
            LOG.error("Failed to registed class. Error = 0x%x", dwError);
            ret = 1;            
            goto Error;
        }
        ret = -1; // generic error        
    } else { // success
        dwError = GetLastError();        
        
        hWnd = CreateWindow(
                        g_szClassName,
                        NULL,
                        WS_BORDER,
                        0,0,
                        0,0,
                        NULL,
                        NULL,
                        (HINSTANCE)module,
                        NULL);

    }      

Error:    
      
    return ret;
}

/**
* It returns the constants that indicates which type is this file item.
* If the name ends with .pwi, it is a note.
* Otherwise it is a briefcase
*/
int getFileItemType(const wchar_t* fileName) {
    int len = wcslen(fileName);
    int ret = 0;
    if ((len > 4) && (fileName[len-1] == 'i' && fileName[len-2] == 'w'
        && fileName[len-3] == 'p' && fileName[len-4] == '.')) {
            return constNotes;
    }
    return ret;

}

bool isNote(const wchar_t* fileName) {
    int ret = getFileItemType(fileName);
    return (ret == constNotes) ? true : false;

}

LRESULT CALLBACK wndMainProcNote(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam) {
    HRESULT     hr  = S_OK;    
    static HWND hWndStats;
    WORD        wHi = HIWORD(wParam), 
                wLo = LOWORD(wParam);
    ClientPushManager* cpManager = ClientPushManager::getInstance();
    
    FILECHANGENOTIFY    *lpfcn;
    FILECHANGEINFO      *lpfci;    
    DWORD               dwAttrs;
    WCHAR               fileBuffer[MAX_PATH];    
    static list<wstring> fileListTemporaryNote;  
    CBR(hwnd);

    switch (message) 
    {
        case WM_COMMAND:
            LOG.info("CP note: command change dir");
            cpManager->unsubscribeNotes();
            cpManager->subscribeNotes();
            break;
                    
        case WM_CLOSE:    
        case WM_DESTROY:
            PostQuitMessage(0);
            break;              
       
        case WM_FILECHANGEINFO:
            // see if the pointer to the file change notify struct is valid
            lpfcn = (FILECHANGENOTIFY *)lParam;
            if (NULL == lpfcn) {
                LOG.error("CP note: error notify briefcase change");
                break;
            }
            // see if the pointer to the file change info structure
            lpfci = &(lpfcn->fci);
            hr = StringCchPrintf(fileBuffer, 
                    (sizeof(fileBuffer)/sizeof(*fileBuffer)), 
                    TEXT("%s"), 
                    lpfci->dwItem1);
            if (hr != S_OK) {
                LOG.error("CP note:Insufficient buffer to get the file name");
                break;
            }
            if (!isNote(fileBuffer)) {
                LOG.info("CP note:  The %S file is not a note: break", fileBuffer);
                break;
            }
            switch (lpfci->wEventId) { 
                case SHCNE_CREATE:
                {                    
                    LOG.info("CP note: Creating Item...");                    
                    dwAttrs = GetFileAttributes(fileBuffer);
                    if (dwAttrs == INVALID_FILE_ATTRIBUTES) {
                        LOG.info("CP note: The file is a temporary file. It must not be signaled"); 
                        fileListTemporaryNote.push_back(fileBuffer);
                    } 
                    else {
                        LOG.info("CP note: It is a new item inserted");
                        cpManager->setPIMModifications(constNotes);
                    }
                }
                break;                

                case SHCNE_RENAMEITEM: 
                {
                    LOG.info("CP note: Renaming Item...");
                    cpManager->setPIMModifications(constNotes);     
 
                    bool found = false;                                        
                    list<wstring>::iterator it;
                    for (it = fileListTemporaryNote.begin(); it != fileListTemporaryNote.end(); it++) {                        
                           if (*it == fileBuffer) {
                               fileListTemporaryNote.erase(it);
                               LOG.info("CP note: It is a renaming of a temp file so a modification of original");
                               found = true;
                               break;
                           }
                    }
                    if (!found) {
                        LOG.info("CP note: It is a renaming...");
                    }                                         
                }
                break;                    
                
                case SHCNE_DELETE: 
                {
                    LOG.info("CP note:Deleting Item...");
                    if (fileListTemporaryNote.empty()) {
                        LOG.info("The item is really deleting...");
                        cpManager->setPIMModifications(constNotes);
                    }                                        
                }
                break;                    
            }
            if (NULL == lpfci) {
                break;
            } 
            SHChangeNotifyFree(lpfcn);
            break;            
                

        default:
            return DefWindowProc(hwnd, message, wParam, lParam);
    }

Error:
   
    return FALSE;
}


LRESULT CALLBACK wndMainProcBriefcase(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam) 
{
    HRESULT     hr  = S_OK;    
    static HWND hWndStats;
    WORD        wHi = HIWORD(wParam), 
                wLo = LOWORD(wParam);
    ClientPushManager* cpManager = ClientPushManager::getInstance();
    
    FILECHANGENOTIFY    *lpfcn;
    FILECHANGEINFO      *lpfci;    
    DWORD               dwAttrs;
    WCHAR               fileBuffer[MAX_PATH];
    static list<wstring> fileListTemporary;    

    CBR(hwnd);

    switch (message) 
    {
        case WM_COMMAND:
            LOG.info("CP Briefcase: command change dir");
            cpManager->unsubscribeBriefcase();
            cpManager->subscribeBriefcase();
            break;

        case WM_CLOSE:
        case WM_DESTROY:
            PostQuitMessage(0);
            break;              
       
        case WM_FILECHANGEINFO:
            // see if the pointer to the file change notify struct is valid
            lpfcn = (FILECHANGENOTIFY *)lParam;
            if (NULL == lpfcn) {
                LOG.error("CP Briefcase: error notify briefcase change");
                break;
            }
            // see if the pointer to the file change info structure
            lpfci = &(lpfcn->fci);
            hr = StringCchPrintf(fileBuffer, 
                    (sizeof(fileBuffer)/sizeof(*fileBuffer)), 
                    TEXT("%s"), 
                    lpfci->dwItem1);

            if (hr != S_OK) {
                LOG.error("CP Briefcase: Insufficient buffer to get the file name");
                break;
            } 
            switch (lpfci->wEventId) { 
                case SHCNE_CREATE:
                {
                    LOG.info("CP Briefcase: Creating Item...");                    
                    dwAttrs = GetFileAttributes(fileBuffer);
                    if (dwAttrs == INVALID_FILE_ATTRIBUTES) {
                        LOG.info("CP Briefcase: The file is a temporary file. It must not be signaled");   
                        fileListTemporary.push_back(fileBuffer);                        
                    } else {
                        LOG.info("CP Briefcase: It is a new item inserted");
                        cpManager->setPIMModifications(constFiles);                        
                    }
                }
                break;                

                case SHCNE_RENAMEITEM: 
                {
                    LOG.info("CP Briefcase: Renaming Item...");
                    cpManager->setPIMModifications(constFiles);
                     
                    //Currently all SHCNE_RENAMEITEM are modification. The following code can be used in conjunction
                    //with the SHCNE_CREATE and SHCNE_DELETE to get the file name modified
                    
                    bool found = false;                                        
                    list<wstring>::iterator it;
                    for (it = fileListTemporary.begin(); it != fileListTemporary.end(); it++) {                        
                           if (*it == fileBuffer) {
                               fileListTemporary.erase(it);
                               LOG.info("CP Briefcase: It is a renaming of a temp file so a modification of original");
                               found = true;
                               break;
                           }
                    }
                    if (!found) {
                        LOG.info("CP Briefcase: It is a renaming...");
                    }                                                        
                }
                break;                    
                
                case SHCNE_DELETE: {
                    LOG.info("CP Briefcase: Deleting Item...");
                    if (fileListTemporary.empty()) {
                        LOG.info("CP Briefcase: The item is really deleting...");
                        cpManager->setPIMModifications(constFiles);
                    }                                        
                }
                break;                    
            }
            if (NULL == lpfci) {
                break;
            } 
            SHChangeNotifyFree(lpfcn);
            break;            
                
        default:
            return DefWindowProc(hwnd, message, wParam, lParam);
    }

Error:
   
    return FALSE;
}

/**
* list of file that can be skipped directly in the handling of file changes...
* 
*/
static bool toSkip(const WCHAR* buffer) {
    bool ret = false;
    if (wcsstr(buffer, TEXT(".dat.jour")) ||
        wcsstr(buffer, TEXT(".dat"))   ) {
        ret = true;
    }
    return ret;
}


LRESULT CALLBACK wndMainProcPicture(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam) 
{
    HRESULT     hr  = S_OK;    
    static HWND hWndStats;
    WORD        wHi = HIWORD(wParam), 
                wLo = LOWORD(wParam);
    ClientPushManager* cpManager = ClientPushManager::getInstance();
    
    FILECHANGENOTIFY    *lpfcn;
    FILECHANGEINFO      *lpfci;    
    DWORD               dwAttrs;
    WCHAR               fileBuffer[MAX_PATH];
    static list<wstring> fileListTemporaryPic;    

    CBR(hwnd);

    switch (message) 
    {
        case WM_COMMAND:
            LOG.info("CP Picture: command change dir");
            cpManager->unsubscribePicture();
            cpManager->subscribePicture();
            break;

        case WM_CLOSE:
        case WM_DESTROY:
            PostQuitMessage(0);
            break;              
       
        case WM_FILECHANGEINFO:
            // see if the pointer to the file change notify struct is valid
            lpfcn = (FILECHANGENOTIFY *)lParam;
            if (NULL == lpfcn) {
                LOG.error("CP Picture: error notify picture change");
                break;
            }
            // see if the pointer to the file change info structure
            lpfci = &(lpfcn->fci);
//            hr = StringCchPrintf(fileBuffer, 
//                    (sizeof(fileBuffer)/sizeof(*fileBuffer)), 
//                    TEXT("%s"), 
//                    lpfci->dwItem1);
            wsprintf(fileBuffer, TEXT("%s"), lpfci->dwItem1);
            if (!toSkip(fileBuffer)) {
                if (hr != S_OK) {
                    LOG.error("CP Picture: Insufficient buffer to get the file name");
                    break;
                } 
                switch (lpfci->wEventId) { 
                    case SHCNE_CREATE:
                    {
                        LOG.info("CP Picture: Creating Item...");  
                        
                        dwAttrs = GetFileAttributes(fileBuffer);                    
                        if (dwAttrs == INVALID_FILE_ATTRIBUTES) {
                            LOG.info("CP Picture: The file is a temporary file. It must not be signaled");   
                            fileListTemporaryPic.push_back(fileBuffer);                        
                        } else {
                            LOG.info("CP Picture: It is a new item inserted");
                            cpManager->setPIMModifications(constPicture);                        
                        }

                    }
                    break;                

                    case SHCNE_RENAMEITEM: 
                    {
                        LOG.info("CP Picture: Renaming Item...");
                        cpManager->setPIMModifications(constPicture);
                         
                        //Currently all SHCNE_RENAMEITEM are modification. The following code can be used in conjunction
                        //with the SHCNE_CREATE and SHCNE_DELETE to get the file name modified
                        
                        bool found = false;                                        
                        list<wstring>::iterator it;
                        for (it = fileListTemporaryPic.begin(); it != fileListTemporaryPic.end(); it++) {                        
                               if (*it == fileBuffer) {
                                   fileListTemporaryPic.erase(it);
                                   LOG.info("CP Picture: It is a renaming of a temp file so a modification of original");
                                   found = true;
                                   break;
                               }
                        }
                        if (!found) {
                            LOG.info("CP Picture: It is a renaming...");
                        }                                                        
                    }
                    break;                    
                    
                    case SHCNE_DELETE: {
                        LOG.info("CP Picture: Deleting Item...");
                        if (fileListTemporaryPic.empty()) {
                            LOG.info("CP Picture: The item is really deleting...");
                            cpManager->setPIMModifications(constPicture);
                        }                                        
                    }
                    break;                    
                }
                if (NULL == lpfci) {
                    break;
                } 
            }
            SHChangeNotifyFree(lpfcn);
            break;            
                
        default:
            return DefWindowProc(hwnd, message, wParam, lParam);
    }

Error:
   
    return FALSE;
}

///////////////////////////////////////////////////////////////////////////////
// Function:
//      DlgMainProc
//
//  Process WM messages to the dialog box. 
//
// Purpose:
//      Checks if the app dialog class already exists. If so, it means the app 
//      is already running so we quit this instance.
//
// Parameters:
//      hDlg        - [IN] Handle to main dialog 
//      message     - [IN] WM message
//      wParam      - [IN] WPARAM for WM message
//      lParam      - [IN] LPARAM for WM message. It is the folder when the
//                         PIM_ITEM_ is invoked
//
// Returns:
//      An LRESULT
//
LRESULT CALLBACK wndMainProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam) 
{
    HRESULT     hr  = S_OK;    
    static HWND hWndStats;
    WORD        wHi = HIWORD(wParam), 
                wLo = LOWORD(wParam);
    ClientPushManager* cpManager = ClientPushManager::getInstance();
    // Keep this static so we reuse the same variable each time this function is called.
    static IPOutlookItemCollection *pCol;
        
    CBR(hwnd);

    switch (message) 
    {
        case WM_CLOSE:
            LOG.info("CP: WM_CLOSE the application");
            cpManager->suspendCP(); // deregister all subscribers
            cpManager->UnInitPOOM();
            // stop the threads to check the startsync
            // stopStartSyncThreadCheck(); 
            // reset the flag in the registry
            resetPIMModifications(NULL);

            LOG.debug("Unregister class");
            UnregisterClass(cpManager->cp_wncClass.lpszClassName,         cpManager->cp_wncClass.hInstance);
            UnregisterClass(cpManager->cp_wncClassNote.lpszClassName,     cpManager->cp_wncClassNote.hInstance);
            UnregisterClass(cpManager->cp_wncClassBriefcase.lpszClassName, cpManager->cp_wncClassBriefcase.hInstance);
            UnregisterClass(cpManager->cp_wncClassPicture.lpszClassName,   cpManager->cp_wncClassPicture.hInstance);
            LOG.debug("End dialog");
            EndDialog(hwnd, 0);
            LOG.debug("Destroy wnd");
            DestroyWindow (hwnd);            
            break;
            
        case WM_DESTROY:
            PostQuitMessage(0);
            break;    

        case WM_COMMAND:
            {                
                if (lParam == 1) {
                    LOG.info("CP PIM WM_COMMAND setFlag %i", wParam);
                    cpManager->setFlags(wParam);
                } else {
                    LOG.info("CP PIM WM_COMMAND resetFlag %i", wParam);
                    cpManager->resetFlags(wParam);
                }               
            }
            break;

        case PIM_ITEM_CREATED_LOCAL:
        case PIM_ITEM_DELETED_LOCAL:
        case PIM_ITEM_CHANGED_LOCAL:
            // no modification in local
            break;
        case PIM_ITEM_CREATED_REMOTE:
        case PIM_ITEM_DELETED_REMOTE:
        case PIM_ITEM_CHANGED_REMOTE:            
            cpManager->setPIMModifications(lParam);
            LOG.debug("CP: Item is %lu", wParam);
            // Caught a POOM notification.
            
            CHR(hr);
            break;               

        default:
            return DefWindowProc(hwnd, message, wParam, lParam);
    }

Error:
   
    return FALSE;
}


///////////////////////////////////////////////////////////////////////////////
// Function:
//      ItemTypeToFolder
//
// Purpose:
//      Returns the Outlook folder type given the Outlook Item type.
//
// Parameters:
//      olItemType - Outlook item type
//
// Returns:
//      An OlDefaultFolders to the specified Item Type
//
OlDefaultFolders ClientPushManager::ItemTypeToFolder(int olItemType)
{
    OlDefaultFolders olFolder = (OlDefaultFolders) 0;
    
    switch(olItemType)
    {
        case olAppointmentItem:
            olFolder = olFolderCalendar;
            break;
            
        case olContactItem:
            olFolder = olFolderContacts;
            break;

        case olTaskItem:
            olFolder = olFolderTasks;
            break;
            
        default:
            ASSERT(FALSE);
    }

    return olFolder;
}


///////////////////////////////////////////////////////////////////////////////
// Function:
//      SubscribeToNotifications
//
// Purpose:
//      This function is used to set the level of notification the current process
//      want to get from POOM.  The options include any one of the three values:
//      PIMFOLDERNOTIFICATION_REMOTE, PIMFOLDERNOTIFICATION_LOCAL,
//      or PIMFOLDERNOTIFICATION_ALL
//
// Parameters:
//      pPoom                   - [IN] ptr to POOM app.
//      olItemType              - [IN] Item type.
//      uiNotificationsType     - [IN] Type of notification, remote or local?
//
// Returns:
//      An HRESULT
//
HRESULT ClientPushManager::subscribeToNotifications(IPOutlookApp2 *pPoom, int olItemType, uint uiNotificationsType)
{
    HRESULT     hr          = 0;
    IFolder     *pFolder    = NULL;
    IItem       *pItem      = NULL;
    CEPROPVAL   propval     = {0};

    // Parameter check
    CBR(NULL != pPoom);
    if(!uiNotificationsType)
    {
        CHR(E_INVALIDARG);
    }

    // Get the folder for the item type.
    hr = pPoom->GetDefaultFolder(ItemTypeToFolder(olItemType), &pFolder);
    CHR(hr);
    CBR(NULL != pFolder);
    
    hr = pFolder->QueryInterface(IID_IItem, (LPVOID*)&pItem);
    CHR(hr);
    CBR(NULL != pItem);

    // Set the props on the folder.
    propval.propid      = PIMPR_FOLDERNOTIFICATIONS;
    propval.val.ulVal   = uiNotificationsType;
    hr = pItem->SetProps(0, 1, &propval);
    CHR(hr);
        
Error:
    RELEASE_OBJ(pItem);
    RELEASE_OBJ(pFolder);
    return hr;    
}


///////////////////////////////////////////////////////////////////////////////
// Function:
//      InitPOOM2
//
// Purpose:
//      Initilizes COM, creates the POOM COM object and calls Logon.
//
// Parameters:
//      void
//
// Returns:
//      An HRESULT
//
HRESULT ClientPushManager::InitPOOM()
{
    HRESULT hr = S_OK;
    
    CHR(CoInitializeEx(NULL, 0));
      
    hr = CoCreateInstance(CLSID_Application, NULL, CLSCTX_INPROC_SERVER, 
        IID_IPOutlookApp2, (LPVOID *) &g_polApp);
    CHR(hr);
    CBR(NULL != g_polApp);

Error:        
    return hr;
}


///////////////////////////////////////////////////////////////////////////////
// Function:
//      UnInitPOOM2
//
// Purpose:
//      Does a logoff on POOM, and un-initilazize COM.
//
// Parameters:
//      void
//
// Returns:
//      An HRESULT
//
HRESULT ClientPushManager::UnInitPOOM()
{
    HRESULT hr = S_OK;

    // Lets release the global ptr
    RELEASE_OBJ(g_pCol);
    
    CBR(g_polApp != NULL);

    // This logoff is for safety. (if its already logged off, S_FALSE will get returned)
    hr = g_polApp->Logoff();
    
Error:    
    RELEASE_OBJ(g_polApp);    
    CoUninitialize();   
    return hr;
}

void ClientPushManager::initCommonVar() {
    
    char* value = node->readPropertyValue(PROPERTY_PATH_INFO);
    wchar_t* v = toWideChar(value);
    appName = v;
    appName += TEXT("\\");
    appName += SYNCAPP;
    
    delete [] value; delete [] v;

    value = node->readPropertyValue(PROPERTY_CLIENT_PUSH_INTERVAL); 
    pushInterval = atoi(value);

}

bool ClientPushManager::rescheduleSync() {
    GetLocalTime(&lpTime);
    pollTime = DT_AddDiff(nano100SecInMin, pushInterval, &lpTime);
    BOOL isSchedul = CeRunAppAtTime((WCHAR*)appName.c_str(), &pollTime); 
    LOG.debug("CP: Next scheduled sync of %S will start at ' %04d-%02d-%02d %02d.%02d.%02d'", 
        appName.c_str(), pollTime.wYear, pollTime.wMonth, pollTime.wDay, pollTime.wHour, 
                     pollTime.wMinute, pollTime.wSecond);

    return isSchedul == TRUE ? true : false;
}

/**
* olFolderType is the value of the contact, calendar and task.
* olFolderType == -1 means that the function is called by the startCP
* If the mask is not 0, means that the device is restarted and the flag is not
* 
*/
void ClientPushManager::setPIMModifications(int olFolderType) {
    
    char* value = NULL;
    int  mask = 0;
    char buf[4];    
    if (!dmt)   { LOG.error("CP setPIMModifications: dmt null"); goto finally; }
    if (!node)  { LOG.error("CP setPIMModifications: node null"); goto finally; }
        
    // Check if we use the queue (queuePush = "1")
    value = node->readPropertyValue(CP_PUSH_LIST);    
    
    if (!value) {
        value = stringdup("0");
    }
    mask = atoi(value);
        
    if (olFolderType == -1 && mask != 0) {                                
        return;
    }    

    //
    // Update the bit-mask with current sources
    //   
    switch(olFolderType)
    {
        case olFolderContacts:     
            if (!syncingContact) {
                mask = mask | 2;
                rescheduleSync();
            }
            break;
        
        case olFolderCalendar:
            if (!syncingCalendar) {
                mask = mask | 4;
                rescheduleSync();
            }
            break;            
        
        case olFolderTasks:
            if (!syncingTask) {
                mask = mask | 8;
                rescheduleSync();
            }
            break;

        case constNotes:
            if (!syncingNote) {
                mask = mask | 16;
                rescheduleSync();
            }
            break;
        
        case constFiles:
            if (!syncingBriefcase) {
                mask = mask | 32;
                rescheduleSync();
            }
            break;
            
        case constPicture:
            if (!syncingPicture) {
                mask = mask | 64;
                rescheduleSync();
            }
            break;
            
        default:
            break;
    }
   
    // Write 'clientPushList' value to registry
    sprintf(buf, "%d", mask);
    node->setPropertyValue(CP_PUSH_LIST, buf);           

finally:
    if (value) delete [] value;

}

void ClientPushManager::setFlags(int wParam) {
    switch(wParam) 
    {
        case SOURCE_CONTACTS:
            syncingContact = true;
            break;
        case SOURCE_CALENDAR:
            syncingCalendar = true;
            break;
        case SOURCE_TASKS:
            syncingTask = true;
            break;
        case SOURCE_NOTES:
            syncingNote = true;
            break;
        case SOURCE_BRIEFCASE:
            syncingBriefcase = true;
            break;        
        case SOURCE_PICTURES:
            syncingPicture = true;
            break; 
    }

}

void ClientPushManager::resetAllFlags() {
    syncingContact = false;
    syncingCalendar = false;
    syncingTask = false;
    syncingNote = false;
    syncingBriefcase = false;
    syncingPicture      = false;
}

void ClientPushManager::resetFlags(int wParam) {
    switch(wParam) 
    {   
        
        case 100:
            resetAllFlags();            
            break;
        case SOURCE_CONTACTS:
            syncingContact = false;
            break;
        case SOURCE_CALENDAR:
            syncingCalendar = false;
            break;
        case SOURCE_TASKS:
            syncingTask = false;
            break;
        case SOURCE_NOTES:
            syncingNote = false;
            break;
        case SOURCE_BRIEFCASE:
            syncingBriefcase = false;
            break;        
        case SOURCE_PICTURES:
            syncingPicture = false;
            break; 
    }
}


int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow) {
    int ret = 0;
    
    // Init log.
    LOG.setLogPath(CP_LOG_PATH);
    LOG.setLogName(CP_LOG_NAME);
    LogLevel l = getLOGLevel();
    LOG.setLevel(l);
    
    // Start CP.
    ClientPushManager* cp = ClientPushManager::getInstance();
    cp->startCP(GetModuleHandle(0));

    return ret;
}

