/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */
// CustomList.cpp : implementation file
//
#include "stdafx.h"
#include "ui.h"
#include "CustomList.h"
#include "uiDlg.h"
#include "HwndFunctions.h"
#include "pim/ClientSettings.h"
#include "pim/maincpp.h"
#include "notify/timed_msgbox.h"

#include "initguid.h"
#include "imgguids.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CCustomList

CCustomList::CCustomList() {

    brushHollow.CreateStockObject(HOLLOW_BRUSH);

    nItemWidth   = 400;
    nItemHeight  = 30;
    locked = false;

    // Main logo members
    mainLogo = NULL;
    mainLogoCounter = 0;
    isMainLogoAnimated = false;    
}

CCustomList::~CCustomList() {
    pItemList.RemoveAll();

    if (mainLogo) {
    	mainLogo->Release();
        mainLogo = NULL;
        CoUninitialize();
    }
    DestroyWindow();
}

BEGIN_MESSAGE_MAP(CCustomList, CListCtrl)
	
	ON_WM_SETFOCUS()
	ON_WM_KILLFOCUS()
    ON_WM_LBUTTONDOWN()
	ON_WM_CREATE()
    ON_MESSAGE(WM_SETFONT, OnSetFont)
    ON_WM_MEASUREITEM_REFLECT( )
    ON_WM_TIMER()
    ON_WM_ERASEBKGND()
    ON_WM_PAINT()
	
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCustomList message handlers

void CCustomList::Init()
{
    // Insert at least one column to the list control
    VERIFY(InsertColumn(0, _T(""), LVCFMT_LEFT) == 0);
	SetColumnWidth(0, LVSCW_AUTOSIZE_USEHEADER);

    // Initialize the main logo only if it's a portal build.
    ClientSettings* cs = getRegConfig();
    if (!cs->getBrandingParams().getShowBriefcaseSource() &&
        !cs->getBrandingParams().getShowNoteSource() ) {
        //if(cs->getIsPortal()) {
        initMainLogo();
    }
    // define the icon size. we use the 44 and 22 ico size
    icon_size = GetSystemMetrics(SM_CXICON);
#if defined(WIN32_PLATFORM_PSPC)  
    if (icon_size == 64) { icon_size = 44; }
    if (icon_size == 43) { icon_size = 22; } // 
    if (icon_size == 32) { icon_size = 22; }
#else
    icon_size = 22;
#endif
    
    x_screen_size = GetSystemMetrics(SM_CXSCREEN);
    
    initializeIconHandle();
}


void CCustomList::initializeIconHandle() {
        
    h0 = (HICON)LoadImage(GetModuleHandle(NULL), 
        MAKEINTRESOURCE(IDI_ARROWS22A), IMAGE_ICON, icon_size, icon_size, 0);
    h1 = (HICON)LoadImage(GetModuleHandle(NULL), 
        MAKEINTRESOURCE(IDI_ARROWS22E), IMAGE_ICON, icon_size, icon_size, 0);
    h2 = (HICON)LoadImage(GetModuleHandle(NULL), 
        MAKEINTRESOURCE(IDI_ARROWS22B), IMAGE_ICON, icon_size, icon_size, 0);
    h3 = (HICON)LoadImage(GetModuleHandle(NULL), 
        MAKEINTRESOURCE(IDI_ARROWS22F), IMAGE_ICON, icon_size, icon_size, 0);
    h4 = (HICON)LoadImage(GetModuleHandle(NULL), 
        MAKEINTRESOURCE(IDI_ARROWS22C), IMAGE_ICON, icon_size, icon_size, 0);
    h5 = (HICON)LoadImage(GetModuleHandle(NULL), 
        MAKEINTRESOURCE(IDI_ARROWS22G), IMAGE_ICON, icon_size, icon_size, 0);
    h6 = (HICON)LoadImage(GetModuleHandle(NULL), 
        MAKEINTRESOURCE(IDI_ARROWS22D), IMAGE_ICON, icon_size, icon_size, 0);
    h7 = (HICON)LoadImage(GetModuleHandle(NULL), 
        MAKEINTRESOURCE(IDI_ARROWS22H), IMAGE_ICON, icon_size, icon_size, 0);    
}


void CCustomList::PreSubclassWindow() {

	Init();
	CListCtrl::PreSubclassWindow();
}

int CCustomList::GetCurSelItem() const {
    
    ASSERT(::IsWindow(m_hWnd));

    // Return the index of the item with the focus
    return GetNextItem(-1, LVNI_FOCUSED);
}

BOOL CCustomList::SetCurSelItem(int nIndex) {

    ASSERT(::IsWindow(m_hWnd));

    // Set the focus to the item with the specified index
    return SetItemState(nIndex, LVIS_FOCUSED, LVIS_FOCUSED);
}



#if defined(_DEVICE_RESOLUTION_AWARE)
/**
 * Returns the offset of status labels from the left edge.
 * Actually we use 1/3 of the main screen width.
 */
int CCustomList::GetTextOffSet()
{
    // Get client area          *** TODO: use a class variable to store client rect ***
    CRect rect;
    GetClientRect(&rect);
    
    int minStatus = 128;

    /*
    * This check is only for smartphone size device that has
    * small screen size 176x220
    */
    if (x_screen_size == 176) {
        minStatus = 132;
    }
    // minimum status with = 128px (otherwise we won't read the whole labels...)
    int statusWidth = max((int)(rect.Width() * 0.74), minStatus);

    // 26% of the width for the source title ("Contacts")
    // 74% of the width for the source status ("Not Synchronized")
    return (rect.Width() - statusWidth);
}
#endif


void CCustomList::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct) {

    ASSERT(::IsWindow(m_hWnd));
    ASSERT(lpDrawItemStruct != 0);
    if( (! lpDrawItemStruct) || (! lpDrawItemStruct->hDC) ){
        return;
    }
    CDC* pDC = CDC::FromHandle(lpDrawItemStruct->hDC);
    ASSERT(pDC != NULL);
    if(!pDC){
        return;
    }

    // fix width
    lpDrawItemStruct->rcItem.right = nItemWidth;

    int nItem = lpDrawItemStruct->itemID;

    int nSavedDC = pDC->SaveDC();
    if(nSavedDC == 0)
        return;

    // a memory DC, we first draw into memory and then copy the content to the device screen
    CDC tempdc; tempdc.CreateCompatibleDC(pDC);

    // Get item image and state info
    LV_ITEM lvi;
    lvi.mask = LVIF_IMAGE | LVIF_STATE;
    lvi.iItem = nItem;
    lvi.iSubItem = 0;
    lvi.stateMask = 0xFFFF;     // get all state flags
    GetItem(&lvi);

    // Should the item be highlighted
    BOOL bHighlight =  lvi.state & LVIS_FOCUSED;

    CRect rectFull(lpDrawItemStruct->rcItem);
    // icon and text rect: get square dimension from panes height
    int iconSize = rectFull.Height();

    /*
    * This check is only for smartphone size device that has
    * small screen size 176x220
    */
    if (x_screen_size == 176) {
        iconSize = 25;
    }
    CRect rectIcon(rectFull.left,  rectFull.top, rectFull.left+iconSize, rectFull.top+iconSize);
    CRect rectText(rectIcon.right, rectFull.top, rectFull.right,         rectFull.bottom);

    CExtItem* pItem = NULL;
    if(lpDrawItemStruct->itemID > (UINT)pItemList.GetUpperBound())
        return;

    pItem = pItemList.GetAt(lpDrawItemStruct->itemID);

    if(! pItem){
        LOG.error("CustomList: item is NULL.");
        return;
    }
    int orientation = -1;
    switch(DRA::GetDisplayMode()) {        
        case DRA::Landscape: 
            {                     
                orientation = 0;                
            }
            break;

        case DRA::Square:
            {
                orientation = 1;
            }
            break;
        default: //  also case DRA::Portrait:      
            {          
                orientation = 2;            
            }    
            break;
    }    

    tempdc.SetBkMode(TRANSPARENT);

    // we initialize the memory DC
    CRect rectClient;
    CBitmap bmpTemp;

    GetClientRect(&rectClient);
    bmpTemp.CreateCompatibleBitmap(pDC,rectClient.Width(), rectClient.Height());

    HBITMAP pOldBitmap = (HBITMAP) tempdc.SelectObject(bmpTemp);

    pDC->SetBkMode(TRANSPARENT);    

    CRect rect(rectFull);                                                                                

    if (bHighlight) {
        tempdc.Rectangle(rect);                                                                                                                                                                                 
        rect.DeflateRect(1,1,1,1);                                                                                                                                                                              
        tempdc.FillSolidRect(rect, LIST_COLOR_SELECTED_ITEM);	// fill item frame                        
    } else {
        tempdc.FillSolidRect(rect, LIST_COLOR_BACKGROUND);	// fill item frame
    }                                                                                                     
    // draw icon            
    int ico_width = rectIcon.Width();
    int offset = max(abs(ico_width - icon_size)/2, 0);    
    if(pItem->getIcon()) {                                                                               
        tempdc.DrawIcon(max(rectIcon.left+offset, 2), rectIcon.top + offset, pItem->getIcon());                
    }                                                                                                    

    if(pItem->isEnabled()){                                                                              
        tempdc.SetTextColor(LIST_COLOR_TEXT);                                                            
    }                                                                                                    
    else{                                                                                                
        tempdc.SetTextColor(GetSysColor(COLOR_GRAYTEXT));                                                
    }                                                                                                    

    CFont* pOldPDCFont = pDC->SelectObject(&fontBold);                                                   
    CFont* pOldtempdcFont = tempdc.SelectObject(&fontBold);                                              

    rect.CopyRect(rectText);                                                                             
    rect.right -= rectIcon.right;                                                                        
    rect.DeflateRect(2,2);                                                                               
                                                                   
    if (orientation == 0) { // landscape                                                                 
        //rect.left += (int)(0.01* rect.right); //OffsetRect(2,0;
        rect.left += (int)(0.01* rect.right); //OffsetRect(2,0;
    } else if (orientation == 1 || orientation == 2) { // square and portratir  
        rect.left += (int)(0.01* rect.right); //OffsetRect(2,0;
       // nothing to do                      
    }                                                                                                    
                                                                                              
    if( pItem->getName() != ""){                                                                         
        tempdc.DrawText(pItem->getName(),lstrlen(pItem->getName()),                                      
            rect, DT_LEFT | DT_SINGLELINE | DT_VCENTER);                                                 
    }                                                                                                    

    tempdc.SelectObject(pOldPDCFont);                                                                    
    pDC->SelectObject(pOldPDCFont);                                                                      
    pOldtempdcFont->DeleteObject();                                                                      

#if defined(WIN32_PLATFORM_PSPC)                                                                     
    rect.OffsetRect(GetTextOffSet(), 0);                                                             
#else                                                                                                
    rect.OffsetRect(GetTextOffSet(), 1);                                                             
#endif                                                                                               

    if( pItem->getText() != ""){                                                                         
        tempdc.DrawText(pItem->getText(), lstrlen(pItem->getText()),                                     
            rect,DT_LEFT | DT_SINGLELINE | DT_VCENTER);                                                  
    }                                                                                                    

    CRect rectBlt(lpDrawItemStruct->rcItem);
    pDC->BitBlt(rectBlt.TopLeft().x, rectBlt.TopLeft().y, rectBlt.Width(),
        rectBlt.Height(), &tempdc, rectBlt.TopLeft().x,rectBlt.TopLeft().y,SRCCOPY);

    tempdc.SelectObject(pOldBitmap);
    bmpTemp.DeleteObject();

    tempdc.DeleteDC();
}

void CCustomList::RepaintSelectedItems()
{
    CRect rcBounds, rcLabel;

    // Invalidate focused item so it can repaint
    int nItem = GetNextItem(-1, LVNI_FOCUSED);

    if(nItem != -1)
    {
        GetItemRect(nItem, rcBounds, LVIR_BOUNDS);
        GetItemRect(nItem, rcLabel, LVIR_LABEL);
        rcBounds.left = rcLabel.left;

        InvalidateRect(rcBounds, FALSE);
    }

    // Invalidate selected items depending on LVS_SHOWSELALWAYS
    if(!(GetStyle() & LVS_SHOWSELALWAYS))
    {
        for(nItem = GetNextItem(-1, LVNI_SELECTED);
        nItem != -1; nItem = GetNextItem(nItem, LVNI_SELECTED))
        {
            GetItemRect(nItem, rcBounds, LVIR_BOUNDS);
            GetItemRect(nItem, rcLabel, LVIR_LABEL);
            rcBounds.left = rcLabel.left;

            InvalidateRect(rcBounds, FALSE);
        }
    }
#ifdef _DEBUG
    else
        ASSERT(FALSE);
#endif

    UpdateWindow();
}

void CCustomList::OnSetFocus(CWnd* pOldWnd)
{
	CListCtrl::OnSetFocus(pOldWnd);

    // check if we are getting focus from label edit box
    if (pOldWnd != NULL && pOldWnd->GetParent() == this){
        return;
    }

    // repaint items that should change appearance
    if ((GetStyle() & LVS_TYPEMASK) == LVS_REPORT){
        RepaintSelectedItems();
    }

    #ifdef _DEBUG
    else
        ASSERT(FALSE);
    #endif
}

void CCustomList::OnKillFocus(CWnd* pNewWnd)
{
	CListCtrl::OnKillFocus(pNewWnd);

    // check if we are losing focus to label edit box
    if (pNewWnd != NULL && pNewWnd->GetParent() == this){
        return;
    }

    // repaint items that should change appearance
    if ((GetStyle() & LVS_TYPEMASK) == LVS_REPORT)
        RepaintSelectedItems();
#ifdef _DEBUG
    else
        ASSERT(FALSE);
#endif
}

void CCustomList::OnLButtonDown(UINT nFlags, CPoint point){
    // if the list is locked we ignore the message
    if(locked)
        return;

    // try to see if this item is selected for the 2nd time
    int oldSel, newSel;
    oldSel = GetCurSelItem();
    Default(); // call default message handler
    newSel = GetCurSelItem();

    CRect rect; GetItemRect(newSel, &rect, LVIR_BOUNDS);

    if( (rect.PtInRect(point)) &&  (newSel == oldSel) ){
        CWnd* wndMain = AfxGetMainWnd();
        if(wndMain)
            ((CuiDlg*)wndMain)->SelectSource(newSel);
    }

    // clears non-item part of list
    Invalidate();
}

void CCustomList::setText(int id, CString text ){
    CString s1;
    s1+= "("; s1+=text;
    s1+=")";

    CExtItem* pItem = getItem(id);
    if(pItem){
        pItem->setText(s1);
        InvalidatePane(id);
    }
}

CString CCustomList::getText(int id){
    CExtItem* pItem = getItem(id);

    if(pItem){
        return pItem->getText();
    }

    return CString("");
}

CString CCustomList::getItemName(int id){
    CExtItem* pItem = getItem(id);

    if(pItem){
        return pItem->getName();
    }

    return CString("");
}

CString CCustomList::getItemElementName(int id){
    CExtItem* pItem = getItem(id);
    if(pItem){
        return pItem->getItemElementName();
    }
    return CString("");
}

string CCustomList::getSourceName(int id){
    CExtItem* pItem = getItem(id);
    if(pItem){
        return pItem->getSourceName();
    }

    return string("");
}

void CCustomList::enableItem( int id, bool enable ){
    CExtItem* pItem = getItem(id);
    pItem->setEnabled(enable);
}

void CCustomList::setState(int id, int state) {
    CExtItem* item = getItem(id);
    if(item){
        item->setItemState(state);
    }
}

void CCustomList::InvalidatePane(int id) {
    CRect rect;
    GetItemRect(idToIndex(id), rect, LVIR_BOUNDS);
    InvalidateRect(rect);
}

void CCustomList::setIcon(int id, HICON hIcon) {
    CExtItem* pItem = getItem(id);
    if(pItem){
        pItem->setIcon(hIcon);
    }
}

void CCustomList::setFirstIconArrow(int id) {
    CExtItem* pItem = getItem(id);
    if(pItem){
        HICON h = (HICON)LoadImage(GetModuleHandle(NULL), 
                MAKEINTRESOURCE(IDI_ARROWS22A), IMAGE_ICON, icon_size, icon_size, 0);
        pItem->setIcon(h);
    }
    
            
    
}

void CCustomList::startAnim(int id) {
    this->SetTimer(id + ANIM_TIMER_OFFSET, ANIM_ICON_DELAY, NULL);
}

void CCustomList::stopAnim(int id) {
    CExtItem* pItem = getItem(id);

    this->KillTimer(id + ANIM_TIMER_OFFSET);
    resetSourceIcon(id);
}

bool CCustomList::isEnabledItem(int id) {
    CExtItem* pItem = getItem(id);
    if(pItem){
        return pItem->isEnabled();
    }
    return false;
}

int CCustomList::getState(int id) {

    CExtItem* pItem = getItem(id);
    if(pItem){
        return pItem->getItemState();
    }
    return 1; // state_ok default
}

void CCustomList::setStateToAll(int state, bool onlyEnabled) {

    for (int i = 0; i <= pItemList.GetUpperBound(); i++){
        CExtItem* pItem = pItemList.GetAt(i);
        if(pItem){
            if(onlyEnabled) {
                if(pItem->isEnabled()) {
                    pItem->setItemState(state);
                }
            } else{
                pItem->setItemState(state);
            }
        }
    }
}


void CCustomList::MeasureItem(LPMEASUREITEMSTRUCT lpMIS) {

    lpMIS->itemHeight  = nItemHeight;
    lpMIS->itemWidth   = nItemWidth;
}


LRESULT CCustomList::OnSetFont(WPARAM wParam, LPARAM) {
    LRESULT res =  Default();

    CRect rc;
    GetWindowRect( &rc );

    WINDOWPOS wp;
    wp.hwnd = m_hWnd;
    wp.cx = rc.Width();
    wp.cy = rc.Height();
    wp.flags = SWP_NOACTIVATE | SWP_NOMOVE | SWP_NOOWNERZORDER | SWP_NOZORDER;
    SendMessage( WM_WINDOWPOSCHANGED, 0, (LPARAM)&wp );

    return res;
}


void CCustomList::addItem(int _id, CString sItemLabel, string sourceName, int _iconIdEnabled, 
                                           int _iconIdDisabled, CString sitemElementName, 
                                           WindowsSyncSourceConfig* sc) {

    CExtItem* pItem = new CExtItem(_id, sItemLabel, sourceName, _iconIdEnabled, 
                                   _iconIdDisabled, sitemElementName, sc);                                              
    pItemList.Add(pItem);
    InsertItem(GetItemCount(), sItemLabel);
}

CExtItem* CCustomList::getItem(int id) {

    int i = 0;
    while(i < pItemList.GetCount()){
        if(pItemList.GetAt(i)->getId() == id){
            return pItemList.GetAt(i);
        }
        i++;
    }
    return NULL; // should not get here
}

int CCustomList::getCountItem() {

    return pItemList.GetCount();
}


void CCustomList::OnTimer(UINT_PTR nIDEvent) {

    if (nIDEvent == MAIN_LOGO_ID) {
        // It's the main logo, manage it separately.
        animateMainLogo();
        return;
    }

    int sourceId = nIDEvent - ANIM_TIMER_OFFSET;
    if(!doesIdExist(sourceId)) {
        return;
    }
    CExtItem* pItem = getItem(sourceId);
    if(pItem == NULL) {
        return;
    }


    int counter = pItem->getCounterAnim();

    if(counter == 4){   
        counter = 0;
    }
    // counter == 4
    switch(counter){
        case 0:
            this->setIcon(sourceId, h0);
            break;        
        case 1:
            this->setIcon(sourceId, h2);
            break;       
        case 2:
            this->setIcon(sourceId, h4);
            break;       
        case 3:
            this->setIcon(sourceId, h6);
            break;        
    };        
 
/*    
    if(counter == 8){   
        counter = 0;
    }

    switch(counter){
        case 0:
            this->setIcon(sourceId, h0);
            break;
        case 1:
            this->setIcon(sourceId, h1);
            break;
        case 2:
            this->setIcon(sourceId, h2);
            break;
        case 3:         
            this->setIcon(sourceId, h3);
            break;
        case 4:
            this->setIcon(sourceId, h4);
            break;
        case 5:
            this->setIcon(sourceId, h5);
            break;
        case 6:
            this->setIcon(sourceId, h6);
            break;
        case 7:
            this->setIcon(sourceId, h7);
            break;
    };
*/
    counter++;

    pItem->setCounterAnim(counter);
    InvalidatePane(sourceId);
}

void CCustomList::resetSourceIcon(int id) {
    
    CExtItem* item = NULL;  
    int iconId = -1;
    item = getItem(id);
    HICON h;
    if (!item) {
        LOG.error("resetSourceIcon: the item is null: id %i", id);
        return;
    }
    if (item->isEnabled()) {
        iconId = item->getIconIdEnabled();
    } else {
        iconId = item->getIconIdDisabled();
    }  
    if (iconId != -1) {
        h = (HICON)LoadImage(GetModuleHandle(NULL), 
                MAKEINTRESOURCE(iconId), IMAGE_ICON, icon_size, icon_size, 0);
            
        item->setIcon(h);
    }
}

int CCustomList::indexToId( int index ){
    if(index < 0){
        return 0;
        // TODO: signal error
    }
    CExtItem* item = pItemList.GetAt(index);
    if(item){
        return item->getId();
    }

    return 0;
}

int CCustomList::idToIndex(int id){
    CExtItem* pItem;
    int i = 0;
    int index = 0;

    while(i <= pItemList.GetUpperBound() ){
        pItem = pItemList.GetAt(i);
        if(pItem){
            if(pItem->getId() == id){
                index = i;
            }
        }
       i++;
    }

    return index;
}


void CCustomList::resetSourceIcons() {
    
    for(int i = 0; i < GetItemCount(); i++)
        resetSourceIcon(indexToId(i));
}



void CCustomList::setIsSyncing(int id, bool value) {
    
    CExtItem* item = getItem(id);
    if(item) {
        item->setIsItemSyncing(value);
    }
}

bool CCustomList::getIsSyncing(int id) {
    
    CExtItem* item = getItem(id);
    if(item) {
        return item->getIsItemSyncing();
    }
    return false;
}

void CCustomList::OnPaint() {
    
    // call default handler to redraw items if needed
    Default();

    // now we repaint area in which there are no items
    CPaintDC dc(this);
    CRect rect, rectLastItem;
    // get client area
    GetClientRect(&rect);

    // get last item area
    int count = GetItemCount();
    GetItemRect(count-1, &rectLastItem, LVIR_BOUNDS);

    // fill the area below the last item
    
    dc.FillSolidRect(0, rectLastItem.BottomRight().y, rect.Width(),
        rect.Height()-rectLastItem.BottomRight().y, LIST_COLOR_BACKGROUND);
    
    // Redraw the logo on the right 
    // We need to calculate again the main logo position, as it could
    // be changed from Init (it happens on square smartphone)
    calculateMainLogoPosition();
    updateMainLogoImage();
}

BOOL CCustomList::OnEraseBkgnd( CDC* pDC ){
    // we disable automatic redrawing of list background

    // now we repaint area in which there are no items
    CRect rect, rectLastItem;
    CDC tempdc; tempdc.CreateCompatibleDC(pDC);
    CBitmap bmpTemp;

    // get client area
    GetClientRect(&rect);

    int nSavedDC = pDC->SaveDC();

    // get last item area
    int count = GetItemCount();
    GetItemRect(count-1, &rectLastItem, LVIR_BOUNDS);

    CRect rectDraw(0, rectLastItem.BottomRight().y, rect.Width(), rect.Height()-rectLastItem.BottomRight().y);
    tempdc.SetBkMode(TRANSPARENT);

    bmpTemp.CreateCompatibleBitmap(&tempdc, rectDraw.Width(), rectDraw.Height());
    HBITMAP pOldBitmap = (HBITMAP) tempdc.SelectObject(bmpTemp);

    // fill the area below the last item
    tempdc.FillSolidRect(0, rectLastItem.BottomRight().y, rect.Width(),
        rect.Height()-rectLastItem.BottomRight().y, LIST_COLOR_BACKGROUND);

    // bitblt into pDC
    pDC->BitBlt(rectDraw.TopLeft().x, rectDraw.TopLeft().y, rectDraw.Width(),
        rectDraw.Height(), &tempdc, rectDraw.TopLeft().x,rectDraw.TopLeft().y,SRCCOPY);

    tempdc.SelectObject(pOldBitmap);
    bmpTemp.DeleteObject();

    tempdc.DeleteDC();

    return TRUE;
}

BOOL CCustomList::GetItemRect( int nItem,LPRECT lpRect, UINT nCode ) const{
    CListCtrl::GetItemRect(nItem, lpRect, nCode);
    lpRect->right = nItemWidth;

    return 0;
}

bool CCustomList::doesIdExist( int id ){
    CExtItem* pItem;
    int i = 0;
    int index = -1;

    while(i <= pItemList.GetUpperBound() ){
        pItem = pItemList.GetAt(i);
        if(pItem){
            if(pItem->getId() == id){
                index = i;
            }
        }
        i++;
    }

    if(index == -1){
        return false;
    }
    else{
        return true;
    }
}
            

/**
 * Animates the main logo.
 * The internal counter is updated, and then the image is refreshed.
 */
void CCustomList::animateMainLogo() {
    
    if (!mainLogo) {
        return;
    }

    mainLogoCounter++;
    if(mainLogoCounter == MAIN_LOGO_NUM_FRAMES){
        mainLogoCounter = 0;
    }

    //Invalidate();
    InvalidateRect(&mainLogoRect);
    updateMainLogoImage();
}


/**
 * Updates the main logo image:
 * - calculates the image section to redraw (depends on 'mainLogoCounter')
 * - draws the image to the screen
 */
void CCustomList::updateMainLogoImage() {

    if (!mainLogo) {
        return;
    }

    // Get the effective image section to be drawn
    CPaintDC dc(this);
    int xOffset = mainLogoCounter * mainLogoSize.cx;
    RECT offsetRc = {xOffset, 0, xOffset + mainLogoSize.cx, mainLogoSize.cy};

    mainLogo->Draw(dc.GetSafeHdc(), &mainLogoRect, &offsetRc);
}


/**
 * Initializes the main logo image.
 * The image 'mainLogo' will be loaded only if it's a portal build AND 
 * the file 'MAIN_LOGO_NAME' is found under the install dir.
 * If it's not a portal build or file not found, no image will be loaded and
 * 'mainLogo' member is NULL.
 * 
 * - gets the full path of the image file
 * - loads the image file and stores it to the internal member 'mainLogo'
 * - calculates the rect where image will be drawn, and real logo dimension
 */
void CCustomList::initMainLogo() {

    ClientSettings* cs = getRegConfig();
    if (cs->getBrandingParams().getShowBriefcaseSource() ||
        cs->getBrandingParams().getShowNoteSource() ) {    
        // need that also with only 1 source set the logo cannot be 
        // properly displayed        
        mainLogo = NULL;
        return;
    }

    // Get the image full path
    string imageName = cs->getAppPath();
    imageName += "\\";
    imageName += MAIN_LOGO_NAME;
    WCHAR* wImageName = toWideChar(imageName.c_str());

    // Create the imaging factory.
    IImagingFactory *pImgFactory = NULL;          
    CoInitializeEx(NULL, COINIT_MULTITHREADED);
    if (SUCCEEDED(CoCreateInstance (CLSID_ImagingFactory, NULL, CLSCTX_INPROC_SERVER, IID_IImagingFactory, (void **)&pImgFactory))) {        
        pImgFactory->CreateImageFromFile(wImageName, &mainLogo);
        pImgFactory->Release();
    }
    if (wImageName) { 
        delete [] wImageName; wImageName = NULL;
    }

    if (mainLogo) {
        // Image found: calculate position and size
        calculateMainLogoPosition();

        // Set the logo dimension: mainLogoSize
        SIZE imageSize;
        mainLogo->GetPhysicalDimension(&imageSize);
        mainLogoSize.cx = imageSize.cx / MAIN_LOGO_NUM_FRAMES;
        mainLogoSize.cy = imageSize.cy;
    }
    else {
        // No image found!
        CoUninitialize();
        mainLogoRect.bottom = mainLogoRect.top = 0;
        mainLogoRect.left = mainLogoRect.right = 0;
        mainLogoSize.cx = mainLogoSize.cy = 0;
    }
}



/**
 * Gets the rectangle for the main logo (mainLogoRect) and the
 * effective logo dimension (mainLogoSize) from the current 
 * main screen size.
 * They're always the same, so this method is called at the
 * beginning or if the screen size changes.
 */
void CCustomList::calculateMainLogoPosition() {

    if (!mainLogo) {
        return;
    }

    CPaintDC dc(this);
    CRect rect;
    GetClientRect(&rect);
    mainLogoRect.left   = rect.Width()  - MAIN_LOGO_LEFT_CORNER;
    mainLogoRect.right  = rect.Width()  - MAIN_LOGO_RIGHT_CORNER;
    mainLogoRect.top    = rect.Height() - MAIN_LOGO_UPPER_CORNER;    
    mainLogoRect.bottom = rect.Height() - MAIN_LOGO_LOWER_CORNER;

    // Correction of logo dimensions: if logo overlaps the last item pane,
    // we decrease the logo size to fill only the white area at the bottom.
    int count = GetItemCount();
    if (count > 0) {
        CRect rectLastItem;
        GetItemRect(count-1, &rectLastItem, LVIR_BOUNDS);
        if (mainLogoRect.top < rectLastItem.bottom) {
            mainLogoRect.left += (rectLastItem.bottom - mainLogoRect.top);
            mainLogoRect.top = rectLastItem.bottom;
        }
    }

    // Avoid drawing logo if it's too small...
    if ((mainLogoRect.bottom - mainLogoRect.top) < 10) {
        mainLogoRect.bottom = mainLogoRect.top = 0;
        mainLogoRect.left = mainLogoRect.right = 0;
    }
}





/**
 * Starts the timer for the main logo animation.
 * 'OnTimer(MAIN_LOGO_ID)' will be called every MAIN_LOGO_DELAY milliseconds.
 */
void CCustomList::startMainLogo() {
    if (!mainLogo) {
        return;
    }
    if (MAIN_LOGO_USE_ANIMATION) {
        SetTimer(MAIN_LOGO_ID, MAIN_LOGO_DELAY, NULL);
        isMainLogoAnimated = true;
    }
}


/**
 * Kills the timer for the main logo animation.
 */
void CCustomList::stopMainLogo() {
    if (!mainLogo) {
        return;
    }
    if (MAIN_LOGO_USE_ANIMATION) {
        if (KillTimer(MAIN_LOGO_ID)) {
            isMainLogoAnimated = false;
        }
    }
}

/**
* Method to repaint all items in the whole list
*/
void CCustomList::refreshSourcePanes() { 

    for (int i = 0; i < pItemList.GetCount(); i++) {                
        refreshSourcePane(pItemList.GetAt(i)->getId());
    }
}

/**
* Refresh the pane of the item gived its id
*/

void CCustomList::refreshSourcePane(int id){
    
    CExtItem* item = getItem(id);    
    CString s1;
    bool enable = item->getSourceConfig()->isEnabled();
    unsigned long lastSync = item->getSourceConfig()->getLast();
    wstring date_time;
    item->setEnabled(enable);
    HICON h;
    int iconId = -1;
    if (enable) {
        iconId = item->getIconIdEnabled();                          
    } else {
        iconId = item->getIconIdDisabled();               
    }  

    h = (HICON)LoadImage(GetModuleHandle(NULL), 
                MAKEINTRESOURCE(iconId), IMAGE_ICON, icon_size, icon_size, 0); 
    item->setIcon(h);
    
    if(item->getSyncSourceState() == SYNCSOURCE_STATE_NOT_SYNCED) {
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LAST_SYNC_FAILED);
    }
    else if(item->getSyncSourceState() == SYNCSOURCE_STATE_CANCELED){
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_LAST_SYNC_CANCELED);
    }
    else if(lastSync == 0) {
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_NOT_SYNCHRONIZED);
    }
    else {           
        date_time = getFormattedDateAndTime(lastSync);
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SYNCHRONIZED); 
        s1+= TEXT(" ");
        s1 += date_time.c_str();
    }
    
    CString text;
    text += "("; text += s1; text += ")";
    item->setText(text);    
    
    // refresh the line
    InvalidatePane(id);
}

void CCustomList::setCanceledState() {
    
    CExtItem* item = NULL;
    int id = -1;
    for (int i = 0; i < pItemList.GetCount(); i++) {                
        id = pItemList.GetAt(i)->getId();
        item = getItem(id);
        if (item->getItemState() == ITEM_STATE_TO_SYNC) {
            item->setSyncSourceState(SYNCSOURCE_STATE_CANCELED);
        }
    }
}

void CCustomList::setSyncSourceState(int id, int sourceState) {
    
    CExtItem* item = getItem(id);    
    item->setSyncSourceState(sourceState);        
}


int CCustomList::getFirstEnabledSource() {
    
    CExtItem* item = NULL;
    int id = -1;
    bool enabled = false;
    for (int i = 0; i < pItemList.GetCount(); i++) {                
        id = pItemList.GetAt(i)->getId();
        item = getItem(id);
        if ((enabled = item->isEnabled()) == true) {
            break;
        }        
    }
    
    return (enabled == true) ? id : -1;

}

void CCustomList::refreshSourceConfig() {
    
    ClientSettings* cs = getRegConfig();
    cs->read();
    //
    // Now all the references to the sources are gone. we have to reassing them...
    //
    CExtItem* item = NULL;
    int id = -1;
    
    for (int i = 0; i < pItemList.GetCount(); i++) {                
        id = pItemList.GetAt(i)->getId();
        item = getItem(id);
        item->setSourceConfig(cs->getWinSyncSourceConfig(item->getSourceName().c_str()));                       
    }            

}

void CCustomList::setSourceUri(int id, const char* uri) {
    
    CExtItem* item = NULL;
    for (int i = 0; i < pItemList.GetCount(); i++) {                        
        item = getItem(id);
        item->getSourceConfig()->setURI(uri);
        break;                       
    } 
}
const char* CCustomList::getSourceUri(int id) {
    const char* uri = NULL;  
    CExtItem* item = NULL;
    for (int i = 0; i < pItemList.GetCount(); i++) {                        
        item = getItem(id);
        uri = item->getSourceConfig()->getURI();
        break;                       
    }         
    return uri;
}

void CCustomList::setUseSif(int id, bool useSif) {
    
    string sif;
    sif = (useSif == true ? "1" : "0");
    CExtItem* item = NULL;
    for (int i = 0; i < pItemList.GetCount(); i++) {                        
        item = getItem(id);
        item->getSourceConfig()->setUseSif(sif);
        break;                       
    } 
}
bool CCustomList::getUseSif(int id) {
    bool ret = false;  
    CExtItem* item = NULL;
    for (int i = 0; i < pItemList.GetCount(); i++) {                        
        item = getItem(id);
        ret = item->getSourceConfig()->getUseSif() == "1" ? true : false;
        break;                       
    }         
    return ret;
}

void CCustomList::setUseEncryption(int id, bool useEnc) {

    CExtItem* item = NULL;
    string enc = (useEnc == true ? "des" : "0");
    for (int i = 0; i < pItemList.GetCount(); i++) {                        
        item = getItem(id);
        item->getSourceConfig()->setEncryption(enc.c_str());        
    }              
}

bool CCustomList::getUseEncryption(int id) {
    const char* encr = NULL;  
    bool enc = false;  

    CExtItem* item = NULL;
    for (int i = 0; i < pItemList.GetCount(); i++) {                        
        item = getItem(id);
        encr = item->getSourceConfig()->getEncryption();
        if (encr) {
            if (strcmp(encr, "des") == 0) {
                enc = true;
            }
            break;                       
        }
    }         
    return enc;
}