/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

// MailSettings.cpp : implementation file
//

#include "stdafx.h"
#include "ui.h"
#include "uiDlg.h"
#include "MailSettings.h"
#include "MailSettingsAdvanced.h"
#include "pim/ClientSettings.h"
#include "pim/ConfigParams.h"

#include "ClientUtil.h"
#include "examples/config.h"
#include "base/fscapi.h"
#include "spds/constants.h"
#include "spdm/DMTree.h"
#include "spdm/DMTreeFactory.h"
#include "spdm/ManagementNode.h"
#include "hwndfunctions.h"
#include "pim/Maincpp.h"
#include "notify/timed_msgbox.h"

#define ENABLE_CHECK_FOR_FILTER     0   // contact filter, leave hidden

int getMsgSizePosition(int value) {
    switch(value){
        case 2:  return 0;
        case 5:  return 1;
        case 10: return 2;
        case 25: return 3;
        case 50: return 4;
        case 100: return 5;
        case 250: return 6;
        case -1: return 7;
    }
    return 3; //default
}

IMPLEMENT_DYNAMIC(CMailSettings, CDialog)

CMailSettings::CMailSettings(CWnd* pParent )
	: CDialog(CMailSettings::IDD, pParent) { 
    
        oldOrientation = -1;
}

CMailSettings::~CMailSettings() { }

void CMailSettings::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);

#if defined(WIN32_PLATFORM_WFSP)
    DDX_Control(pDX, IDC_SETTMAIL_SPIN2, spinMailSince );
    DDX_Control(pDX, IDC_SETTMAIL_SPIN3, spinMsgSize );
#endif
    DDX_Control(pDX, IDC_FILTERS_MAILSINCE,     lstMailSince );
    DDX_Control(pDX, IDC_FILTERS_MSGSIZE,       lstMsgSize );
    DDX_Control(pDX, IDC_MAIL_LINK_ADVANCED,    linkAdvanced);
    DDX_Control(pDX, IDC_MAIL_CHECK_ATTACHMENTS,checkAttachments);
    DDX_Control(pDX, IDC_MAIL_CHECK_FILTER,     checkFilter);
}

BEGIN_MESSAGE_MAP(CMailSettings, CDialog)
#if defined(_DEVICE_RESOLUTION_AWARE)
    ON_WM_SIZE()
#endif
    ON_BN_CLICKED(IDC_MAIL_LINK_ADVANCED,  &CMailSettings::OnBnClickedMailLinkAdvanced)
    ON_COMMAND(ID_MENU_ADVANCEDSETTINGS,   &CMailSettings::OnBnClickedMailLinkAdvanced)
    ON_MESSAGE(ID_MYMSG_CONFIG_SYNC_ENDED, &CMailSettings::OnMsgConfigSyncEnded)    
    ON_WM_CTLCOLOR()
END_MESSAGE_MAP()

void CMailSettings::InitializeFont() { 

    memset(&lf, 0, sizeof(LOGFONT));
    HFONT hFont = (HFONT) GetStockObject(SYSTEM_FONT);
    GetObject(hFont, sizeof lf, &lf);
    lf.lfHeight += FONT_HEIGHT_OFFSET;
    VERIFY(defaultFontSize.CreateFontIndirect(&lf));
    
    // add the bold
    lf.lfWeight = FW_BOLD;
    VERIFY(fontBold.CreateFontIndirect(&lf));
    
}

bool CMailSettings::createMenu() {
    
    ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
    dlgCommandBar.cbSize = sizeof(SHMENUBARINFO);
    dlgCommandBar.hwndParent = this->GetSafeHwnd();
    dlgCommandBar.dwFlags = SHCMBF_HMENU;
    dlgCommandBar.nToolBarId = IDM_MAIL_SAVE_CANCEL;

    dlgCommandBar.hInstRes = getLocalizationUtils()->getLocaleResource();
    dlgCommandBar.nBmpId = 0;
    dlgCommandBar.cBmpImages = 0;
    if(!SHCreateMenuBar(&dlgCommandBar)){
        TRACE(_T("Cannot create command bar!"));
        return false;
    }
    return true;
}

void CMailSettings::loadSettings() {

    // read from the registry
    ClientSettings* cs = getRegConfig();
    int pos = 0;

    // Download mail since    
    switch(cs->mailssconfig->getDownloadAge()) {

        case -1:  pos = 0; break;
        case  0:  pos = 1; break;
        case  3:  pos = 2; break;
        case  5:  pos = 3; break;
        case  7:  pos = 4; break;
        case 10:  pos = 5; break;
    }

    lstMailSince.SetCurSel(pos);

    // mail size
    pos = 0;
    lstMsgSize.SetCurSel(getMsgSizePosition(cs->mailssconfig->getAttachSize()));

    // attachments
    if (cs->mailssconfig->getBodySize() == -1){
        checkAttachments.SetCheck(BST_CHECKED);
    }

    if (ENABLE_CHECK_FOR_FILTER) {
        // Filter
        WindowsSyncSourceConfig* ssc = cs->getWinSyncSourceConfig("config");
        if (ssc && ssc->isEnabled()) {
            if (cs->getEmailFilter() == "1") {
                checkFilter.SetCheck(BST_CHECKED);
            }
        }
        else {
            // SSource "config" is not supported on Server (we had a 404): hide checkbox.
            checkFilter.EnableWindow(SW_HIDE);
        }
    }
}

BOOL CMailSettings::OnInitDialog() {

    CString s1;
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_MAIL_SETTINGS); SetWindowText(s1);
    CDialog::OnInitDialog();

    // contact check filter is hidden
    // currently disabled
    if (ENABLE_CHECK_FOR_FILTER == 0) {                    
        checkFilter.ShowWindow(SW_HIDE);
    }
    GetDlgItem(IDC_ALERT)->ShowWindow(SW_HIDE);

#if defined(WIN32_PLATFORM_WFSP)
    spinMailSince.SetRange(5,0); spinMailSince.SetBuddy(&lstMailSince);
    spinMsgSize.SetRange(7,0);	 spinMsgSize.SetBuddy(&lstMsgSize);
#endif

    InitializeFont();
    
    LoadStringResources();

    createMenu();    

    loadSettings();
    
    
    // fix for wrong title problem, we get & set the title for the main window
    wchar_t title[256];
    AfxGetMainWnd()->GetWindowText(title,256);
    AfxGetMainWnd()->SetWindowText(title);

    return FALSE;
}


// CMailSettings message handlers

void CMailSettings::OnOK() {

    // save values in registry
    ClientSettings* cs = getRegConfig();
    CString s1, s2; int pos = 0;

    BOOL dirtyFlag = FALSE;

    if(checkStartSync()){
        CDialog::OnOK();
    }

    // mail since
    int value = -1;
    pos = lstMailSince.GetCurSel();

    switch(pos) {
        case 0: value = -1; break;
        case 1: value =  0; break;
        case 2: value =  3; break;
        case 3: value =  5; break;
        case 4: value =  7; break;
        case 5: value = 10; break;
    }

    if(cs->mailssconfig->getDownloadAge() != value) {
        cs->mailssconfig->setDownloadAge(value);
        getRegConfig()->setDirty(CS_DIRTY_SOURCE_MAIL);
        dirtyFlag = TRUE;
    }

    // mail size
    value = -1;
    pos = lstMsgSize.GetCurSel();
    switch(pos) {
        case 0: value=2;break;
        case 1: value=5;break;
        case 2: value=10;break;
        case 3: value=25;break;
        case 4: value=50;break;
        case 5: value=100;break;
        case 6: value=250;break;
        case 7: value=-1;break;
    }

    if(cs->mailssconfig->getAttachSize() != value) {
        cs->mailssconfig->setAttachSize(value);
        cs->setDirty(CS_DIRTY_SOURCE_MAIL);
        dirtyFlag = TRUE;
    }

    // attachments
    if(checkAttachments.GetCheck() == BST_CHECKED) {
        if(cs->mailssconfig->getBodySize() != -1){
            cs->mailssconfig->setBodySize(-1);
            cs->setDirty(CS_DIRTY_SOURCE_MAIL);
            dirtyFlag = TRUE;
        }
    } else {
        if(cs->mailssconfig->getBodySize() != 1) {
            cs->mailssconfig->setBodySize(1);
            cs->setDirty(CS_DIRTY_SOURCE_MAIL);
            dirtyFlag = TRUE;
        }
    }

    if (ENABLE_CHECK_FOR_FILTER) {
        // filter
        if(checkFilter.GetCheck() == BST_CHECKED) {
            if(cs->getEmailFilter() != "1") {
                cs->setEmailFilter("1");
                cs->setConfigSyncDirty(CP_DIRTY_EMAIL_FILTER_STATUS);   // For config sync
                cs->setDirty(CS_DIRTY_ROOT_CONFIG);
                dirtyFlag = TRUE;
            }
        } else {
            if(cs->getEmailFilter() != "0") {
                cs->setEmailFilter("0");
                cs->setConfigSyncDirty(CP_DIRTY_EMAIL_FILTER_STATUS);   // For config sync
                cs->setDirty(CS_DIRTY_ROOT_CONFIG);
                dirtyFlag = TRUE;
            }
        }
    }
    // Save to registry.
    if(dirtyFlag) {
        cs->saveDirty();
    }

    if (ENABLE_CHECK_FOR_FILTER) {
        if (cs->getConfigSyncDirty() != 0) {
            // Start a config sync.
            // Note: here we check the dirty flag, because the email address or
            // visible name could have been modified in advanced dialog.
            if (startConfigSync() == 0) {
                // Dialog is stuck with waiting cursor.
                // Will be re-enabled when sync has finished, calling OnMsgConfigSyncEnded().
                return;
            }
        }
    }
    SetCursor(LoadCursor(NULL, IDC_ARROW));
    CDialog::OnOK();
}

#if defined(_DEVICE_RESOLUTION_AWARE) 
void CMailSettings::OnSize(UINT, int, int) {
    
    int resId = IDD_UI_MAIL;
    if (oldOrientation == -1) {
        DRA::RelayoutDialog( AfxGetInstanceHandle(), this->m_hWnd, MAKEINTRESOURCE(resId));
        oldOrientation = -2;
    }
    int y_offset_from_label = 0;
    int y_offset = 0;
    int x_offset = 0;
    int y_offset_from_attach_box = 0;
    int value_for_spin_distance = 0;
    int y_size = 18;                  // fixed size
    
    int moltiplicator = 1;
    if (CuiDlg::isVGA()) {
        moltiplicator = 2;
    }
    switch(DRA::GetDisplayMode()) {
        case DRA::Landscape:
            {
                
                y_offset_from_label = 10*moltiplicator; // distance between the first label                                        
                y_offset = 13*moltiplicator;            // distance between the checkbox
                x_offset = 4*moltiplicator;            // offset from the IDC_MAIL_STATIC_DOWNLOAD label
                y_offset_from_attach_box = 20*moltiplicator;
                value_for_spin_distance = 2*moltiplicator;
                
            }
            break;        
        case DRA::Square:
            {
                y_offset_from_label = 10*moltiplicator; // distance between the first label                                        
                y_offset = 13*moltiplicator;            // distance between the checkbox  
                x_offset = 4*moltiplicator;            // offset from the IDC_MAIL_STATIC_DOWNLOAD label
                y_offset_from_attach_box = 20*moltiplicator;
                value_for_spin_distance = 3*moltiplicator;
#if defined(WIN32_PLATFORM_PSPC)                 
                if (CuiDlg::isQVGASquare()) {
                     y_size = y_size + 4;   
                }
#endif   
            }
            break;
        default: 
            {
                y_offset_from_label = 10*moltiplicator; // distance between the first label                                        
                y_offset = 13*moltiplicator;            // distance between the checkbox  
                x_offset = 4*moltiplicator;            // offset from the IDC_MAIL_STATIC_DOWNLOAD label
                y_offset_from_attach_box = 20*moltiplicator;
                value_for_spin_distance = 3*moltiplicator;
                
            }
            break;
    }
    
    CRect clientRect;  GetClientRect(&clientRect);    
    CRect winRect;     GetWindowRect(&winRect);
    
    // difference between top of windowRect and Client rect. This is used in the
    // SetWindowsPos
    int offset = winRect.BottomRight().y - clientRect.BottomRight().y;        
    
    CWnd* pWnd = NULL;
        
#if defined(WIN32_PLATFORM_PSPC)
    
    CRect labelTitleRec;
    GetDlgItem(IDC_HEADER_STATIC)->GetWindowRect(&labelTitleRec);               
    
    // draw the line separator
    int line_x_start = 0;
    int line_y_start = labelTitleRec.BottomRight().y - offset + 2*moltiplicator;
    pWnd = GetDlgItem(IDC_STATIC2);
    pWnd->SetWindowPos(GetActiveWindow(), line_x_start,
                                          line_y_start,
                                          clientRect.BottomRight().x,
                                          2*moltiplicator,
                                          SWP_NOZORDER);
    pWnd->SetForegroundWindow();
#endif    
    
    CRect labelRec;    
    GetDlgItem(IDC_MAIL_STATIC_DOWNLOAD)->GetWindowRect(&labelRec);   

    CRect filterRec; GetDlgItem(IDC_FILTERS_MAILSINCE)->GetClientRect(&filterRec);   
    
    int x_start     = (int)(winRect.BottomRight().x * 0.05);  // 10% of the lenght of the screen. (31 original)
    int y_start     = labelRec.BottomRight().y - offset + y_offset_from_label;
    int x_size      = filterRec.BottomRight().x; // size of the component
    y_size          = y_size*moltiplicator;                  // fixed size
       
    
    pWnd = GetDlgItem(IDC_FILTERS_MAILSINCE);
    pWnd->SetWindowPos(GetActiveWindow(), x_start,
                                          y_start,
                                          x_size,
                                          y_size,
                                          SWP_NOZORDER);

#if defined(WIN32_PLATFORM_WFSP)
    // set the spin object for this label
    pWnd = GetDlgItem(IDC_SETTMAIL_SPIN2);
    CRect spinRec2;
    pWnd ->GetClientRect(&spinRec2);
    
    pWnd->SetWindowPos(GetActiveWindow(), x_size + (int)((spinRec2.BottomRight().x) / 
                                                          value_for_spin_distance),
                                          y_start,
                                          spinRec2.BottomRight().x,
                                          y_size,
                                          SWP_NOZORDER);   
    pWnd->SetForegroundWindow();

#endif

    y_start += (y_size + y_offset);

    pWnd = GetDlgItem(IDC_FILTERS_MSGSIZE);
    pWnd->SetWindowPos(GetActiveWindow(), x_start,
                                          y_start,
                                          x_size,
                                          y_size,
                                          SWP_NOZORDER);

#if defined(WIN32_PLATFORM_WFSP)
    // set the spin object for this label
    pWnd = GetDlgItem(IDC_SETTMAIL_SPIN3);
    CRect spinRec3; 
    pWnd->GetClientRect(&spinRec3);    
    pWnd->SetWindowPos(GetActiveWindow(), x_size + (int)((spinRec3.BottomRight().x) / 
                                                          value_for_spin_distance),
                                          y_start,
                                          spinRec3.BottomRight().x,
                                          y_size,
                                          SWP_NOZORDER);   
    pWnd->SetForegroundWindow();

#endif
    y_start += (y_size + y_offset);

    pWnd = GetDlgItem(IDC_MAIL_CHECK_ATTACHMENTS);
    pWnd->SetWindowPos(GetActiveWindow(), x_start,
                                          y_start,
                                          x_size,
                                          y_size,
                                          SWP_NOZORDER);

    y_start += (y_size + y_offset_from_attach_box);

    pWnd = GetDlgItem(IDC_MAIL_LINK_ADVANCED);
    pWnd->SetWindowPos(GetActiveWindow(), x_start,
                                          y_start,
                                          x_size,
                                          y_size + 4*moltiplicator,
                                          SWP_NOZORDER);
    

}
#endif

void CMailSettings::LoadStringResources(){
    CString s1;
        
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_FILTERS_DOWNMAIL_ALL);    lstMailSince.AddString(s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_FILTERS_DOWNMAIL_0DAY);   lstMailSince.AddString(s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_FILTERS_DOWNMAIL_3DAY);   lstMailSince.AddString(s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_FILTERS_DOWNMAIL_5DAY);   lstMailSince.AddString(s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_FILTERS_DOWNMAIL_7DAY);   lstMailSince.AddString(s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_FILTERS_DOWNMAIL_10DAY);  lstMailSince.AddString(s1);

    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_FILTERS_GETMSGSIZE_2K);   lstMsgSize.AddString(s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_FILTERS_GETMSGSIZE_5K);   lstMsgSize.AddString(s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_FILTERS_GETMSGSIZE_10K);  lstMsgSize.AddString(s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_FILTERS_GETMSGSIZE_25K);  lstMsgSize.AddString(s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_FILTERS_GETMSGSIZE_50K);  lstMsgSize.AddString(s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_FILTERS_GETMSGSIZE_100K); lstMsgSize.AddString(s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_FILTERS_GETMSGSIZE_250K); lstMsgSize.AddString(s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_FILTERS_GETMSGSIZE_500K); lstMsgSize.AddString(s1);

    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_ADVANCED_SETTINGS);       linkAdvanced.setText(s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_ATTACHMENTS);             SetDlgItemText(IDC_MAIL_CHECK_ATTACHMENTS,   s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_DOWNLOAD);                SetDlgItemText(IDC_MAIL_STATIC_DOWNLOAD,     s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SETTMAIL_BUTFOLDERS);     SetDlgItemText(IDC_SETTMAIL_BUTFOLDERS,      s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SETTMAIL_BUTFILTER);      SetDlgItemText(IDC_SETTMAIL_BUTFILTER,       s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SETTMAIL_BUTACCOUNT);     SetDlgItemText(IDC_SETTMAIL_BUTACCOUNT,      s1);
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CHECKING_CONFIG);         SetDlgItemText(IDC_ALERT,                    s1);
    
    if (ENABLE_CHECK_FOR_FILTER) {
        s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_MAILS_FILTER_MYCONTACTS); SetDlgItemText(IDC_MAIL_CHECK_FILTER,        s1);
    }

#if defined(WIN32_PLATFORM_PSPC)
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_MAIL_SETTINGS);       SetDlgItemText(IDC_HEADER_STATIC,            s1);
    GetDlgItem(IDC_HEADER_STATIC)->SetFont(&fontBold);
#endif    

    GetDlgItem(IDC_MAIL_STATIC_DOWNLOAD    )->SetFont(&fontBold);
    GetDlgItem(IDC_ALERT                   )->SetFont(&fontBold);
}


void CMailSettings::OnBnClickedMailLinkAdvanced()
{
    CMailSettingsAdvanced wndMailAdvanced;

    INT_PTR result = wndMailAdvanced.DoModal();
}

// capture pressing of 'Advanced' link
BOOL CMailSettings::PreTranslateMessage(MSG* pMsg){
    bool bProcessed = false;
    if(pMsg->message == WM_KEYDOWN) {
        if(pMsg->wParam == VK_RETURN) {
            if(GetFocus()->GetSafeHwnd() == linkAdvanced.GetSafeHwnd()) {
                bProcessed = true;
                OnBnClickedMailLinkAdvanced();
            }
        }
    }

    if(bProcessed) {
        return TRUE;
    } else {
        return CDialog::PreTranslateMessage(pMsg);
    }
}

HBRUSH CMailSettings::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) {
    HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

    // set ppc header text color
#if defined(WIN32_PLATFORM_PSPC)
    if(pWnd->GetDlgCtrlID() == IDC_HEADER_STATIC) {
        pDC->SetBkMode(TRANSPARENT);
        pDC->SetTextColor(PPC_HEADER_TEXT_COLOR);
    }
#endif
    if(pWnd->GetDlgCtrlID() == IDC_ALERT) {
        pDC->SetBkMode(TRANSPARENT);
        pDC->SetTextColor(ALERT_COLOR);
    }
    return hbr;
}



int CMailSettings::startConfigSync() {

    // First of all, check the syncmode. If "disabled" don't do anything.
    ClientSettings* cs = getRegConfig();
    SyncSourceConfig* ssc = cs->getSyncSourceConfig("config");
    if (!ssc) {
        LOG.error("Config Syncsource not existing, aborting config sync.");
        return 1;
    }
    if (ssc->isEnabled() == false) { 
        //LOG.debug("Config syncmode is 'none', aborting config sync.");
        LOG.debug("Config syncmode is 'none', enabled is false, aborting config sync.");
        return 1;
    }

    // Disable the UI.
    SetCursor(LoadCursor(NULL, IDC_WAIT));
    enableUIObjects(false);

    CString s1;
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CHECKING_CONFIG);
    GetDlgItem(IDC_ALERT)->ShowWindow(SW_SHOW);
    SetDlgItemText(IDC_ALERT, s1);

    // Start the config sync, in a separate thread.
    HWND thisWindow = GetSafeHwnd();
    if ( !CreateThread(NULL, 0, configWorker, (LPVOID)thisWindow, 0, NULL) ) {
        LOG.error("Error creating configWorker thread.");
        return 1;
    }

    return 0;             // Dialog is stuck with the waiting cursor here...
}


void CMailSettings::enableUIObjects(bool enable) {

    HWND thisWnd = this->GetSafeHwnd();

    EnableButton(thisWnd, 0, enable);        // '0' is the ID of the 'menu' on the right.
    EnableButton(thisWnd, IDOK, enable);

    lstMailSince.EnableWindow(enable);
    lstMsgSize.EnableWindow(enable);
    linkAdvanced.EnableWindow(enable);
    checkAttachments.EnableWindow(enable);
    if (ENABLE_CHECK_FOR_FILTER) {
        checkFilter.EnableWindow(enable);
    }
   

#if defined(WIN32_PLATFORM_WFSP)
    spinMailSince.EnableWindow(enable);
    spinMsgSize.EnableWindow(enable);
   
#endif

}

LRESULT CMailSettings::OnMsgConfigSyncEnded(WPARAM wparam, LPARAM lparam) {

    bool isOldServer = false;

    int exitcode = (int)wparam;

    switch (exitcode) {
        case 0:                             // OK
            break;

        case ERR_CONFIG_SOURCE_NOT_FOUND:   // 2100 = "config" source not found on the Server.
            // Silently catched in uiDlg::OnStartSyncEnded()
            isOldServer = true;
            break;

        case 401:
        case 402:
        case 403:
            // Msgbox is already displayed.
            LOG.info("Authentication failed, code %d", exitcode);
            break;

            // case 404: should not happen (404 is catched and converted into code 2100)

        default:
            if (exitcode >= 2000) {
                // Error msg already displayed -> uiDlg::OnStartSyncEnded()
                LOG.error("Network error, code %d", exitcode);
            }
            else {
                LOG.error("Generic error, code %d", exitcode);
            }
            break;
    }


    //
    // We MUST not change the flag if Server didn't send a status 200 for EmailFilter.
    //
    if (!isOldServer) {
        ClientSettings* cs = getRegConfig();
        if (cs->getConfigSyncDirty() & CP_DIRTY_EMAIL_FILTER_STATUS) {
            // Means that something went wrong, and the filterStatus
            // was not updated on the Server. So return to the previous selection.
            if (cs->getEmailFilter() == "0") {
                cs->setEmailFilter("1");
            } 
            else {
                cs->setEmailFilter("0");
            }

            // Save the previous value, and warn the user.
            cs->clearConfigSyncDirty(CP_DIRTY_EMAIL_FILTER_STATUS);
            cs->setDirty(CS_DIRTY_ROOT_CONFIG);
            cs->saveDirty();

            TimedMessageBox(GetSafeHwnd(), TEXT("Email options could not be updated correctly. Check log for details."), 
                getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
                MB_OK | MB_ICONHAND | MB_SETFOREGROUND, 10 * 1000);
        }
    }


    // Re-enable UI dialog.
    GetDlgItem(IDC_ALERT)->ShowWindow(SW_HIDE);
    SetDlgItemText(IDC_ALERT, TEXT(""));
    enableUIObjects(true);
    SetCursor(LoadCursor(NULL, IDC_ARROW));

    // Will go to main settings screen. 
    CDialog::OnOK();
    return 0;
}
