/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

// RecoverSettings.cpp : implementation file
//

#include "stdafx.h"
#include "ui.h"
#include "uiDlg.h"
#include "RecoverSettings.h"
#include "pim/utils.h"
#include "pim/Maincpp.h"
#include "base/startcmd.h"
#include "pim/SettingFunctions.h"
#include "pim/ClientSettings.h"
#include "pim/account.h"
#include "ClientUtil.h"
#include "notify/timed_msgbox.h"
#include "HwndFunctions.h"
#include "winreg.h"
#include <vector>
#include "localizationUtils.h"
#include "processUtils.h"

static BOOL cleanupStarted = false;

static int checkRefreshFromServer(HWND wndUI, vector<CString> &sources, BOOL cleanup);
static wstring getCommandLine(vector<const wchar_t*> &sources);
int endCleanUpProcess(HWND wndUI, vector<CString> &sources);
static HANDLE hMutex;

static vector<CString> sourceToSync;   
static vector<CString> labelToSync;   
    
#define CUSTOM_ID_BUTTON    770

// CRecoverSettings dialog
void writeSourceForSlow(const wchar_t*  value);
IMPLEMENT_DYNAMIC(CRecoverSettings, CDialog)

/**
 * Parameters for the cleanup task
 */
class CleanUpParams {
public:
    CDialog &wndRef;
    vector<CString>& srcRef;
    vector<CString>& labelRef;

    CuiDlg* mainDlg;

    CleanUpParams(CDialog &wnd, vector<CString> &sources, 
                                vector<CString> &labels, CuiDlg* dlg) : 
            wndRef(wnd), srcRef(sources), labelRef(labels), mainDlg(dlg) { };
    ~CleanUpParams() {};
} ;

// Cleanup thread
static DWORD WINAPI cleanUpTask(LPVOID lpv);


CRecoverSettings::CRecoverSettings(CWnd* pParent /*=NULL*/)
    : CDialog(CRecoverSettings::IDD, pParent), 
      customList(((CuiDlg*)AfxGetMainWnd())->getSourceListCtrl()) {

          oldOrientation = -1;
          
}

CRecoverSettings::~CRecoverSettings() {    
    cbuttonArray.RemoveAll();
    DestroyWindow();    
}

/**
* The checkbox currently can change the name because they are set from the 
* list of sources
*/
void CRecoverSettings::DoDataExchange(CDataExchange* pDX) {
    CDialog::DoDataExchange(pDX);
}


BEGIN_MESSAGE_MAP(CRecoverSettings, CDialog)

    ON_BN_CLICKED(CUSTOM_ID_BUTTON,     &CRecoverSettings::OnBnClickedAll)
    ON_BN_CLICKED(CUSTOM_ID_BUTTON + 1, &CRecoverSettings::OnBnClickedAll)
    ON_BN_CLICKED(CUSTOM_ID_BUTTON + 2, &CRecoverSettings::OnBnClickedAll)
    ON_BN_CLICKED(CUSTOM_ID_BUTTON + 3, &CRecoverSettings::OnBnClickedAll)
    ON_BN_CLICKED(CUSTOM_ID_BUTTON + 4, &CRecoverSettings::OnBnClickedAll)
    ON_BN_CLICKED(CUSTOM_ID_BUTTON + 5, &CRecoverSettings::OnBnClickedAll)
    ON_BN_CLICKED(CUSTOM_ID_BUTTON + 6, &CRecoverSettings::OnBnClickedAll)
    ON_BN_CLICKED(CUSTOM_ID_BUTTON + 7, &CRecoverSettings::OnBnClickedAll)
    ON_WM_SIZE()
    ON_WM_CTLCOLOR()
    ON_WM_PAINT()

    
END_MESSAGE_MAP()


void CRecoverSettings::InitializeFont() { 

    memset(&lf, 0, sizeof(LOGFONT));
    HFONT hFont = (HFONT) GetStockObject(SYSTEM_FONT);
    GetObject(hFont, sizeof lf, &lf);
    lf.lfHeight += FONT_HEIGHT_OFFSET;
    VERIFY(defaultFontSize.CreateFontIndirect(&lf));
    
    // add the bold
    lf.lfWeight = FW_BOLD;
    VERIFY(fontBold.CreateFontIndirect(&lf));
    
}

bool CRecoverSettings::createMenu() {

    ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
    dlgCommandBar.cbSize = sizeof(SHMENUBARINFO);
    dlgCommandBar.hwndParent = this->GetSafeHwnd();
    dlgCommandBar.dwFlags = SHCMBF_HMENU;
    dlgCommandBar.nToolBarId = IDM_RECOVER;
    
    dlgCommandBar.hInstRes = getLocalizationUtils()->getLocaleResource();
    dlgCommandBar.nBmpId = 0;
    dlgCommandBar.cBmpImages = 0;
    if(!SHCreateMenuBar(&dlgCommandBar)){
        TRACE(_T("Cannot create command bar!"));
        return false;
    }
    return true;
}

void CRecoverSettings::LoadStaticResources() {

    CString s1;
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RECOVER_ITEMS); SetDlgItemText(IDC_RECOVER_STATIC, s1);
    GetDlgItem(IDC_RECOVER_STATIC)->SetFont(&fontBold);

#if defined(WIN32_PLATFORM_PSPC)
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CLEANUP_TITLE); SetDlgItemText(IDC_RECOVER_STATIC_TITLE, s1);    
    GetDlgItem(IDC_RECOVER_STATIC_TITLE)->SetFont(&fontBold);
#endif

}

   
BOOL CRecoverSettings::CreateCustomButton() {
    
    for (int i = 0; i < customList.getCountItem(); i++) {
        int id = customList.indexToId(i);
        CString label = customList.getItemName(id);
        CButton* cb = new CButton();
        CRect rect(40, 40, 100, 20); // default values
        cb->Create(label, BS_AUTOCHECKBOX | BS_NOTIFY | WS_VISIBLE | WS_TABSTOP, rect, GetActiveWindow(), CUSTOM_ID_BUTTON + i);        
        cb->SetFont(&defaultFontSize);
        cb->EnableWindow(customList.isEnabledItem(id));
        // if the first item, set the focus there
        if (i == 0) { 
            cb->SetFocus();
        }
        cbuttonArray.Add(cb);
    }
    return true;
}


// CRecoverSettings message handlers
BOOL CRecoverSettings::OnInitDialog() {
    
    CDialog::OnInitDialog();

    CString s1;
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_RECOVER); SetWindowText(s1);    

    if (!createMenu()) {
        return false;
    }

    SetDlgItemText(IDC_CLEANUP, TEXT(""));
    
    InitializeFont();       

    CreateCustomButton();

    EnableRecoverButton(false);        
    
    LoadStaticResources();

    // fix for wrong title problem, we get & set the title for the main window
    wchar_t title[256];
    AfxGetMainWnd()->GetWindowText(title, 256);
    AfxGetMainWnd()->SetWindowText(title);
      
    return TRUE;
}

/**
* The ok button performs only the cleanup. The CArray contains the list of the check buttons.
* We are confident the list is the same as the list of sources, so we can use the
* indexToId and idToIndex on the CArray
* 
*/
void CRecoverSettings::OnOK(){

    wstring command;
    int id = -1;
    sourceToSync.clear();
    labelToSync.clear();   
    
    if (cbuttonArray.GetSize() > 0) {   
        for (int i = 0; i < cbuttonArray.GetSize(); i++) {
            CButton* cb = cbuttonArray.GetAt(i);
            if (cb && (cb->GetCheck() ==  BST_CHECKED)) {
                id = customList.indexToId(i);                
                CString name(customList.getSourceName(id).c_str()); 
                CString label(customList.getItemName(id));                 
                sourceToSync.push_back(name);
                labelToSync.push_back(label);                
            }
        }
    }
    
    if (sourceToSync.size() > 0) {
        /*
        * Create the Mutex of the sync to avoid to start the sync when the
        * clean up process is in progress. The mutex is closed when the 
        * cleanup procedure is finished.
        */

        hMutex = NULL;  //Handle to mutex...
        hMutex = CreateMutex(NULL, TRUE, TEXT("FunSyncInProgress") );
        CString sss;                              
        switch(GetLastError()) {
            case ERROR_SUCCESS:
                LOG.debug("Mutex created.");
                cleanupStarted = true;
                break;
            case ERROR_ALREADY_EXISTS:
                if (!cleanupStarted) {
                    LOG.info("Cleanup process: a sync is in progress. No cleanup possible...");
                    sss.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CLEANUP_SYNC_IN_PROG);
                    TimedMessageBox(GetSafeHwnd(), sss, 
                        getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
                        MB_OK | MB_ICONEXCLAMATION | MB_SETFOREGROUND, ASK_REFRESH_TIMEOUT * 1000); // default ASK_TIMEOUT = 10
                } 
                // close the handle of the cleanup
                if (hMutex) {
                    CloseHandle(hMutex);                    
                }                
                return;
            default:
                LOG.error("Failed to create mutex");
                // close the handle of the cleanup
                if (hMutex) {
                    CloseHandle(hMutex);
                    cleanupStarted = false;
                }                
                return;                        
        }

        EnableRecoverButton(false);
        EnableCancelButton(false);

        if (checkRefreshFromServer(GetSafeHwnd(), labelToSync, TRUE) == 0) {
            CString s1;
            s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CLEANUP_IN_PROG);
            SetDlgItemText(IDC_CLEANUP, s1);                      
            try {                        
                GetDlgItem(IDC_CLEANUP)->SetFont(&fontBold);                                                                
            } catch (...) {
                LOG.debug("RecoverSettings: font not loaded");
            }                    
            disableCheckbox();
            startCleanUp(sourceToSync, labelToSync);
        }
        else {
            if (hMutex) {
                CloseHandle(hMutex);
                cleanupStarted = false;
            } 
            SetDlgItemText(IDC_CLEANUP, CString(""));
            enableCheckbox();
            EnableRecoverButton(true);
            EnableCancelButton(true);
        }
        GetActiveWindow()->Invalidate();
        GetActiveWindow()->UpdateWindow();            
    }
}


int CRecoverSettings::startCleanUp(vector<CString> &sources, vector<CString> &labels) {
    CleanUpParams *par = new CleanUpParams(*this, sources, labels, ((CuiDlg*)AfxGetMainWnd()));

    if ( !CreateThread(NULL, 0, cleanUpTask, (LPVOID)par, 0, NULL) ) {
        LOG.error("startCleanUp: error running thread.");
        return -1;
    }
    else {
        return 0;
    }
}

/**
 * Clean up thread.
 */
static DWORD WINAPI cleanUpTask(LPVOID lpv) {

    CleanUpParams *par = (CleanUpParams *)lpv;
    if( !par ) {
        LOG.error("cleanUpTask: invalid parameters.");
        return 0;
    }
    SetCursor(LoadCursor(NULL, IDC_WAIT));        
    
    //
    // Call the startsync process: "startsync.exe cleanup <source1>,<source2>..."
    //
    wstring cmdLine(TEXT("cleanup"));
    vector<CString>& sourceToSync = par->srcRef;
    vector<CString>::iterator it;
    int count = 0;
    wchar_t** currentSources = new wchar_t*[10];
    for (it = sourceToSync.begin(); it != sourceToSync.end(); it++) {
        if (count == 0) { cmdLine += L" "; }
        else            { cmdLine += L","; }
        cmdLine += ((*it));   
        currentSources[count] = wstrdup(*it);
        count++;
    }
    currentSources[count] = NULL;

    int pid = startProgram(SYNCAPP, cmdLine.c_str());

    // Wait till the end of the cleanup...
    int ret = waitProcess(pid, INFINITE);
    
    SetCursor(LoadCursor(NULL, IDC_ARROW));
    par->wndRef.SetDlgItemText(IDC_CLEANUP, TEXT(""));

    // read the queue list and remove the sources that are already clenedup.
    // set in the registry only the ones that are not cleaned up.
    wchar_t** queue = readQueueList();
    if (queue) {
        //vector<const wchar_t*> sourceToSync;
        //sourceToSync = par->srcRef;
        //vector<const wchar_t*>::iterator it;
        bool found = false;
        wchar_t** newQueue = new wchar_t*[10];
        int i = 0;
        for (int count = 0; queue[count] != NULL; count++) {    
            for (it = sourceToSync.begin(); it != sourceToSync.end(); it++) {
                if (wcscmp(queue[count], (*it)) == 0) {
                    found = true;
                    break;
                }
            }   
            if (found) {
                if (sourceToSync.size() > 0) {
                    sourceToSync.erase(it);
                } 
                found = false;
            } else {
                newQueue[i] = wstrdup(queue[count]);
                i++;                    
            }            
        } 
        newQueue[i] = NULL;
           
        for (int i = 0; queue[i]; i++) {
            delete [] queue[i];
        } 
        delete [] queue;
        updateQueueList((const wchar_t**)newQueue);
        delete [] newQueue;
        for (int i = 0; newQueue[i]; i++) {
            delete [] newQueue[i];
        } 
        delete [] newQueue;
    }
        
    endCleanUpProcess(par->wndRef.GetSafeHwnd(), par->labelRef);
    
    // try to reset the pim queue if there are any modification after a cleanup
    // it prevents to start a sync that the user doesn't want
    resetPIMModifications((const wchar_t**)currentSources);
    for (int i = 0; currentSources[i]; i++) {
        delete [] currentSources[i];
    }
    delete [] currentSources;

    // Release the mutex if the cleanup started
    if (hMutex) {
        CloseHandle(hMutex);
        cleanupStarted = false;
    } 

    par->wndRef.EndDialog(RECOVER_SYNC_STARTED);
    delete par;
    return 0;
}


void CRecoverSettings::EnableButton(bool bEnable, int button) {
    
    HWND hwndMB = SHFindMenuBar (this->GetSafeHwnd());
    TBBUTTONINFO tbi;
    if (!hwndMB) {
        return;
    }
    memset (&tbi, 0, sizeof (tbi));
    tbi.cbSize = sizeof (tbi);
    tbi.dwMask = TBIF_STATE;
    ::SendMessage (hwndMB, TB_GETBUTTONINFO, button, (LPARAM)&tbi);

    if(bEnable) {
        tbi.fsState = TBSTATE_ENABLED;
    } else {
        tbi.fsState &= ~TBSTATE_ENABLED;
    }
    ::SendMessage (hwndMB, TB_SETBUTTONINFO, button, (LPARAM)&tbi);

}

void CRecoverSettings::EnableRecoverButton(bool bEnable) {

    EnableButton(bEnable, IDOK);       
}

void CRecoverSettings::EnableCancelButton(bool bEnable) {
    
    EnableButton(bEnable, IDCANCEL); 
}

void CRecoverSettings::OnBnClickedAll() {

   EnableRecoverButton(isCheckboxChecked());   
}

bool CRecoverSettings::isCheckboxChecked() {
    
    bool value = false;
    if (cbuttonArray.GetSize() > 0) {   
        for (int i = 0; i < cbuttonArray.GetSize(); i++) {
            CButton* cb = cbuttonArray.GetAt(i);
            if (cb && (value = (cb->GetCheck() ==  BST_CHECKED))) {                
                break;
            }
        }
    }
    return value;
}

#if defined(_DEVICE_RESOLUTION_AWARE)
void CRecoverSettings::OnSize(UINT, int, int) {
    
    int resId = IDD_UI_RECOVER;
    if (oldOrientation != -1) {
        DRA::RelayoutDialog( AfxGetInstanceHandle(), this->m_hWnd, MAKEINTRESOURCE(resId));
        oldOrientation = 0;;
    }
    if (cbuttonArray.GetSize() == 0) {
        return;
    }
    
    CString s1 ,s2;
    int y_offset_from_label = 0;
    int y_offset = 0;
    int y_offset_from_last_source = 0;
    int y_checkbox_size = 18;
    int y_line_separator = 2;
    
    int moltiplicator = 1;
    if (CuiDlg::isVGA()) {
        moltiplicator = 2;
    }
    
    switch(DRA::GetDisplayMode()) {        
        case DRA::Landscape: 
            {   
                y_offset_from_label = 5*moltiplicator; // distance between the first label                                        
                y_offset = 2*moltiplicator;            // distance between the checkbox
                y_offset_from_last_source = 2*moltiplicator; // for the cleanup label info
                y_checkbox_size = y_checkbox_size*moltiplicator;
                y_line_separator *= moltiplicator;
            }
            break;
        
        case DRA::Square:
            {
                y_offset_from_label = 3*moltiplicator; // distance between the first label                                        
                y_offset = 2*moltiplicator;            // distance between the checkbox  
                y_offset_from_last_source = 2*moltiplicator; // for the cleanup label info
                y_checkbox_size = y_checkbox_size*moltiplicator;
                y_line_separator *= moltiplicator;
#if defined(WIN32_PLATFORM_PSPC)
                if (CuiDlg::isQVGASquare()) {                
                    y_offset_from_label = 5*moltiplicator; // distance between the first label                                        
                    y_offset = 3*moltiplicator;            // distance between the checkbox  
                    y_offset_from_last_source = 3*moltiplicator; // for the cleanup label info
                    y_checkbox_size = (y_checkbox_size + 4)*moltiplicator;                    
                }
#endif                
            }
            break;
        default: //  also case DRA::Portrait:      
        {      
            y_offset_from_label = 5*moltiplicator; // distance between the first label                                        
            y_offset = 4*moltiplicator;            // distance between the checkbox  
            y_offset_from_last_source = 4*moltiplicator; // for the cleanup label info
            y_checkbox_size = y_checkbox_size*moltiplicator;
            y_line_separator *= moltiplicator;
        }    
        break;
    };        
 
    CRect clientRect;  GetClientRect(&clientRect);    
    CRect winRect;     GetWindowRect(&winRect);
    
    // difference between top of windowRect and Client rect. This is used in the
    // SetWindowsPos
    int offset = winRect.BottomRight().y - clientRect.BottomRight().y;        

    CRect labelRec, staticRec;    
    GetDlgItem(IDC_RECOVER_STATIC)->GetWindowRect(&labelRec);    
    
    CWnd* pWnd = NULL;

#if defined(WIN32_PLATFORM_PSPC)

    CRect labelTitleRec;
    GetDlgItem(IDC_RECOVER_STATIC_TITLE)->GetWindowRect(&labelTitleRec);               
    
    // draw the line separator
    int line_x_start = 0;
    int line_y_start = labelTitleRec.BottomRight().y - offset + y_line_separator;
    pWnd = GetDlgItem(IDC_STATIC2);
    pWnd->SetWindowPos(GetActiveWindow(), line_x_start,
                                          line_y_start,
                                          clientRect.BottomRight().x,
                                          y_line_separator,
                                          SWP_NOZORDER);
    pWnd->SetForegroundWindow();
#endif    

    // start info for the checkbox and draw them
    int x_start     = (int)(winRect.BottomRight().x * 0.1);  // 10% of the lenght of the screen. (31 original)
    int y_start     = labelRec.BottomRight().y - offset + y_offset_from_label;
    int x_checkBox  = (int)(winRect.BottomRight().x * 0.40); // 40% of lenght of screen (210)
    
    for (int i = 0; i < customList.getCountItem(); i++) {        
        if (cbuttonArray.GetSize() > 0) {
            CButton* cb = cbuttonArray.GetAt(i);
            if (cb) {
                cb->SetWindowPos(GetActiveWindow(), x_start,
                                                  y_start,
                                                  x_checkBox,
                                                  y_checkbox_size,
                                                  SWP_NOZORDER);
                y_start += (y_checkbox_size + y_offset);

            }
        }

    }
    
    // position of the box with the cleanup in progress write
    pWnd = GetDlgItem(IDC_CLEANUP);
    pWnd->SetWindowPos(GetActiveWindow(), labelRec.TopLeft().x,
                                          y_start + y_offset_from_last_source,
                                          clientRect.BottomRight().x,
                                          y_checkbox_size,
                                          SWP_NOZORDER);
    
    if (cleanupStarted)   s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_CLEANUP_IN_PROG); 
    else                  s1 = L"";
    SetDlgItemText(IDC_CLEANUP, s1);
      
}
#endif

wstring getCommandLine(vector<const wchar_t*> &sources) {

    wstring sourceList;
    vector<const wchar_t*>::iterator sourcesIterator;
    int i = 1;
    for (sourcesIterator = sources.begin();
        sourcesIterator != sources.end();
        sourcesIterator++) {

            sourceList.append(*sourcesIterator);
            if (sources.size() - i != 0) {
                sourceList.append(TEXT(","));
            }
            i++;
    }

    return sourceList;
}

int checkRefreshFromServer(HWND wndUI, vector<CString> &sources,  BOOL cleanup) {
    
    wstring sourceList;
    int i = 1, ret = 0;
    wchar_t t[512];
    vector<CString>::iterator sourcesIterator;

    for (sourcesIterator = sources.begin();
            sourcesIterator != sources.end();
            sourcesIterator++) {

            sourceList.append(*sourcesIterator);
            if (sources.size() - i == 1) {
                sourceList.append(TEXT(" "));
                sourceList.append(getLocalizationUtils()->getLocalizationString(IDS_AND));
                sourceList.append(TEXT(" "));

            } else if (sources.size() - i != 0) {
                sourceList.append(TEXT(", "));
            }
            i++;
    }

    if (cleanup) {
        wsprintf(t, getLocalizationUtils()->getLocalizationString(IDS_CLEANUP_MSG),
            sourceList.c_str());
    } else {
        wsprintf(t, getLocalizationUtils()->getLocalizationString(IDS_RS_ARMSG1),
            sourceList.c_str());
    }

    wstring message = t;    

    if(cleanup){
        message.append(getLocalizationUtils()->getLocalizationString(IDS_CLEANUP_MSG3));
    }

    HwndFunctions::closePreviousMsgBox();
    int val = TimedMessageBox(wndUI, message.c_str(), getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
        MB_YESNO | MB_ICONQUESTION | MB_SETFOREGROUND, ASK_REFRESH_TIMEOUT * 1000); // default ASK_TIMEOUT = 10

    if (val == IDYES) {
        LOG.debug("User said yes to delete local %S and refresh.", sourceList.c_str());
    }
    else {
        LOG.debug("User said no to delete local %S and refresh." , sourceList.c_str());
        ret = -1;
    }

    return ret;
}

/**
* Message used after the clean up process
*/
int endCleanUpProcess(HWND wndUI, vector<CString> &labels) {

    wchar_t t[512];
    int i = 1;
    wstring sourceList;
    vector<CString>::iterator sourcesIterator;

    for (sourcesIterator = labels.begin();
        sourcesIterator != labels.end();
        sourcesIterator++) {

            sourceList.append(*sourcesIterator);

            if (labels.size() - i == 1) {
                sourceList.append(TEXT(" "));
                sourceList.append(getLocalizationUtils()->getLocalizationString(IDS_AND));
                sourceList.append(TEXT(" "));

            } else if (labels.size() - i != 0) {
                sourceList.append(TEXT(", "));
            }
            i++;
    }

    wsprintf(t, getLocalizationUtils()->getLocalizationString(IDS_CLEANUP_COMPLETE_MSG),
        sourceList.c_str());

    HwndFunctions::closePreviousMsgBox();
    int val = TimedMessageBox(wndUI, t, getLocalizationUtils()->getLocalizationString(IDS_FUNAMBOL_ALERT),
        MB_OK | MB_ICONINFORMATION | MB_SETFOREGROUND, ASK_REFRESH_TIMEOUT * 1000); // default ASK_TIMEOUT = 10

    return 0;
}

HBRUSH CRecoverSettings::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) {
    HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

    // set ppc header text color
#if defined(WIN32_PLATFORM_PSPC)
    if(pWnd->GetDlgCtrlID() == IDC_RECOVER_STATIC_TITLE) {
        pDC->SetBkMode(TRANSPARENT);
        pDC->SetTextColor(PPC_HEADER_TEXT_COLOR);
    }
#endif

    return hbr;
}

/**
* It disables all the checkbox to avoid they are pressed when the
* cleanup is in progress
*/
void CRecoverSettings::disableCheckbox() {
    
    if (cbuttonArray.GetSize() > 0) {   
        for (int i = 0; i < cbuttonArray.GetSize(); i++) {
            CButton* cb = cbuttonArray.GetAt(i);
            if (cb) {
                cb->EnableWindow(FALSE);
            }
        }
    }
}

void CRecoverSettings::enableCheckbox() {
    
    if (cbuttonArray.GetSize() > 0) {   
        for (int i = 0; i < cbuttonArray.GetSize(); i++) {
            CButton* cb = cbuttonArray.GetAt(i);
            if (cb) {
                int id = customList.indexToId(i);     
                cb->EnableWindow(customList.isEnabledItem(id));
            }
        }
    }

}

void CRecoverSettings::OnPaint() {

    // Set the cursor if cleanup is running or not: we could have switched to 
    // another window, so we set again the cursor.
    if (cleanupStarted)  SetCursor(LoadCursor(NULL, IDC_WAIT ));
    else                 SetCursor(LoadCursor(NULL, IDC_ARROW));

    CDialog::OnPaint();
}