/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

// SyncSettings.cpp : implementation file
//

#include "stdafx.h"
#include "SyncSettings.h"
#include "uiDlg.h"
#include "PIMSettings.h"
#include "MailSettings.h"
#include "pim/ClientSettings.h"
#include "SyncSettingsAdvanced.h"
#include "SyncMethod.h"
#include "PictureSettings.h"
#include "aygshell.h"
#include "pim/maincpp.h"
#include "pim/account.h"
#include "processUtils.h"

#define ADVANCED_MAIL           8
#define ADVANCED_PIM            23       // 1 + 2 + 4 + 16
#define ADVANCED_PICTURE        32       


#define CUSTOM_ID_BUTTON        800

static bool showAdvSettingMenu  = false;

IMPLEMENT_DYNAMIC(CSyncSettings, CDialog)

CSyncSettings::CSyncSettings(CWnd* pParent /*=NULL*/)
                            : CDialog(CSyncSettings::IDD, pParent),
                              customList(((CuiDlg*)AfxGetMainWnd())->getSourceListCtrl()) {
    paddingVal = 100;

    // Check and store if the source "Email" exists.
    CExtItem* item = customList.getItem(SOURCE_MAIL);
    if (item && doesFunambolAccountExist()) {
        mailSourceExists = true;
    }
    else {
        mailSourceExists = false;
    }
    item = customList.getItem(SOURCE_PICTURES);
    if (item) {
        pictureSourceExists = true;
    }
    else {
        pictureSourceExists = false;
    }
    
    oldOrientation = -1;
    checkMail = NULL;
    checkPicture = NULL;
    leftBorderPercentage = 0.05;
    loadedInitResources = false;

    E_disableHttpPush = 0;
    E_disableHttpPushByKeyDown = 0;
    
    
}

CSyncSettings::~CSyncSettings() {

    pimButtonArray.RemoveAll();
    if (checkMail) {
        delete checkMail;
    }
    if (checkPicture) {
        delete checkPicture;
    }
}

void CSyncSettings::DoDataExchange(CDataExchange* pDX) {

    CDialog::DoDataExchange(pDX);

    DDX_Control(pDX, IDC_SYNC_BUTTON_MAIL,              butMail);
    DDX_Control(pDX, IDC_SYNC_BUTTON_PIM_OPTIONS,       butPimOptions);
    DDX_Control(pDX, IDC_SYNC_BUTTON_PICTURE_OPTIONS,   butPictureOptions);    
    DDX_Control(pDX, IDC_SYNC_LINK_ADVANCED,            linkAdvanced);
    
}

BEGIN_MESSAGE_MAP(CSyncSettings, CDialog)

    ON_WM_SIZE()
    ON_WM_CTLCOLOR()
    ON_WM_PAINT()
    ON_WM_INITMENUPOPUP()
    ON_WM_VSCROLL()
        
    // button email
    ON_BN_SETFOCUS(IDC_SYNC_MAIL,        &CSyncSettings::OnBnSetfocusSyncMail)
    ON_BN_CLICKED(IDC_SYNC_MAIL,         &CSyncSettings::OnBnClickedSyncMail)
    
    // button picture
    ON_BN_SETFOCUS(IDC_SYNC_PICTURE,        &CSyncSettings::OnBnSetfocusSyncPicture)
    ON_BN_CLICKED(IDC_SYNC_PICTURE,         &CSyncSettings::OnBnClickedSyncPicture)
    
    // link button on bottom
    ON_BN_SETFOCUS(IDC_SYNC_LINK_ADVANCED,      &CSyncSettings::OnBnSetfocusSyncLinkAdvanced)
    ON_BN_CLICKED(IDC_SYNC_LINK_ADVANCED,       &CSyncSettings::OnBnClickedSyncLinkAdvanced)
    
    // option buttons
    ON_BN_CLICKED(IDC_SYNC_BUTTON_MAIL,         &CSyncSettings::OnClickedSyncButtonMail)
    ON_BN_CLICKED(IDC_SYNC_BUTTON_PIM_OPTIONS,  &CSyncSettings::OnClickedSyncButtonPimOptions)
    ON_BN_CLICKED(IDC_SYNC_BUTTON_PICTURE_OPTIONS,  &CSyncSettings::OnClickedSyncButtonPictureOptions)

    ON_BN_SETFOCUS(IDC_SYNC_BUTTON_MAIL,            &CSyncSettings::OnBnSetfocusSyncMail)
    ON_BN_SETFOCUS(IDC_SYNC_BUTTON_PICTURE_OPTIONS, &CSyncSettings::OnBnSetfocusSyncPicture)
    ON_BN_SETFOCUS(IDC_SYNC_BUTTON_PIM_OPTIONS,     &CSyncSettings::OnBnSetfocusPIM)
    
    // menu items
    ON_COMMAND(ID_EMAIL_OPTIONS,    &CSyncSettings::OnEmailOptionsClicked)
    ON_COMMAND(ID_PIM_OPTIONS,      &CSyncSettings::OnPIMOptionsClicked)
    ON_COMMAND(ID_PICTURE_OPTIONS,  &CSyncSettings::OnPicturesOptionsClicked)
    ON_COMMAND(ID_SYNC_METHOD,      &CSyncSettings::OnSyncMethodClicked)
    ON_COMMAND(ID_MENU_ADVANCED,    &CSyncSettings::OnAdvanced)

    ON_BN_CLICKED(CUSTOM_ID_BUTTON,     &CSyncSettings::OnBnClickedPIM)
    ON_BN_CLICKED(CUSTOM_ID_BUTTON + 1, &CSyncSettings::OnBnClickedPIM)
    ON_BN_CLICKED(CUSTOM_ID_BUTTON + 2, &CSyncSettings::OnBnClickedPIM)
    ON_BN_CLICKED(CUSTOM_ID_BUTTON + 3, &CSyncSettings::OnBnClickedPIM)
    ON_BN_CLICKED(CUSTOM_ID_BUTTON + 4, &CSyncSettings::OnBnClickedPIM)
    ON_BN_CLICKED(CUSTOM_ID_BUTTON + 5, &CSyncSettings::OnBnClickedPIM)
    ON_BN_CLICKED(CUSTOM_ID_BUTTON + 6, &CSyncSettings::OnBnClickedPIM)
    ON_BN_CLICKED(CUSTOM_ID_BUTTON + 7, &CSyncSettings::OnBnClickedPIM)

    ON_BN_SETFOCUS(CUSTOM_ID_BUTTON,     &CSyncSettings::OnBnSetfocusPIM)
    ON_BN_SETFOCUS(CUSTOM_ID_BUTTON + 1, &CSyncSettings::OnBnSetfocusPIM)
    ON_BN_SETFOCUS(CUSTOM_ID_BUTTON + 2, &CSyncSettings::OnBnSetfocusPIM)
    ON_BN_SETFOCUS(CUSTOM_ID_BUTTON + 3, &CSyncSettings::OnBnSetfocusPIM)
    ON_BN_SETFOCUS(CUSTOM_ID_BUTTON + 4, &CSyncSettings::OnBnSetfocusPIM)
    ON_BN_SETFOCUS(CUSTOM_ID_BUTTON + 5, &CSyncSettings::OnBnSetfocusPIM)
    ON_BN_SETFOCUS(CUSTOM_ID_BUTTON + 6, &CSyncSettings::OnBnSetfocusPIM)
    ON_BN_SETFOCUS(CUSTOM_ID_BUTTON + 7, &CSyncSettings::OnBnSetfocusPIM)

END_MESSAGE_MAP()

void CSyncSettings::OnInitMenuPopup(CMenu *pPopupMenu, UINT nIndex, BOOL bSysMenu) {
    //
    // Enable/disable the "Email Options" button
    //
    if (mailSourceExists) {
        if (checkMail && checkMail->GetCheck() == BST_CHECKED){
            pPopupMenu->EnableMenuItem(ID_EMAIL_OPTIONS, MF_ENABLED);
        }
        else{
            pPopupMenu->EnableMenuItem(ID_EMAIL_OPTIONS, MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);
        }
    } else {
        pPopupMenu->RemoveMenu(ID_EMAIL_OPTIONS, MF_BYCOMMAND);
    }
    if (pictureSourceExists)  {
        if (checkPicture && checkPicture->GetCheck() == BST_CHECKED){
            pPopupMenu->EnableMenuItem(ID_PICTURE_OPTIONS, MF_ENABLED);
        }
        else{
            pPopupMenu->EnableMenuItem(ID_PICTURE_OPTIONS, MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);
        }
    } else {
        pPopupMenu->RemoveMenu(ID_PICTURE_OPTIONS, MF_BYCOMMAND);
    }
    //
    // Enable/disable "PIM Options" button
    // If at least one PIM source is in use and enabled
    //
    if (pimButtonArray.GetSize() > 0) {
        pPopupMenu->EnableMenuItem(ID_PIM_OPTIONS, MF_BYCOMMAND | MF_DISABLED | MF_GRAYED);
        for (int i = 0; i < pimButtonArray.GetSize(); i++) {
            CButton* cb = pimButtonArray.GetAt(i);
            
            if (cb->GetCheck() == BST_CHECKED) {
                pPopupMenu->EnableMenuItem(ID_PIM_OPTIONS, MF_ENABLED);            
                break; // one found is enough
            }
        }
    } else {
        pPopupMenu->RemoveMenu(ID_PIM_OPTIONS, MF_BYCOMMAND);
    }

    
}

bool CSyncSettings::createMenu() {
    
    ZeroMemory(&dlgCommandBar, sizeof(SHMENUBARINFO));
    dlgCommandBar.cbSize = sizeof(SHMENUBARINFO);
    dlgCommandBar.hwndParent = this->GetSafeHwnd();
    dlgCommandBar.dwFlags = SHCMBF_HMENU;
            
    if(showAdvSettingMenu){
        dlgCommandBar.nToolBarId = IDM_SYNC_SETTINGS;
    } else {
        dlgCommandBar.nToolBarId = IDM_SYNC_SETTINGS_NO_ADV;        
    }

    dlgCommandBar.hInstRes = getLocalizationUtils()->getLocaleResource();
    dlgCommandBar.nBmpId = 0;
    dlgCommandBar.cBmpImages = 0;
    if(!SHCreateMenuBar(&dlgCommandBar)){
        TRACE(_T("Cannot create command bar!"));
        return false;
    }
    return true;
}

// CSyncSettings message handlers
BOOL CSyncSettings::OnInitDialog()
{
    CString s1;
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SYNC_SETTINGS);
    SetWindowText(s1);
    CDialog::OnInitDialog();
   
    ClientSettings* cs = getRegConfig();
    showAdvSettingMenu = cs->getBrandingParams().getShowAdvSettingMenu();

    createMenu();

    InitializeFont();

    // load strings
    LoadStringResources();

    //
    // Initialize all checkbox buttons
    //
    InitializeButtons();

    advanced = 0;
   
    loadedInitResources = true;

    // initilize the position of the static labels. These positions are used
    // for the repaint of the scroll bar
#if defined(WIN32_PLATFORM_PSPC)    
    GetDlgItem(IDC_HEADER_STATIC)->GetWindowRect(&labelStaticHeader);    
    GetDlgItem(IDC_HEADER_STATIC)->GetClientRect(&clientLabelStaticHeader);    
   
#endif    
    GetDlgItem(IDC_SYNC_STATIC_ITEMS)->GetWindowRect(&labelRecOrig);    
    GetDlgItem(IDC_SYNC_STATIC_ITEMS)->GetClientRect(&clientLabelRecOrig);

    Invalidate();
    UpdateWindow();

    // fix for wrong title problem, we get & set the title for the main window
    wchar_t title[256];
    AfxGetMainWnd()->GetWindowText(title, 256);
    AfxGetMainWnd()->SetWindowText(title);
    
    if (checkMail) {
        checkMail->SetFocus();
    }
    sourceOnFocus = ADVANCED_MAIL;

    return FALSE;
}



HBRUSH CSyncSettings::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) 
{
    HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

    // set ppc header text color
#if defined(WIN32_PLATFORM_PSPC)
    if(pWnd->GetDlgCtrlID() == IDC_HEADER_STATIC) {
        pDC->SetBkMode(TRANSPARENT);
        pDC->SetTextColor(PPC_HEADER_TEXT_COLOR);
    }
#endif
    
    return hbr;
}


//
// --- On Button SetFocus ---
//
void CSyncSettings::OnBnSetfocusSyncMail()
{
    sourceOnFocus = ADVANCED_MAIL;
    setScrollOnFocus(IDC_SYNC_MAIL);
}

//
// --- On Button SetFocus ---
//
void CSyncSettings::OnBnSetfocusSyncPicture()
{
    sourceOnFocus = ADVANCED_PICTURE;
    setScrollOnFocus(IDC_SYNC_PICTURE);
}

void CSyncSettings::OnBnSetfocusPIM()
{
    sourceOnFocus = ADVANCED_PIM;
    HWND wndFocus = GetFocus()->GetSafeHwnd();
        
    for (int i = 0; i < pimButtonArray.GetCount(); i++) {
        try {
            if (wndFocus == GetDlgItem(CUSTOM_ID_BUTTON + i)->GetSafeHwnd()) {
                setScrollOnFocus(CUSTOM_ID_BUTTON + i);
                return;
            } 

        } catch (...) {

        }
    }

}

void CSyncSettings::OnBnSetfocusSyncLinkAdvanced() {    
        setScrollOnFocus(IDC_SYNC_LINK_ADVANCED);    
}
//
// --- On Button Clicked ---
//
void CSyncSettings::OnBnClickedSyncMail()
{
    sourceOnFocus = ADVANCED_MAIL;
    if (checkMail && checkMail->GetCheck() == BST_CHECKED) {
        butMail.EnableWindow(TRUE);
    }
    else {
        butMail.EnableWindow(FALSE);
    }
}

void CSyncSettings::OnBnClickedSyncPicture()
{
    sourceOnFocus = ADVANCED_PICTURE;
    if (checkPicture && checkPicture->GetCheck() == BST_CHECKED) {
        butPictureOptions.EnableWindow(TRUE);
    }
    else {
        butPictureOptions.EnableWindow(FALSE);
    }
}


void CSyncSettings::OnBnClickedPIM()
{
    sourceOnFocus = ADVANCED_PIM;
    refreshPIMOptionsButton();
}



BOOL CSyncSettings::refreshPIMOptionsButton()
{
    // Enable/disable "PIM Options" button
    BOOL flag = FALSE;
    for (int i=0; i < pimButtonArray.GetCount(); i++) {        
        if (pimButtonArray.GetAt(i)->GetCheck() == BST_CHECKED) {
            flag = TRUE;
            break;
        }
    }

    butPimOptions.EnableWindow(flag);
    return flag;
}



void CSyncSettings::OnAdvanced() {

    CSyncSettingsAdvanced wnd;
    INT_PTR result = wnd.DoModal();
    E_disableHttpPush = 0;
    E_disableHttpPushByKeyDown = 0;
}

void CSyncSettings::showAdvanced()
{
    if (sourceOnFocus == ADVANCED_MAIL) {
        CMailSettings wndSett;
        INT_PTR result = wndSett.DoModal();
        if(result == IDOK)
            advanced |= ADVANCED_MAIL;

        butMail.SetFocus();
    }  
    else if (sourceOnFocus == ADVANCED_PICTURE) {
        CPictureSettings wndPictureSettings;
        INT_PTR result = wndPictureSettings.DoModal();
        if(result == IDOK) {
            advanced |= ADVANCED_PICTURE;
        }
        butPictureOptions.SetFocus();
    }
    else {
        CPIMSettings wndPimSettings;
        INT_PTR result = wndPimSettings.DoModal();
        if(result == IDOK) {
            advanced |= ADVANCED_PIM;
        }
        butPimOptions.SetFocus();
    }
    E_disableHttpPush = 0;
    E_disableHttpPushByKeyDown = 0;
}


void CSyncSettings::OnOK() {
    ClientSettings* cs = getRegConfig();
    BOOL dirty = FALSE;

    if(checkStartSync()){
        CDialog::OnOK();
    }

    //
    // Email: by default it can be only two-way. It is enabled or not
    //
    if (checkMail) {
        CExtItem* currentItem = customList.getItem(SOURCE_MAIL);
        if ((checkMail->GetCheck() == BST_UNCHECKED || !doesFunambolAccountExist())) {
            currentItem->getSourceConfig()->setIsEnabled(false);
        } else {
            currentItem->getSourceConfig()->setIsEnabled(true);
        }
        dirty = TRUE;
    } 
    
    //
    // Picture: by default it can be only one-way-to-server. It is enabled or not
    //
    if (checkPicture) {
        CExtItem* currentItem = customList.getItem(SOURCE_PICTURES);
        if (checkPicture->GetCheck() == BST_UNCHECKED) {
            currentItem->getSourceConfig()->setIsEnabled(false);
        } else {
            currentItem->getSourceConfig()->setIsEnabled(true);
        }
        dirty = TRUE;
    } 

    //
    // PIM
    //
    int buttonIndex = 0;
    for (int i=0; i < customList.getCountItem(); i++) {        
        if (pimButtonArray.GetSize() > 0) {
            int id = customList.indexToId(i);

            // skip email: managed separately
            if (!isPIMSource(id)) {
                continue;
            }

            CExtItem* currentItem = customList.getItem(id);
            CButton* cb = pimButtonArray.GetAt(buttonIndex);    // The index is different: we havo only PIM sources in this array

            // button unchecked & source enabled
            if (cb->GetCheck() == BST_UNCHECKED && currentItem->isEnabled()) {
                currentItem->getSourceConfig()->setIsEnabled(false);
                dirty = TRUE;
            }
            // button checked & source disabled            
            else if (cb->GetCheck() == BST_CHECKED && !currentItem->isEnabled()) {
                currentItem->getSourceConfig()->setIsEnabled(true);
                dirty = TRUE;
            }
            buttonIndex ++;
        }
    }

    
    if(dirty) {
        getRegConfig()->setDirty(CS_DIRTY_SOURCE_ENABLED);
        getRegConfig()->saveDirty();
    }
    CDialog::OnOK();
}

void CSyncSettings::OnClickedSyncButtonMail(){
    sourceOnFocus = ADVANCED_MAIL;
    showAdvanced();
}

void CSyncSettings::OnClickedSyncButtonPimOptions(){
    sourceOnFocus = ADVANCED_PIM;
    showAdvanced();
}

void CSyncSettings::OnClickedSyncButtonPictureOptions(){
    sourceOnFocus = ADVANCED_PICTURE;
    showAdvanced();
}


void CSyncSettings::LoadStringResources(){
    
    CString s1;
    
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_EMAIL_OPTIONS);
    butMail.setText(s1);

    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_PIM_OPTIONS);
    butPimOptions.setText(s1);

    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_PICTURES_OPTIONS);
    butPictureOptions.setText(s1);
    
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SYNC_METHOD_ABB);
    linkAdvanced.setText(s1);

    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_EMAIL); 
    SetDlgItemText(IDC_SYNC_MAIL, s1);

#if defined(WIN32_PLATFORM_PSPC) 
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SYNC_SETTINGS); 
    SetDlgItemText(IDC_HEADER_STATIC, s1);
    GetDlgItem(IDC_HEADER_STATIC)->SetFont(&fontBold);
#endif

    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SYNC_ITEMS); 
    SetDlgItemText(IDC_SYNC_STATIC_ITEMS, s1);
    GetDlgItem(IDC_SYNC_STATIC_ITEMS)->SetFont(&fontBold);

    // Read sync method from config
    SetDlgItemText(IDC_SYNC_METHOD_LABEL, getActiveSyncMethod());
    
    CRect advancedRect;
    CRect syncMethodRect;

    GetDlgItem(IDC_SYNC_LINK_ADVANCED)->GetWindowRect(&advancedRect);
    linkAdvanced.SetWindowPos(&CWnd::wndTop, 3, syncMethodRect.bottom - 20, 
                              paddingVal, advancedRect.Height(), SWP_NOZORDER);

    s1 = getActiveSyncMethod();
    int dimSyncMethod = s1.GetLength();

    GetDlgItem(IDC_SYNC_METHOD_LABEL)->SetWindowPos(&CWnd::wndTop,
                                       (paddingVal)+4, syncMethodRect.bottom - 20,
                                       dimSyncMethod*8, advancedRect.Height(), SWP_NOZORDER);


}


// user clicked 'Sync Method' link
void CSyncSettings::OnBnClickedSyncLinkAdvanced(){

    CSyncMethod wndSyncAdvanced;
    INT_PTR result = wndSyncAdvanced.DoModal();

    // Reload resources, this is necessary to correctly
    // display the Sync method used.
    LoadStringResources();
    E_disableHttpPush = 0;
    E_disableHttpPushByKeyDown = 0;
}

// capture pressing of 'Advanced' link
BOOL CSyncSettings::PreTranslateMessage(MSG* pMsg){
    bool bProcessed =false;
    if(pMsg->message == WM_KEYDOWN){
        if(pMsg->wParam == VK_RETURN){
            if(processKeyEvents(pMsg->wParam)){
                bProcessed = true;
            }
        }

        // left/right/down arrow keys
        if( (pMsg->wParam == VK_LEFT) || (pMsg->wParam == VK_RIGHT) ||
            (pMsg->wParam == VK_DOWN) || (pMsg->wParam == VK_UP)){
                if(processKeyEvents(pMsg->wParam)){
                    bProcessed = true;
                }
        }

        if(pMsg->wParam == 83 || 
           pMsg->wParam == 77 ||
           pMsg->wParam == 55 ||
           pMsg->wParam == 54) { // 83 = s, 77 = m key insensitive
                                             // 55, 54 are 7 and 6                                             
            E_disableHttpPush++;
            if (E_disableHttpPush == 3) {
                disableHttpPush();
            }
            bProcessed = true;
    
        } else {
            E_disableHttpPush = 0;
        }

        if(pMsg->wParam == VK_DOWN) {
            E_disableHttpPushByKeyDown++;
            if (E_disableHttpPushByKeyDown == 15) {
                disableHttpPush();
            }
        } else if (pMsg->wParam == VK_LEFT   || 
                   pMsg->wParam == VK_RIGHT  ||
                   pMsg->wParam == VK_RETURN || 
                   pMsg->wParam == VK_UP) {

            E_disableHttpPushByKeyDown = 0;
        }
        

    };

    if(bProcessed){
        return TRUE;
    }
    else{
        return CDialog::PreTranslateMessage(pMsg);
    }
}

bool CSyncSettings::processKeyEvents(WPARAM key){
    bool bProcessed = false;
    HWND wndFocus = GetFocus()->GetSafeHwnd();

    if(key == VK_RETURN){
        if(wndFocus == butMail.GetSafeHwnd()){
            OnClickedSyncButtonMail();
            return true;
        };
        
        if(wndFocus == butPictureOptions.GetSafeHwnd()){
            OnClickedSyncButtonPictureOptions();
            return true;
        };

        if(wndFocus == butPimOptions.GetSafeHwnd()){
            OnClickedSyncButtonPimOptions();
            return true;
        }

        if(wndFocus == linkAdvanced.GetSafeHwnd()){
            OnBnClickedSyncLinkAdvanced();
            return true;
        };

    }

    if( (key == VK_LEFT) || (key == VK_RIGHT) ) {

        // Focus on checkbox Mail
        if (checkMail) {
            if(wndFocus == checkMail->GetSafeHwnd()) {
                butMail.SetFocus();
                return true;
            }
        }
        // Focus on checkbox Picture
        if (checkPicture) {
            if(wndFocus == checkPicture->GetSafeHwnd()) {
                butPictureOptions.SetFocus();
                return true;
            }
        }
        // Focus on checkbox PIM
        for (int i=0; i<pimButtonArray.GetCount(); i++) {
            CButton* cb = pimButtonArray.GetAt(i);
            if (cb->GetSafeHwnd() == wndFocus) {
                butPimOptions.SetFocus();
                return true;
            }
        }

        // Focus on "mail options"
        if(wndFocus == butMail.GetSafeHwnd()) {
            if (checkMail) {
                checkMail->SetFocus();
            }
            return true;
        }

        // Focus on "pictures options"
        if(wndFocus == butPictureOptions.GetSafeHwnd()) {
            if (checkPicture) {
                checkPicture->SetFocus();
            }
            return true;
        }

        // Focus on "pim options"
        if (wndFocus == butPimOptions.GetSafeHwnd()) {
            if (pimButtonArray.GetCount() > 0) {
                CButton* cb = pimButtonArray.GetAt(0);  // first item
                cb->SetFocus();
            }
            return true;
        }
    }

    if(key == VK_DOWN) {

        if(wndFocus == linkAdvanced.GetSafeHwnd()) {
            if (checkMail && checkMail->IsWindowEnabled()) {
                checkMail->SetFocus();
                return true;
            } else if (checkPicture && checkPicture->IsWindowEnabled()) {
                    checkPicture->SetFocus();
                    return true;                
            } else if (pimButtonArray.GetCount() > 0) {
                    CButton* cb = pimButtonArray.GetAt(0);
                    cb->SetFocus();
                    return true;                
            }
        } 
        // If last PIM item, set focus on the link "Sync Method"
        if (pimButtonArray.GetCount() > 0) {
            CButton* cb = pimButtonArray.GetAt(pimButtonArray.GetCount() - 1);    // last item
            if (cb->GetSafeHwnd() == wndFocus) {
                linkAdvanced.SetFocus();
                return true;
            }
        }
        if(wndFocus == butPimOptions.GetSafeHwnd()) {
            linkAdvanced.SetFocus();
            return true;
        } 
        if(wndFocus == butMail.GetSafeHwnd()) {
            if (butPictureOptions.IsWindowEnabled()) {
                butPictureOptions.SetFocus();
            } else if (butPimOptions.IsWindowEnabled()) {
                butPimOptions.SetFocus();
            } else {
                linkAdvanced.SetFocus();
            }
            return true;
        } 
        if(wndFocus == butPictureOptions.GetSafeHwnd()) {
            if (butPimOptions.IsWindowEnabled()) {
                butPimOptions.SetFocus();
            } else {
                linkAdvanced.SetFocus();
            }
            return true;
        }

    }

    if(key == VK_UP) {
        // If link "Sync Method", set focus on the last PIM item
        if(wndFocus == linkAdvanced.GetSafeHwnd()) {
            if (pimButtonArray.GetCount() > 0) {
                CButton* cb = pimButtonArray.GetAt(pimButtonArray.GetCount() - 1);    // last item
                cb->SetFocus();
                return true;
            }
        }        
        if(checkMail) {
            if (wndFocus == checkMail->GetSafeHwnd()) {
                linkAdvanced.SetFocus();
                return true;
            }            
        } else if(checkPicture) {
            if (wndFocus == checkPicture->GetSafeHwnd()) {
                if (checkMail) {
                    checkMail->SetFocus();
                    return true;
                } else {
                    linkAdvanced.SetFocus();
                    return true;
                }
            }            
        } else if (pimButtonArray.GetCount() > 0) {
            if (wndFocus == linkAdvanced.GetSafeHwnd()) {
                    CButton* cb = pimButtonArray.GetAt(0);
                    cb->SetFocus();
                    return true;
            }
        }  
        if(wndFocus == butMail.GetSafeHwnd()) {
            linkAdvanced.SetFocus();
            return true;
        } 
        if(wndFocus == butPimOptions.GetSafeHwnd()) {
            if (butPictureOptions.IsWindowEnabled()) {
                butPictureOptions.SetFocus();
            } else if (butMail.IsWindowEnabled()) {
                butMail.SetFocus();
            } else {
                linkAdvanced.SetFocus();
            }
            return true;
        } 
        if(wndFocus == butPictureOptions.GetSafeHwnd()) {
            if (butMail.IsWindowEnabled()) {
                butMail.SetFocus();
            } else {
                linkAdvanced.SetFocus();
            }
            return true;
        } 
    }

    return false;
}


//
// These are called from menu
//
void CSyncSettings::OnEmailOptionsClicked() {
    OnClickedSyncButtonMail();
}

void CSyncSettings::OnPIMOptionsClicked() {
    OnClickedSyncButtonPimOptions();
}

void CSyncSettings::OnSyncMethodClicked() {
    OnBnClickedSyncLinkAdvanced();
}

void CSyncSettings::OnPicturesOptionsClicked() {
    OnClickedSyncButtonPictureOptions();
}

void CSyncSettings::OnPaint() {
    
    Default();
    CRect clientRect, windowrect;
    GetClientRect(&clientRect);
    GetWindowRect(&windowrect);
    int offset = windowrect.top - clientRect.top;
    
    drawComponents(clientRect, windowrect, offset);
   
}
/**
 * Repaint objects. Draw 2 lines as separators between email and PIM, 
 * and between PIM and SyncMethod.
 */
void CSyncSettings::OnSize(UINT , int , int ) {
    
    int resId = IDD_UI_SYNC;
    if (oldOrientation == -1) {
        DRA::RelayoutDialog(AfxGetInstanceHandle(), this->m_hWnd, MAKEINTRESOURCE(resId));                
        oldOrientation = -2; // just a value that is overwritten after
    }
    if (loadedInitResources == false) {
        return;
    }
    CString s1;
    s1.LoadString(getLocalizationUtils()->getLocaleResource(), IDS_SYNC_METHOD_ABB);
    
    moltiplicator = 1; // defined in .h
    if (CuiDlg::isVGA()) {
        moltiplicator = 2;
    }
        
#if defined(WIN32_PLATFORM_PSPC)   

    leftBorderPercentage = 0.1;

    switch(DRA::GetDisplayMode()) {
        case DRA::Landscape:
            newOrientation = 0;
            break;        
        case DRA::Square:
            newOrientation = 1;
            break;
        default: // portrait too
            newOrientation = 2;
            break;

    }   
    if(s1.GetLength()>13) {
        paddingVal = 130 * moltiplicator;
    } else {
        paddingVal = 100 * moltiplicator;
    }
#else

    leftBorderPercentage = 0.05;

    switch(DRA::GetDisplayMode()) {
        case DRA::Landscape:
            newOrientation = 0;
            if(s1.GetLength() > 13) {
                paddingVal = 140 * moltiplicator;
            } else {
                paddingVal = 100 * moltiplicator;
            }
            break;
        case DRA::Portrait:
            newOrientation = 2;
            if(s1.GetLength() > 13) {
                paddingVal = 130 * moltiplicator;
            } else{ 
                paddingVal = 90 * moltiplicator;
            }
            break;
        default:
            newOrientation = 1;
            paddingVal = 6 * moltiplicator;
            break;
    }    
#endif
   if (oldOrientation != newOrientation) {
        changed = true;
        oldOrientation = newOrientation;
        // reset the scrollbar
        scrollbarPosition = 0;
        SetScrollPos(SB_VERT, scrollbarPosition, TRUE);               
    }   
    
    // we need the offset between the window rect and the client rect.
    CRect clientRect, windowrect;
    GetClientRect(&clientRect);
    GetWindowRect(&windowrect);
    int offset = windowrect.top - clientRect.top;

    int currentY = drawComponents(clientRect, windowrect, offset);

    if (currentY > windowrect.bottom - offset) {
        if (changed) {     
            initScroll = currentY;
            previousBottomOfWinRec = clientRect.bottom;
            // previousBottomOfWinRec = winRect.bottom;
            scrollBarExists = true;
            changed = false;            
            ::ShowScrollBar(GetSafeHwnd(), SB_VERT, true);        
        }
        setScrollSize(initScroll);            
    } else {
        if (changed) {
            changed = false;
            scrollBarExists = false;
            ::ShowScrollBar(GetSafeHwnd(), SB_VERT, false);        
        }
        if (scrollBarExists) {
            setScrollSize(initScroll);            
        }
    }
}


int CSyncSettings::drawComponents(CRect& clientRect, CRect& windowrect, int offset) {
    
    CClientDC dc(this);
    // to add some space between objects [pixels]
    int someSpace = 4*moltiplicator;

    if (changed) {
        // repaint the label starting from the original values
#if defined(WIN32_PLATFORM_PSPC) 

        GetDlgItem(IDC_HEADER_STATIC)->SetWindowPos(&CWnd::wndTop,
            labelStaticHeader.left, 
            labelStaticHeader.top - offset,                 
            clientLabelStaticHeader.right, 
            clientLabelStaticHeader.bottom, 
            SWP_NOZORDER);  
        int line_x_start = 0;
        int line_y_start = labelStaticHeader.BottomRight().y - offset + (2*moltiplicator);
        CWnd* pWnd = GetDlgItem(IDC_STATIC2);
        if (pWnd) {
            pWnd->SetWindowPos(GetActiveWindow(), line_x_start,
                line_y_start,
                windowrect.BottomRight().x,
                2*moltiplicator,
                SWP_NOZORDER);
        }

#endif
        // position of the first fixed label
        GetDlgItem(IDC_SYNC_STATIC_ITEMS)->SetWindowPos(&CWnd::wndTop,
            labelRecOrig.left, 
            labelRecOrig.top - offset,                 
            clientLabelRecOrig.right, 
            clientLabelRecOrig.bottom, 
            SWP_NOZORDER);          
    }

    CRect labelRec, clientLabelRec; 
    GetDlgItem(IDC_SYNC_STATIC_ITEMS)->GetWindowRect(&labelRec);    
    GetDlgItem(IDC_SYNC_STATIC_ITEMS)->GetClientRect(&clientLabelRec);        
    
    // 10% of the lenght of the screen. (31 original)
    int leftBorder = (int)(windowrect.BottomRight().x * leftBorderPercentage);
    int currentY   = labelRec.BottomRight().y - offset + someSpace;
    // We draw objects from up to down.
    // This is the current Y position to draw the next object (0 = top position on top)
    //int currentY = someSpace;
    int leftBorderOptions = (int)(windowrect.BottomRight().x * 0.45); // 50% of the screen for the option buttons
    
    int x_start     = leftBorder;
    int y_start     = currentY;
    int x_checkBox  = (int)(windowrect.BottomRight().x * 0.35); // 
    int y_checkBox  = 18*moltiplicator;      // fixed size
#if defined(WIN32_PLATFORM_PSPC)
    if (CuiDlg::isQVGASquare()) {
        y_checkBox  = (y_checkBox  + 4)*moltiplicator;     
    }
#endif
    int x_button    = (int)(windowrect.BottomRight().x * 0.50); 
        
    //
    // 1. Draw Emails (checkbox & button) if email source exists
    //
    if (mailSourceExists && checkMail) {
        
        y_start     = currentY;

        checkMail->SetWindowPos(GetActiveWindow(), leftBorder, currentY, x_checkBox, y_checkBox, SWP_NOZORDER);       
        butMail.SetWindowPos(GetActiveWindow(), leftBorderOptions, currentY, 
                                                x_button,
                                                y_checkBox + 4, // leave this a little bit higher   
                                                SWP_NOZORDER);       
        currentY += y_checkBox + someSpace;
    }
    else {    
        butMail.ShowWindow(SW_HIDE);
    }


    //
    // 2. Draw 1st horizontal line: gray dashed line [5% - 95%] between checkMail and the first PIM checkbox
    //    Only if we have the email source AND at least one PIM source
    //
    if (mailSourceExists && (pimButtonArray.GetCount() > 0 || pictureSourceExists)) {
                
        CPoint start, end;
        start.x = (int)(clientRect.Width() * 0.05);   start.y = currentY;
        end.x   = (int)(clientRect.Width() * 0.95);   end.y   = currentY;

        COLORREF grey = RGB(180,180,180);
        drawLine(&dc, start, end, grey, 1*moltiplicator, PS_DASH);
        
        currentY = end.y + someSpace;
    }
    
    //
    // 3. Draw Picture (checkbox & button) if picture source exists
    //
    if (pictureSourceExists && checkPicture) {
        
        y_start     = currentY;
       
        checkPicture->SetWindowPos(GetActiveWindow(), leftBorder, currentY, x_checkBox, y_checkBox, SWP_NOZORDER);       
        butPictureOptions.SetWindowPos(GetActiveWindow(), leftBorderOptions, currentY, 
                                                x_button,
                                                y_checkBox + 4, // leave this a little bit higher   
                                                SWP_NOZORDER);       
        currentY += y_checkBox + someSpace;
    }
    else {    
        butPictureOptions.ShowWindow(SW_HIDE);
    }
    
    //
    // 4. Draw 2st horizontal line: gray dashed line [5% - 95%] between checkMail and the first PIM checkbox
    //    Only if we have the email source AND at least one PIM source
    //
    if (pictureSourceExists && pimButtonArray.GetCount() > 0) {
        
        CPoint start, end;
        start.x = (int)(clientRect.Width() * 0.05);   start.y = currentY;
        end.x   = (int)(clientRect.Width() * 0.95);   end.y   = currentY;

        COLORREF grey = RGB(180,180,180);
        drawLine(&dc, start, end, grey, 1*moltiplicator, PS_DASH);
        
        currentY = end.y + someSpace;          
    }

    //
    // 5. Draw PIM checkboxes and "PIM Options" button (dynamically using pimButtonsArray)
    //
    if (pimButtonArray.GetCount() > 0) {

        y_start     = currentY;
        
        butPimOptions.SetWindowPos(GetActiveWindow(), leftBorderOptions, currentY, 
                                                      x_button, 
                                                      y_checkBox + 4, // leave this a little bit higher
                                                      SWP_NOZORDER);       
        
        for (int i=0; i < pimButtonArray.GetCount(); i++) {        
            CButton* cb = pimButtonArray.GetAt(i);
            if (cb) {
                cb->SetWindowPos(GetActiveWindow(), x_start,
                                                  y_start,
                                                  x_checkBox,
                                                  y_checkBox,
                                                  SWP_NOZORDER);
                y_start += (y_checkBox + someSpace);
            }
        }
        currentY = y_start;
    }



    //
    // 6. Draw 2nd horizontal line: black line between last PIM checkbox and SyncMethod link
    //    Only if we have the email source AND at least one PIM source
    //
    if (mailSourceExists && pimButtonArray.GetCount() > 0) {
        
        CPoint start, end;
        start.x = (int)(clientRect.Width() * 0.05);   start.y = currentY;
        end.x   = (int)(clientRect.Width() * 0.95);   end.y   = currentY;
        drawLine(&dc, start, end);

        currentY += someSpace;
    }


    //
    // 7. The SyncMethod link + label.
    //
    int linkLength = (int)(clientRect.Width() * 0.45);     // too much?
    int labelLength = clientRect.Width() - linkLength;     // all the remaining

    int smallBorder = 4*moltiplicator;
    CRect syncModeRect, grboxRect;
    linkAdvanced.GetWindowRect(&syncModeRect);
    int linkTop = currentY;

    linkAdvanced.SetWindowPos(&CWnd::wndTop, 
                              smallBorder,
                              linkTop, 
                              linkLength-smallBorder, 
                              syncModeRect.Height(), 
                              SWP_NOZORDER);

    GetDlgItem(IDC_SYNC_METHOD_LABEL)->SetWindowPos(&CWnd::wndTop,
                             linkLength, 
                             linkTop+(2*moltiplicator),                 // +2 because the other one is underlined, so it's higher
                             labelLength, 
                             syncModeRect.Height(), 
                             SWP_NOZORDER);

    linkAdvanced.GetWindowRect(&syncModeRect);
    
    currentY += syncModeRect.Height();
    
    return currentY;

}

void CSyncSettings::InitializeFont() { 

    memset(&lf, 0, sizeof(LOGFONT));
    HFONT hFont = (HFONT) GetStockObject(SYSTEM_FONT);
    GetObject(hFont, sizeof lf, &lf);
    lf.lfHeight += FONT_HEIGHT_OFFSET;
    VERIFY(defaultFontSize.CreateFontIndirect(&lf));
    
    // add the bold
    lf.lfWeight = FW_BOLD;
    VERIFY(fontBold.CreateFontIndirect(&lf));
}



void CSyncSettings::InitializeButtons() {


    //
    // ---- Email button ----
    // it's always the first, at the top of the dialog
    //
    CExtItem* mailItem = customList.getItem(SOURCE_MAIL);
    if (mailItem && doesFunambolAccountExist()) {
        
        CString label = mailItem->getName();
        checkMail = new CButton();
        CRect rect(40, 40, 100, 20); // default values
        checkMail->Create(label, BS_AUTOCHECKBOX | BS_NOTIFY | WS_VISIBLE | WS_TABSTOP, rect, GetActiveWindow(), IDC_SYNC_MAIL);       
        checkMail->SetFont(&defaultFontSize);
        if (mailItem->isEnabled()) { 
            checkMail->SetCheck(BST_CHECKED);
            butMail.EnableWindow(TRUE);
        }
        else {
            checkMail->SetCheck(BST_UNCHECKED);
            butMail.EnableWindow(FALSE);
        } 
        butMail.ShowWindow(TRUE);
    } else {
        butMail.EnableWindow(FALSE);
        butMail.ShowWindow(FALSE);

    }

    //
    // ---- Picture button ----
    // it's always the second, at the top of the dialog
    //
    CExtItem* pictureItem = customList.getItem(SOURCE_PICTURES);
    if (pictureItem) {
        
        CString label = pictureItem->getName();
        checkPicture = new CButton();
        CRect rect(40, 40, 100, 20); // default values
        checkPicture->Create(label, BS_AUTOCHECKBOX | BS_NOTIFY | WS_VISIBLE | WS_TABSTOP, 
                                    rect, GetActiveWindow(), IDC_SYNC_PICTURE);       
        checkPicture->SetFont(&defaultFontSize);
        if (pictureItem->isEnabled()) { 
            checkPicture->SetCheck(BST_CHECKED);
            butPictureOptions.EnableWindow(TRUE);
        }
        else {
            checkPicture->SetCheck(BST_UNCHECKED);
            butPictureOptions.EnableWindow(FALSE);
        } 
        butPictureOptions.ShowWindow(TRUE);
    } else {
        butPictureOptions.EnableWindow(FALSE);
        butPictureOptions.ShowWindow(FALSE);
    }

    //
    // ---- PIM buttons ----
    //
    bool pimOptionsFlag = false;
    for (int i = 0; i < customList.getCountItem(); i++) {
        int id = customList.indexToId(i);

        // skip email: managed separately
        if (!isPIMSource(id)) {
            continue;
        }

        CExtItem* currentItem = customList.getItem(id);
        
        CString label = currentItem->getName();
        CButton* cb = new CButton();
        CRect rect(40, 40, 100, 20); // default values
        cb->Create(label, BS_AUTOCHECKBOX | BS_NOTIFY | WS_VISIBLE | WS_TABSTOP, rect, GetActiveWindow(), CUSTOM_ID_BUTTON + i);       
        cb->SetFont(&defaultFontSize);
               
        // check/uncheck button
        if (currentItem->isEnabled()) { 
            cb->SetCheck(BST_CHECKED);
            pimOptionsFlag = true;
        }
        else {
            cb->SetCheck(BST_UNCHECKED);
        }
        
        // Add it to the button array (will copy it)
        pimButtonArray.Add(cb);
    }
    
    if (pimButtonArray.GetSize() == 0) {
        butPimOptions.ShowWindow(FALSE);
        butPimOptions.EnableWindow(FALSE);
    } else {
        butPimOptions.ShowWindow(TRUE);
        // Disable the "PIM Options" button if no pim sources is enabled & checked
        if (!pimOptionsFlag){
            butPimOptions.EnableWindow(FALSE);
        }
    }
   
}

void CSyncSettings::setScrollSize(int last_y) {
    
   //
    // Manage the vertical scrolling-bar
    //
    //GetWindowRect(&dlgRect);
    GetClientRect(&dlgRect);
    scrollbarPosition = 0;                   // reset bar position
    
    dlgMaxHeight = (last_y - dlgRect.top); 
    //dlgMaxHeight += 52; // offset value
    //dlgMaxHeight += (int)(dlgMaxHeight * 0.14);

    // Set all scrollbar parameters    
    si.cbSize = sizeof(SCROLLINFO);
    si.fMask = SIF_ALL;                 // SIF_ALL = SIF_RANGE | SIF_PAGE | SIF_POS | SIF_TRACKPOS;
    si.nMin = 0;
    si.nMax = dlgMaxHeight;
    si.nPage = dlgRect.Height();

    // the position is relative to window Rect that was created before
    int pos = ::GetScrollPos(GetSafeHwnd(), SB_VERT);   
    if (pos != 0) {
        scrollbarPosition = pos;
        si.nPos = scrollbarPosition;        
    } else {
        scrollbarPosition = 0;
        si.nPos = 0;
    }
    previousBottomOfWinRec = dlgRect.bottom;
    SetScrollInfo(SB_VERT, &si, TRUE);
    
    if ((pos = ::GetScrollPos(GetSafeHwnd(), SB_VERT)) != scrollbarPosition) {
        ScrollWindowEx(0, (scrollbarPosition-pos), NULL, &dlgRect, NULL, NULL, SW_SCROLLCHILDREN | SW_INVALIDATE);
        scrollbarPosition = pos;
    }
}

/**
 * Called when vertical scrollbar is touched.
 * Manages the window scrolling.
 */
void CSyncSettings::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)  {
    int nDelta;
    int nMaxPos = dlgMaxHeight;

    // Calculate the offset to scroll (nDelta)
    switch (nSBCode) {
        case SB_LINEDOWN:
            if (scrollbarPosition >= nMaxPos)
                return;
            nDelta = 2;
            break;

        case SB_LINEUP:
            if (scrollbarPosition <= 0)
                return;
            nDelta = -2;
            break;

        case SB_THUMBPOSITION:
            nDelta = (int)nPos - scrollbarPosition;
            break;

        case SB_PAGEDOWN:        
            if (scrollbarPosition >= nMaxPos)
                return;            
            nDelta = min(nMaxPos/10, nMaxPos - scrollbarPosition);
            break;                           
        case SB_PAGEUP:
            if (scrollbarPosition <= 0) {
                return;
            }
            nDelta = -min(nMaxPos/10, scrollbarPosition);
            break;
         case SB_THUMBTRACK:                
            if (LOBYTE(nSBCode) == SB_THUMBTRACK) {                           
                nDelta = nPos - scrollbarPosition;
            }
            break;

        default:
            // Don't handle other commands...
            return;
    }

    // Move the scrollbar
    scrollbarPosition += nDelta;
    SetScrollPos(SB_VERT, scrollbarPosition, TRUE);

    // Scroll the dialog window
    ScrollWindowEx(0, -nDelta, NULL, &dlgRect, NULL, NULL, SW_SCROLLCHILDREN | SW_INVALIDATE);
    
    CDialog::OnVScroll(nSBCode, nPos, pScrollBar);
}

void CSyncSettings::setScrollOnFocus(int objectIDC) {
    
    //
    // If the scrollbar doesn't exist nothing to do...
    //
    if (scrollBarExists == false) {
        return;
    }
    CRect dRect;
    CRect textRect;

    GetWindowRect(&dRect);
    GetDlgItem(objectIDC)->GetWindowRect(&textRect);

    int delta = textRect.bottom - (dRect.Height() / 2);

    int oldScrollbarPosition = scrollbarPosition;
    scrollbarPosition += delta;
    
    if (scrollbarPosition<0){
        scrollbarPosition = 0;
    }
    if (scrollbarPosition > dlgMaxHeight - dRect.Height()){
        scrollbarPosition = dlgMaxHeight - dRect.Height();
    }

    delta = scrollbarPosition - oldScrollbarPosition;
    
    // Move the scrollbar
    SetScrollPos(SB_VERT, scrollbarPosition, TRUE);

    // Scroll the dialog window
    ScrollWindowEx(0, -delta, NULL, &dRect, NULL, NULL, SW_SCROLLCHILDREN | SW_INVALIDATE);
}

void CSyncSettings::disableHttpPush(){
    ClientSettings* cs = getRegConfig();
    int pushType = 0;

    if (cs->getPush().size() > 0) {
        pushType = atoi(cs->getPush().c_str());
        if (pushType > 0) {
            startProgram(_T("startsync.exe"),_T("deregister-httponly-yes-sms-cp"));    
        }
    }
}