/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#if !defined(AFX_CUSTOMLIST_H__3BCE6295_516B_4715_A125_E1A03E26DF40__INCLUDED_)
#define AFX_CUSTOMLIST_H__3BCE6295_516B_4715_A125_E1A03E26DF40__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#define MAXITEMSTRING	256

#define ITEM_STATE_OK 1
#define ITEM_STATE_TO_SYNC 2

#define ANIM_TIMER_OFFSET 500     //for 4 images
#define ANIM_ICON_DELAY 250       //for 4 images
//#define ANIM_TIMER_OFFSET 250
//#define ANIM_ICON_DELAY 125

#define MAIN_LOGO_USE_ANIMATION     1                   /**< Set to 0 to disable animation */
#define MAIN_LOGO_ID                700                 /**< The unique ID for the main logo timer */
#define MAIN_LOGO_DELAY             100                 /**< The delay for the main logo timer [milliseconds] */
#define MAIN_LOGO_NAME              "mainLogo.gif"      /**< The main logo file name */
#define MAIN_LOGO_NUM_FRAMES        20                  /**< The main logo total # of frames */

                                                        // mainLogo is 60x60 pixel
#define MAIN_LOGO_UPPER_CORNER      70                  /**< The main logo distance of upper corner */
#define MAIN_LOGO_LOWER_CORNER      10                  /**< The main logo distance of lower corner */
#define MAIN_LOGO_LEFT_CORNER       80                  /**< The main logo distance of left corner */
#define MAIN_LOGO_RIGHT_CORNER      20                  /**< The main logo distance of right corner */


// unselected pane default bk color (white)
#define LIST_COLOR_BACKGROUND RGB(255,255,255)

//#define LIST_COLOR_SELECTED_ITEM RGB(220,220,255)

// selected pane default bk color
#define LIST_COLOR_SELECTED_ITEM RGB(153, 209, 252) // #99d1fc

// pane text color
#define LIST_COLOR_TEXT RGB(0,0,0)

#include <afxtempl.h>
#include "imaging.h"
#include "HwndFunctions.h"
#include "pim/ClientSettings.h"

/**
 * main window implemented as owner-draw CListCtrl (list) control where each item is a CExtItem,
 *
 */


/**
 * this is an item from the main list, it represents a sync source
 */
class CExtItem
{
protected:
    /**
     * item label (i.e. 'Mail', 'Contacts',..) it is visualized on the
     * source item
     */
    CString itemLabel;
    
    /**
    * The name of the item that appears in the text when the item are exchanged with
    * the server (i.e. updating contact 1/15, receiving mail 2/14...)
    */
    CString itemElementName;
    
    /**
     * sync source represented by this item (SOURCE_MAIL, SOURCE_BRIEFCASE, ..)
     */
    int id;
    
    /**
    * The name as it is written in the configuration. It is like "contact", "calendar"
    * and it is unique and it doesn't depends on the language
    */
    string sourceName;
    
    /**
     * item status displayed in the right of the source name
     * eg. : '(Last sync failed)', '(Not synchronized)',...
     */
    CString itemStatus;
    
    /**
    * It represents the state of the sync source
    * SYNCSOURCE_STATE_OK                 1
    * SYNCSOURCE_STATE_NOT_SYNCED         2
    * SYNCSOURCE_STATE_CANCELED           3
    */
    int syncSourceState;

    /**
    * It represents the current sync state (ITEM_STATE_OK, ITEM_STATE_TO_SYNC). It is 
    * set when the source is going to be synced (SelectSource method)
    */
    int itemState;

    /**
    * true if the item(source) is enabled
    */
    bool enabled;
    
    /**
    * Id of the icons that represent enable or disable. They are set when enable/disable
    * item is set (through enableItem method or setEnable method)
    */
    int iconIdEnabled;
    int iconIdDisabled;      

    /**
     * handle to the item's icon, changes if the item is disabled/enabled
     */
    HICON	hIcon;

    /**
     *  in which phase the anim icon is
     */
    int counterAnim;

    /**
     *  true if this source is currently syncing
     */
    bool bSyncing;
    
    /**
     * set the pointer of the sync source config to have access to the parameter
     * and be able to draw what needed
     */
    WindowsSyncSourceConfig* sConfig;   

public:
    CExtItem(int _id, CString sItemLabel, string sourceName, int _iconIdEnabled, 
             int _iconIdDisabled, CString sitemElementName, WindowsSyncSourceConfig* sc) {
        itemLabel    = sItemLabel;
        itemState   = ITEM_STATE_OK;
        enabled     = true;
        id          = _id;
        counterAnim = 0;
        bSyncing    = false;
        this->sourceName    = sourceName;
        iconIdEnabled       = _iconIdEnabled;
        iconIdDisabled      = _iconIdDisabled;
        this->hIcon         = NULL; 
        sConfig             = sc;
        syncSourceState     = SYNCSOURCE_STATE_OK;
        itemElementName     = sitemElementName;
    }

    /**
     * accessors
     */
    CString getName()           {return itemLabel; }
    void setName(CString s1)    {itemLabel = s1;   }
    
    CString getItemElementName()           {return itemElementName; }
    void setItemElementName(CString s1)    {itemElementName = s1;   }
    

    CString getText()           {return itemStatus;}
    void setText(CString s1)    {itemStatus = s1;  }

    int getItemState() const    {return itemState; }
    void setItemState(int val)  {itemState = val;  }

    bool isEnabled() const      {return enabled; }
    void setEnabled(bool val)   {enabled = val;  }

    int getId() const           {return id;}
    void setId(int val)         {id = val; }

    HICON getIcon() const       {return hIcon; }
    void setIcon(HICON val)     {hIcon = val;  }

    int getCounterAnim() const  {return counterAnim; }
    void setCounterAnim(int val){counterAnim = val;  }

    bool getIsItemSyncing() const   {return bSyncing; }
    void setIsItemSyncing(bool val) {bSyncing = val;  }

    int getIconIdDisabled() const   {return iconIdDisabled; }
    void setIconIdDisabled(int val) {iconIdDisabled = val;  }
    
    int getIconIdEnabled() const   {return iconIdEnabled; }
    void setIconIdEnabled(int val) {iconIdEnabled = val;  }

    int getSyncSourceState() const   {return syncSourceState; }
    void setSyncSourceState(int val) {syncSourceState = val;  }
    
    string getSourceName()           {return sourceName; }
    
    WindowsSyncSourceConfig* getSourceConfig() {return sConfig; }
    void setSourceConfig(WindowsSyncSourceConfig* s) {sConfig = s; }
  
};
//////////////////////////////////////////////////////////////////////////


/**
 * extension of  CListCtrl UI control,
 * contains an array of CExtItem items (pItemList) with info about each source
 */
class CCustomList : public CListCtrl {

private:
    /**
    * inits the list by inserting a column
    */
    void Init();

    CBrush brushHollow;
    int nItemHeight, nItemWidth; // item sizes

    /**
    *  true if the items is locked, so the user cannot select any item
    */
    bool locked;

    /**
    * array of CExtItem, holds info about each source item
    */
    CArray<CExtItem *> pItemList;

    /**
    * The main logo image.
    */
    IImage* mainLogo;

    /// Counter for the main logo: indicates the frame# to be showed.
    int mainLogoCounter;

    /// The rectangle on UI where main logo is shown.
    RECT mainLogoRect;

    /// The main logo real size of each frame (0.01mm units)
    SIZE mainLogoSize;

    /// true if main logo animation is running
    bool isMainLogoAnimated;

    /**
    * return the offset between the name of the source and the status
    */
    int GetTextOffSet();

    /**
    * Manages the animation of main logo: refresh the image based on
    * the internal counterAnim of CExtItem.
    */
    void animateMainLogo();
    void updateMainLogoImage();

    HICON h0, h1, h2, h3, h4, h5, h6, h7;
    
protected:

    void RepaintSelectedItems();

    // Overrides	
    virtual void PreSubclassWindow();	

    afx_msg void OnSetFocus(CWnd* pOldWnd);
    afx_msg void OnKillFocus(CWnd* pNewWnd);
    afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
    afx_msg LRESULT OnSetFont(WPARAM wParam, LPARAM);

    /**
    * @return : the default width and height for a list item
    */
    afx_msg void MeasureItem ( LPMEASUREITEMSTRUCT lpMeasureItemStruct );

    /**
    * called when a source is syncing and the sync arrows are spinning,
    * there is a timer for each source sync icon, the source id is (sourceId = nIDEvent - ANIM_TIMER_OFFSET;)
    */
    afx_msg void OnTimer( UINT_PTR nIDEvent );

    /**
    * when repainting by default only the items are redrawn,
    * so here we redraw also the empty space (if any) below the last item
    */
    afx_msg void OnPaint( );

    /**
    * do the same as for  OnPaint
    * @return : TRUE so we disable default processing of the message ( which would lead to excessive redrawing and flickering)
    */
    afx_msg BOOL OnEraseBkgnd(CDC* pDC );
    
    /**
    * size of the icon from the system
    */
    int icon_size;
    
    /**
    * The size of the screen. Used only for smartphone 176x220 (the smaller one)
    */
    int x_screen_size;

    void initializeIconHandle();

    DECLARE_MESSAGE_MAP()

public:   

    CCustomList();
    virtual ~CCustomList();
    
    virtual void DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct);
    BOOL GetItemRect(int nItem, LPRECT lpRect, UINT nCode ) const;
    
    CFont fontBold;
    
    int  GetCurSelItem() const;
    BOOL SetCurSelItem(int nIndex);

    // sets the default icon for a source
    void resetSourceIcon(int id);
    
    /**
    * Return the single element given the id of the item in the list of sources
    */
    CExtItem* getItem(int id);        

   /**
    *  adds a new item to the list, used only on application startup
    */
    // void addItem(int id, CString lpszItemName, HICON hIcon);
    void addItem (int _id, CString sItemLabel, string sourceName, int _iconIdEnabled, 
                                         int _iconIdDisabled, CString sitemElementName, 
                                         WindowsSyncSourceConfig* sc);
    
    /**
    * Set the text given the internal id of the source
    */
    void setText(int id, CString text);
    CString getText(int id);
    
    /**
    * Return the name of the item that is displayed on the main screen (Contact, Calendar, E-Mail)
    */
    CString getItemName(int id);
    
    /**
    * Return the name of the item that appears in the text when the item are exchanged with
    * the server (i.e. updating contact, calendar...)
    */
    CString getItemElementName(int id);
    

    /**
    * Return the name of the source. It is language independent (contact, calendar, mail)
    */
    string getSourceName(int id);
    
    /**
    * Need to refresh the config of all, at least to refresh the status of the
    * last time stamp. 
    * @todo: see if it is possible to use only the last
    */
    void refreshSourceConfig();
    
    
    /**
    * Used to set/get the uri to the proper source
    */
    void setSourceUri(int id, const char* uri);
    const char* getSourceUri(int id);
    
    void setUseSif(int id, bool useSif);
    bool getUseSif(int id);

    void setUseEncryption(int id, bool useSif);
    bool getUseEncryption(int id);

    /**
    * Set all the canceled states to the items that are in state_to_sync, 
    * because the user stop them. It could be more than a source if it is 
    * a multiple sync
    */
    void setCanceledState();

    void setIcon(int id, HICON hIcon);
    void setItemHeight(int height) { nItemHeight = height; }
    int getItemHeight(){return nItemHeight;}
    void setItemWidth(int width)  { nItemWidth = width; }
    
    /**
    * It sets the first icon arrow when the focus goes on it
    */
    void setFirstIconArrow(int sourceId);
    
    /**
    * Return the id of the first enabled sync source. It is used by the
    * synchronization process to understand which source must be marked 
    * and selected. In this case at least one source must be enabled since 
    * the button sync all is disabled.
    * It is used also to check if enable/disable the "Sync all" button must 
    * be activated
    * return the right id if found, -1 otherwise
    */
    int getFirstEnabledSource();

    void refreshSourcePanes();
    /**
    * @param id - the identifier set into the item
    */
    void refreshSourcePane(int id);

    /**
    * displays the sync arrows in the <id> source, and starts spinning it
    */
    void startAnim(int id);

    /**
    *  for the <id> source stops the animation, deletes the timer, restore the default icon
    */
    void stopAnim(int id);

    int getState(int id);
    void setState(int id, int state);
    void setStateToAll(int state, bool onlyEnabled = false);
    void enableItem(int id, bool enable);
    bool isEnabledItem(int id);
    
    int getCountItem();
    /**
    * Set the source state of the item with the id
    * SYNCSOURCE_STATE_OK                 1
    * SYNCSOURCE_STATE_NOT_SYNCED         2
    * SYNCSOURCE_STATE_CANCELED           3
    */
    void setSyncSourceState(int id, int sourceState);
    

    /**
    * repaints the <id> source pane
    */
    void InvalidatePane(int id);

    /**
    * conversion between the source id and the index in the list  pItemList
    */
    int indexToId(int index);
    int idToIndex(int id);

    bool doesIdExist(int id);

    void setIsSyncing(int id, bool value);
    bool getIsSyncing(int id);

    /**
    * resets the panes to their default icons
    */
    void resetSourceIcons();

    void lockList(){locked = true;}
    void unlockList() {locked = false;}
    bool getIsLocked() const {return locked;}

    void initMainLogo();
    void calculateMainLogoPosition();
    void startMainLogo();
    void stopMainLogo();
    

};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif