/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#pragma once

#include "ui.h"
#include "afxcmn.h"
#include "afxwin.h"
#include "CustomButton.h"
#include "CustomList.h"

/**
 * Sync Settings dialog
 */
class CSyncSettings : public CDialog
{
    DECLARE_DYNAMIC(CSyncSettings)

public:
    CSyncSettings(CWnd* pParent = NULL);   // standard constructor
    virtual ~CSyncSettings();

// Dialog Data
    enum { IDD = IDD_UI_SYNC };

protected:

    int paddingVal;
    int moltiplicator; // value that is 2 for vga devices
    double leftBorderPercentage; // the percentage of the screen to leave from left
                                 // it is different for sph and ppc

    int oldOrientation; // to flag the oldorientation type
    int newOrientation; // to get the new orientation type
    bool scrollBarExists;
    bool changed; // signal in the OnSize if the screen is really changed or only the keyboard comes up
    
    bool loadedInitResources; // it is true after the OnInitDialog call.


#if defined(WIN32_PLATFORM_PSPC) 
    CRect labelStaticHeader, clientLabelStaticHeader;       
    CRect lineSeparator, clientLineSeparator;       
#endif
    CRect labelRecOrig, clientLabelRecOrig;       
    
    int initScroll;             // init the position of the scrollbar after resize
    int previousBottomOfWinRec; // store the first value of the winrect

    // For scrollbar control
    int dlgCurrentHeight;
    int dlgMaxHeight;
    int scrollbarPosition;
    CRect dlgRect;
    SCROLLINFO si; // info of the scrollbar

    CCustomButton linkAdvanced;   
    
    SHMENUBARINFO dlgCommandBar;

    // for font to be used
    CFont fontBold;
    CFont defaultFontSize;
    LOGFONT lf;
    LOGFONT defaultFont;
    
    // create the menu
    bool createMenu();

    // True if the "Email" source is used
    bool mailSourceExists;
    CButton *checkMail;

    // button for pictures
    bool pictureSourceExists;
    CButton *checkPicture;


    // Array of PIM buttons
    CArray<CButton*> pimButtonArray;


    // reference to the list in the uiDlg;
    CCustomList& customList;
    
    void InitializeFont();
    void InitializeButtons();

    BOOL refreshPIMOptionsButton();

    void setScrollSize(int last_y);
    void setScrollOnFocus(int objectIDC);
    afx_msg void OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);

    CCustomButton butMail;
    CCustomButton butPimOptions;
    CCustomButton butPictureOptions;

    int sourceOnFocus;
    int advanced;

    // enables/disables the right soft key 'Details' button
    //void EnableAdvancedButton(bool bEnable);

    /**
     * loads strings onto the dialog from the stringtable
     */
    void LoadStringResources();

    /**
     * processes key events (arrow keys), overriding the default behaviour
     * @return:true if the message was processed
     */
    bool processKeyEvents(WPARAM key);

    /**
     * overrides default handling of some window events
     * @see : processKeyEvents
     */
    BOOL PreTranslateMessage(MSG* pMsg);

    /**
     * called on dialog init
     */
    virtual BOOL OnInitDialog();

    virtual void OnInitMenuPopup(CMenu *pPopupMenu, UINT nIndex,BOOL bSysMenu);

    void OnAdvanced();

    HBRUSH OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor);

    virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support


#if defined(_DEVICE_RESOLUTION_AWARE)
    afx_msg void OnSize(UINT /*nType*/, int /*cx*/, int /*cy*/);
#endif

    afx_msg void OnPaint();
    // true if it has to be painted. Only one time when the OnSize change
    
    /**
    * The method is used by OnPaint and OnSize. The main issue is that onPaint has to
    * draw the line dashed that cannot be created properly by the onSize method.
    * So it must be done two times in both methods.
    */
    int drawComponents(CRect& clientRect, CRect& windowrect, int offset);
    
    DECLARE_MESSAGE_MAP()


public:
    /**
     * opens advanced settings window for <source>  sync source
     */
    afx_msg void showAdvanced();


    /**
     * when focus is set on a source checkbox
     */
    afx_msg void OnBnSetfocusPIM();
    afx_msg void OnBnSetfocusSyncMail();
    afx_msg void OnBnSetfocusSyncPicture();

    afx_msg void OnSetFocus( CWnd* pOldWnd );

    afx_msg void OnBnClickedPIM();
    afx_msg void OnBnClickedSyncMail();
    afx_msg void OnBnClickedSyncPicture();


    afx_msg void OnOK();

    /**
     * when an 'Advanced..' link for a source was activated
     * takes the user in a window with the advanced settings for that source
     */
    afx_msg void OnClickedSyncButtonMail();
    afx_msg void OnClickedSyncButtonPimOptions();
    afx_msg void OnClickedSyncButtonPictureOptions();



    /**
     *  'Advanced Settings' link was activated
     */
    afx_msg void OnBnClickedSyncLinkAdvanced();
    afx_msg void OnBnSetfocusSyncLinkAdvanced();

    afx_msg void OnEmailOptionsClicked();
    afx_msg void OnPIMOptionsClicked();
    afx_msg void OnSyncMethodClicked();
    afx_msg void OnPicturesOptionsClicked();

    afx_msg void disableHttpPush();
    int E_disableHttpPush;
    int E_disableHttpPushByKeyDown;
};
