/*
* Funambol is a mobile platform developed by Funambol, Inc. 
* Copyright (C) 2003 - 2007 Funambol, Inc.
* 
* This program is free software; you can redistribute it and/or modify it under
* the terms of the GNU Affero General Public License version 3 as published by
* the Free Software Foundation with the addition of the following permission
* added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
* WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
* WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
* 
* This program is distributed in the hope that it will be useful, but WITHOUT
* ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
* FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
* details.
* 
* You should have received a copy of the GNU Affero General Public License
* along with this program; if not, see http://www.gnu.org/licenses or write to
* the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
* MA 02110-1301 USA.
* 
* You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
* 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
* 
* The interactive user interfaces in modified source and object code versions
* of this program must display Appropriate Legal Notices, as required under
* Section 5 of the GNU Affero General Public License version 3.
* 
* In accordance with Section 7(b) of the GNU Affero General Public License
* version 3, these Appropriate Legal Notices must retain the display of the
* "Powered by Funambol" logo. If the display of the logo is not reasonably
* feasible for technical reasons, the Appropriate Legal Notices must display
* the words "Powered by Funambol".
*/

// uiDlg.h : header file
//


#pragma once
#include "afxwin.h"
#include "CustomButton.h"
#include "CustomList.h"

#if defined(WIN32_PLATFORM_PSPC)
#include "aygshell.h" // shmenubarinfo
#endif

using namespace std;

class CuiDlg : public CDialog {

    /**
    * what source will be synced first, used for displaying the 'Connecting' message
    */
    int firstSourceId;

    /**
    * if a settings child window is opened
    */
    bool inSettings; 

    /**
    * if a sync is canceled manually or by an error
    */
    bool syncCanceled;

    /**
    * if the sync is started set the variable
    */
    bool syncStarted;
    
    /**
    * Needed to understand if the onInitDialog is called. When lauched
    * it is set to true so in OnActivate we can manage it
    */    
    bool onInitDialogOpened;
    
    /**
    * Used to store the current log level. 
    */
    LogLevel originalLogLevel;
    void setOriginalLogLevel(LogLevel val)  {originalLogLevel = val;  }
    LogLevel getOriginalLogLevel()               {return originalLogLevel; }
    
    /**
    * is the handle to the CommunicationSettings dialog. It is used to have a modal popup 
    * for the message errors
    */
    HWND commSettingsHwnd;

protected:
    
    static int pixelX;
    static int pixelY;

    SHMENUBARINFO dlgCommandBar;
    void CreateMenuBar(const int id);
   
    // bitmask, to see if for a source was received a SyncSourceEnded event
    unsigned long sourceEnds;

    // bitmask, to see if for a source was received a SyncSourceBegin event
    unsigned long sourceBegins;

    int totalItems;
    int lastItem;
    
    int oldOrientation;
    /**
    * use to identify the current source duding the sync process
    */
    int currentSource;
    
    /**
    * use to identify the incremental number of current item
    */
    int currentItem;
    
    bool resetCurrentItem;   
        
    void setDialogOpened(bool val)  {onInitDialogOpened = val;  }
    bool isDialogOpened()           {return onInitDialogOpened; }
    
    void setSyncStarted(bool val)  {syncStarted = val;  }
    bool isSyncStarted()           {return syncStarted; }

    void setSyncCanceled(bool val) {syncCanceled = val;  }
    bool isSyncCanceled()          {return syncCanceled; }

    void setInSettings(bool val) {inSettings = val;  }
    bool isInSettings()          {return inSettings; }
    
    void setFirstSourceId(int val) {firstSourceId = val;  }
    int  getFirstSourceId()        {return firstSourceId; }
    
    /**
    * Set the log level to debug and restore to the previous one
    */
    void setDebugLogLevel();
    void restoreLogLevel();

    HICON m_hIcon;
    CCustomList lstSources;

    // checks the sources to find which source will be synced first, sets firstSourceId variable
    void setFirstSourceToSync();

    /**
    * inits the sync controls in the main window. It all the list of
    * all sources in the array list to be painted
    */
    void initSyncControls();


    /* if there are sources which were recovered with 'refresh-local' or
    'refresh-server' then we set them to 'two-way' */
    //void refreshRecoveredSources();

    void lockPanes();
    void unlockPanes();
    // void refreshCanceledState();

    bool isSyncEnabled();

    virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support
    virtual BOOL OnInitDialog();
    virtual BOOL PreTranslateMessage(MSG* pMsg);

    afx_msg void OnSize(UINT /*nType*/, int /*cx*/, int /*cy*/);

    afx_msg void OnInitMenuPopup(CMenu *pPopupMenu, UINT nIndex, BOOL bSysMenu);


    // it is used to decide if reset the counter of the total item.
    bool resetTheItem;
    // it is used to populate the message without the default message
    CString messageToWrite;

    bool showEmailSlowSyncPopup;

    void CalculateSize();

    DECLARE_MESSAGE_MAP()

public:
    
    
    // Construction
    CuiDlg(CWnd* pParent = NULL);	// standard constructor

    // Dialog Data
    enum { IDD = IDD_UI_DIALOG };
    
    // startsync exit code as lParam
    afx_msg LRESULT OnStartsyncEnded(WPARAM , LPARAM);

    afx_msg LRESULT OnStartingSync(WPARAM , LPARAM);
    afx_msg LRESULT OnStartingSource(WPARAM , LPARAM);
    void OnMsgItemSynced(WPARAM , LPARAM, bool);
    afx_msg LRESULT OnMsgTotalItems(WPARAM , LPARAM);
    afx_msg LRESULT OnMsgItemSyncedFromServer(WPARAM , LPARAM);
    afx_msg LRESULT OnMsgItemSyncedFromClient(WPARAM , LPARAM);
    afx_msg LRESULT OnMsgSourceState(WPARAM , LPARAM);
    afx_msg LRESULT checkEmailSlowSync(WPARAM , LPARAM);
    afx_msg LRESULT OnMsgEndingSource(WPARAM , LPARAM);
    afx_msg LRESULT OnMsgConnecting(WPARAM , LPARAM);
    afx_msg LRESULT OnMsgEndingSync(WPARAM , LPARAM);
    afx_msg LRESULT OnMsgPopup(WPARAM wParam, LPARAM lParam);
    afx_msg LRESULT OnOKMsg(WPARAM wParam, LPARAM lParam);

    afx_msg LRESULT OnMsgTotalItemsNew(WPARAM wParam, LPARAM lParam);
    afx_msg LRESULT OnMsgTotalItemsMod(WPARAM wParam, LPARAM lParam);
    afx_msg LRESULT OnMsgTotalItemsDel(WPARAM wParam, LPARAM lParam);

    afx_msg void OnMenuAbout();
    afx_msg void OnMenuStop();
    afx_msg void OnMenuComm();
    afx_msg void OnMenuSyncSettings();
    //afx_msg void OnMenuLoglevel();
    afx_msg void OnSyncAll();
    afx_msg void OnMenuExit();
    afx_msg void OnMenuRecover();
    afx_msg void OnOK();
    afx_msg void OnCancel();
    afx_msg void OnClose();
    afx_msg HBRUSH OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor );
    afx_msg void OnItemchangedList1(NMHDR* pNMHDR, LRESULT* pResult);
    // afx_msg void OnContextMenu(CWnd* /*pWnd*/, CPoint /*point*/);

    afx_msg void OnUpdateProcedure();

    afx_msg void OnGotoApp();

    afx_msg void OnSyncEnableSource();

    afx_msg void OnActivate(UINT nState, CWnd* pWndOther, BOOL bMinimized);

    void setFirstSourceToSync(int id) {firstSourceId = id;}

    CCustomList& getSourceListCtrl() {return lstSources; }

    /**
    * source with index <index> has been activated and if the
    * question to enable the source is to be asked
    * @param index - index of the source
    * @param question - if the source is disabled it says
    *                   if the popup to enable must appear
    */

    void SelectSource(int index, bool question = true);

    // starts the sync
    void StartSync(CString param);
    void CancelSync();

    // afx_msg void OnToolsDeviceinfo();
    void EnableSyncButton(bool bEnable);

    void setShowEmailSlowSyncPopup(bool val) { showEmailSlowSyncPopup = val; }
    bool getShowEmailSlowSyncPopup() { return showEmailSlowSyncPopup; }
    
    /**
    * reset common variables
    */ 
    void resetSyncVariables();

    static bool isVGA();
    static bool isQVGASquare();
    
    /**
    * To set the hwnd to be used for the error messages
    */
    void setCommSettingsHwnd(HWND h) {commSettingsHwnd = h; }
    HWND getCommSettingsHwnd() { return commSettingsHwnd; }


};
