/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#ifndef PICTURESSOURCESYNC_H_
#define PICTURESSOURCESYNC_H_


#include "base/fscapi.h"
#include "spds/constants.h"
#include "spds/SyncItem.h"
#include "spds/SyncMap.h"
#include "spds/SyncStatus.h"
#include "base/util/ItemContainer.h"
#include "spds/FileData.h"
#include "client/CacheSyncSource.h"
#include "client/MediaSyncSource.h"

BEGIN_NAMESPACE

/**
 * This class extends the CacheSyncSource abstract class, implementing a plain
 * file datastore. All the files in a folder are synchronized with the server.
 * Depending on the MIME type defined for the source (see
 * SyncSourceConfig::getType()), this class can work in two ways:
 *
 * - if the type is "application/vnd.omads-file+xml", the files are wrapped into
 *   the OMA File Object representation, to preserve the file name and attributes.
 * - otherwise, the file is sent as it to the server.
 *
 * for incoming items, the format of the file is detected by the content.
 *
 */
class PicturesSyncSource : public MediaSyncSource 
{
public:
    PicturesSyncSource(const WCHAR* name, 
                       AbstractSyncSourceConfig* sc, 
                       const StringBuffer& aDir);
    
    ~PicturesSyncSource() {};

    
protected:
    
    /**
     * Filter out unwanted folders, and files that are not images.
     */
    bool filterOutgoingItem(const StringBuffer& fullName, struct stat& st);
    
private:
        
};

END_NAMESPACE

#endif /*PICTURESSOURCESYNC_H_*/


