/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2003 - 2007 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

// handleservice.cpp - module used to deregister push setting by setup.dll
//
//

// Windows Header Files:
#include <windows.h>
#include "notify/s4n_service.h"
#include "Tlhelp32.h"
#include "http/GPRSConnection.h"
#include "pim/difftime.h"
#include <Notify.h>
#include "pim/SettingFunctions.h"
#include "pim/account.h"
#include "base/startcmd.h"
#include "base/Log.h"
#include "base/util/utils.h"
#include "pim/ClientSettings.h"
#include "customization.h"
#include "pim/mail/MailUtils.h"

#if _WIN32_WCE > 0x500
    #include <snapi.h>
#endif

#include <mapidefs.h>
#include <mapicode.h>
#include <mapitags.h>
#include <mapix.h>
#include <mapiutil.h>
#include <cemapi.h>

#include "Cfgmgrapi.h" //-- for DMProcessConfigXML


#if _WIN32_WCE > 0x500
#include "Regext.h"
#endif

#define ACCOUNT L"<wap-provisioningdoc> \
    <characteristic type=\"EMAIL2\"> \
    <characteristic type=\"" ACCOUNT_GUID L"\"> \
    <parm name=\"SERVICENAME\" value=\"" ACCOUNT_NAME_TO_CREATE L"\"/> \
    <parm name=\"SERVICETYPE\" value=\"" TEXT(SERVICE_TYPE)     L"\"/> \
    <parm name=\"INSERVER\"    value=\"" INCOMING_MAIL_SERVER   L"\"/> \
    <parm name=\"OUTSERVER\"   value=\"" OUTGOING_MAIL_SERVER   L"\"/> \
    <parm name=\"AUTHNAME\"    value=\"username\"/> \
    <parm name=\"AUTHSECRET\"  value=\"password\"/> \
    <parm name=\"DOMAIN\"      value=\"" DOMAIN_NAME L"\"/> \
    <parm name=\"REPLYADDR\"   value=\"default@email\"/> \
    <parm name=\"NAME\"        value=\"\"/> \
    <parm name=\"LINGER\"      value=\"0\"/> \
    </characteristic> \
    </characteristic> \
    </wap-provisioningdoc>"

const wchar_t* acc  = TEXT("<wap-provisioningdoc><characteristic type=\"EMAIL2\"><nocharacteristic type=\"%s\"/></characteristic></wap-provisioningdoc>");

void createAccount() {
    LPWSTR out = NULL;
    wchar_t* ret = NULL;
    ret = getFunambolGUID();
    HRESULT hr;
    if (!ret) {
        hr = DMProcessConfigXML(ACCOUNT, CFGFLAG_PROCESS, &out);
    }

}

void deleteAccount(wchar_t* guid) {
    LPWSTR out = NULL;
    wchar_t account[512];

    memset(account, 0, 512*sizeof(wchar_t));
    wsprintf(account, acc, guid);
    HRESULT ret = DMProcessConfigXML(account, CFGFLAG_PROCESS, &out);

    if (ret != S_OK) {
        // it should never appear
        wchar_t* gg = getFunambolGUID();        
        memset(account, 0, 512*sizeof(wchar_t));
        wsprintf(account, acc, gg);
        ret = DMProcessConfigXML(account, CFGFLAG_PROCESS, &out);            
    }

}

enum {
    eePR_ENTRYID,
    eePR_DISPLAY_NAME,
    eeNUM_COLS};

// These tags represent the message information we would like to pick up
static SizedSPropTagArray(eeNUM_COLS, Columns) =
{
    eeNUM_COLS,
    PR_ENTRYID,
    PR_DISPLAY_NAME
};


wchar_t* setMailAccountSignature(const char* signature = NULL) {

    HRESULT           hr;
    IMAPISession *    pSession    =  NULL;
    IMAPITable*       pTable      =  NULL;
    SRowSet*          psrs        =  NULL;
    IMsgStore*        pStore      =  NULL;           
    BOOL accountFound             = FALSE;         
    SPropValue*       rgprops     =  NULL;       
    wchar_t* ret                  = NULL;
    bool msgset = false;

    // First log on to the store.
    hr = MAPILogonEx (NULL, NULL, NULL, NULL, &pSession);
    EXIT_ON_FAILED (hr);

    // Get the message stores table
    hr = pSession->GetMsgStoresTable (MAPI_UNICODE, &pTable);
    EXIT_ON_FAILED (hr);

    hr = pTable->SetColumns((LPSPropTagArray)&Columns, 0);
    EXIT_ON_FAILED(hr);


    while (!msgset){
        // Get a row
        hr = pTable->QueryRows (1, 0, &psrs);
        EXIT_ON_FAILED (hr);

        // Did we hit the end of the table?
        if (psrs->cRows != 1) {
          break;
        }
        //
        // the display name is the name of the account.
        //
        wchar_t* displayName = psrs->aRow[0].lpProps[eePR_DISPLAY_NAME].Value.lpszW;

        //
        // The name of the MessageStore that we use for the own Transport
        //
        if (wcscmp(displayName, ACCOUNT_NAME) == 0) {
            accountFound = TRUE;            

            // Open this message store
            hr = pSession->OpenMsgStore (NULL,
                           psrs->aRow[0].lpProps[0].Value.bin.cb,
                          (ENTRYID *) psrs->aRow[0].lpProps[0].Value.
                           bin.lpb, NULL, 0, &pStore);
            EXIT_ON_FAILED (hr);
                         
            SPropValue rgprops[3] = {0};
            
            wstring sign = TEXT(FUNAMBOL_SIGNATURE);
            if (signature) {
                wchar_t* t = toWideChar(signature);
                sign = t;
                delete [] t;
            }
            rgprops[0].ulPropTag = PR_CE_SIGNATURE;
            rgprops[0].Value.lpszW = (wchar_t*)sign.c_str();

            rgprops[1].ulPropTag = PR_CE_USE_SIGNATURE;
            rgprops[1].Value.b = true;
            
            rgprops[2].ulPropTag = PR_CE_USE_SIGNATURE_REPLY_FORWARD;
            rgprops[2].Value.b = true;
            
            hr = pStore->SetProps(3, rgprops, NULL);                       
            EXIT_ON_FAILED (hr);           
        }

        // Clean up
        MAPIFreeBuffer (rgprops);
        FreeProws   (psrs);

        rgprops   =  NULL;
        psrs    =  NULL;

        RELEASE_OBJ (pStore);
    }
    if (accountFound == FALSE) {
        LOG.error("No " PROVIDER " account found");
    }
FuncExit:

    MAPIFreeBuffer (rgprops);
    FreeProws   (psrs);

    RELEASE_OBJ (pStore);
    RELEASE_OBJ (pTable);
    RELEASE_OBJ (pSession);
        
    return ret;
}

//-------------------------------------------------------------------------- MAIN

int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
   
    int ret = 0;
    
    if(wcscmp(lpCmdLine,_T("createAccount")) == 0) {
        createAccount();
    } else if(wcscmp(lpCmdLine,_T("deleteAccount")) == 0) {
        wchar_t* xx = getFunambolGUID();
        deleteAccount(xx);
    } else if(wcscmp(lpCmdLine,_T("setSignature")) == 0) {                
        setMailAccountSignature();
    }

	return ret;
}

